Package: EBImage
Version: 4.54.0
Title: Image processing and analysis toolbox for R
Encoding: UTF-8
Author: Andrzej Oleś, Gregoire Pau, Mike Smith, Oleg Sklyar, Wolfgang
        Huber, with contributions from Joseph Barry and Philip A.
        Marais
Maintainer: Andrzej Oleś <andrzej.oles@gmail.com>
Depends: methods
Imports: BiocGenerics (>= 0.7.1), graphics, grDevices, stats, abind,
        tiff, jpeg, png, locfit, fftwtools (>= 0.9-7), utils,
        htmltools, htmlwidgets, RCurl
Suggests: BiocStyle, digest, knitr, rmarkdown, shiny
Description: EBImage provides general purpose functionality for image
        processing and analysis. In the context of (high-throughput)
        microscopy-based cellular assays, EBImage offers tools to
        segment cells and extract quantitative cellular descriptors.
        This allows the automation of such tasks using the R
        programming language and facilitates the use of other tools in
        the R environment for signal processing, statistical modeling,
        machine learning and visualization with image data.
License: LGPL
LazyLoad: true
biocViews: Visualization
VignetteBuilder: knitr
URL: https://github.com/aoles/EBImage
BugReports: https://github.com/aoles/EBImage/issues
Config/pak/sysreqs: cmake libfftw3-dev make libjpeg-dev libpng-dev
        libtiff-dev libuv1-dev
Repository: https://bioc-release.r-universe.dev
Date/Publication: 2026-04-28 12:31:40 UTC
RemoteUrl: https://github.com/bioc/EBImage
RemoteRef: RELEASE_3_23
RemoteSha: 183e0037ac81df55139b8e9a569b95719ea96c9b
NeedsCompilation: yes
Packaged: 2026-04-29 13:00:36 UTC; root
Built: R 4.6.0; x86_64-apple-darwin20; 2026-04-29 13:05:39 UTC; unix

useDynLib("EBImage", .registration = TRUE, .fixes = "C_")

importFrom("BiocGenerics", "combine", "normalize")
importFrom("methods", "as", "callGeneric", "callNextMethod", "extends", "is", "new", "Ops", "slot", "show", "validObject")
importFrom("graphics", "abline", "clip", "hist", "hist.default", "image", "image.default", "matplot", "par", "plot", "rasterImage")
importFrom("grDevices", "as.raster", "col2rgb", "gray", "heat.colors", "rgb")
importFrom("stats", "dnorm", "mad", "median", "predict", "quantile", "sd")
importFrom("abind", "abind", "adrop", "asub")
importFrom("locfit", "locfit", "lp")
importFrom("tiff", "readTIFF", "writeTIFF")
importFrom("jpeg", "readJPEG", "writeJPEG")
importFrom("png", "readPNG", "writePNG")
importFrom("fftwtools", "fftw2d")
importFrom("utils", "browseURL")
importFrom("RCurl", "base64Encode")
importFrom("htmltools", "htmlDependency")
importFrom("htmlwidgets", "createWidget", "sizingPolicy")

## classes
exportClasses(Image)

## image class, accessors
export(Image, is.Image, as.Image)
export(Grayscale, Color)
exportMethods(Ops, "[")
export(colorMode, "colorMode<-", imageData, "imageData<-")
export(getFrame, getFrames, numberOfFrames)
S3method(as.array, Image)
S3method(as.raster, Image)
S3method(print, Image)
S3method(as.Image, Image)
S3method(as.Image, default)

## image IO, display
exportMethods(show, image)
export(readImage, writeImage)

## image display
export(display)
S3method(plot, Image)

# shiny bindings
export(displayOutput, renderDisplay)

## spatial transform
export(resize, rotate)
export(flip, flop)
export(translate)
export(affine)
export(transpose)

## image segmentation, objects manipulation
export(thresh, bwlabel, otsu)
export(watershed, propagate)
export(ocontour)
export(paintObjects, rmObjects, reenumerate, colorLabels)

## statistics
S3method(median, Image)
S3method(quantile, Image)
exportMethods(hist)

## image enhancement, filtering
export(filter2)
export(gblur)
export(medianFilter)
export(equalize)
export(clahe)

## morphological operations
export(makeBrush)
export(erode, dilate, opening, closing, whiteTopHat, blackTopHat, selfComplementaryTopHat)
export(distmap)
export(floodFill, fillHull)

## colorspace
export(rgbImage, channel, toRGB, colormap)

## image stacking, tiling
export(stackObjects)
export(tile, untile)

## drawables
export(drawCircle)

## features extraction
export(computeFeatures)
export(computeFeatures.basic, computeFeatures.haralick, computeFeatures.moment, computeFeatures.shape)
export(standardExpandRef, gblob)
export(localCurvature)

## methods extending S4 generics from the BiocGenerics package
exportMethods(normalize, combine)

## enable abind for Image objects
exportMethods(abind)

## legacy code because of broken R 3.1.2
importFrom("methods", "Math2")
exportMethods(Math2)
exportMethods(log)

## Installation

The package is distributed as part of the [Bioconductor](http://bioconductor.org/packages/EBImage) project.
To install this package, start R and enter:

```
install.packages("BiocManager")
BiocManager::install("EBImage")
```

## Documentation

For an overview of the package functionality, see the [package vignette](http://bioconductor.org/packages/devel/bioc/vignettes/EBImage/inst/doc/EBImage-introduction.html).

Once you install the package, you can access individual man pages by a call to, e.g., `?Image`.

## FAQ

See [my answers on Stack Overflow](http://stackoverflow.com/search?q=user%3A2792099+%09is%3Aanswer+EBimage).

## Bug reports and feature requests

Bug reports, feature requests, or any other issues with the package should be reported at https://github.com/aoles/EBImage/issues.

## This scripts makes two figures for the explanation
##   of the distance map transformation that we use in talks
## WH 5.4.2007
##
library("EBImage")
library("RColorBrewer")
library("lattice")

h = matrix(0, nrow=500, ncol=500)

x=140; y=140; r=80
h[ ((col(h)-x)^2+(row(h)-y)^2) < r^2 ] = 1

x=250; y=290; r=150
h[ ((col(h)-x)^2+(row(h)-y)^2) < r^2 ] = 1

dm = distMap(Image(h, dim=dim(h)))

png("tmp/distmap1.png", width=500, heigth=500)
par(mai=rep(0,4))
image(h, col=c("white", "black"))
dev.off()

png("tmp/distmap2.png", width=512, heigth=660)
print(levelplot(matrix(dm, nrow=nrow(h), ncol=ncol(h)),
   col.regions=colorRampPalette(c("white", brewer.pal(9, "YlOrRd")))(256)))
dev.off()

## cat readWriteTest.R | R --vanilla &>readWriteTest.Rout.save

library("EBImage")

## Fetch test images frome the on-line respository
repositoryURL  = "http://www-huber.embl.de/EBImage/ExampleImages"

testImages = c(	
  "lena.jpg",		# JPEG Greyscale
  "lena-color.jpg",	# JPEG Color
  "dices-grey.png",	# PNG GA
  "dices.png",		# PNG RGBA
  "photo_16b.png",	# PNG RGB 16bps
  "dices-grey.tif",	# TIFF GA 8bps
  "photo_8b.tif",	# TIFF RGB 8bps
  "lena-original.tif",	# TIFF RGB 8bps
  "dices.tif",		# TIFF RGBA 8bps
  "nuc.tif",		# TIFF G 16bps
  "photo_16b.tif"	# TIFF RGB 8bps multiple pages
)

names(testImages) = rep(repositoryURL, length(testImages))

for (i in seq_along(testImages))
  if (!file.exists(testImages[i]))
    system(paste("wget ", names(testImages)[i], "/", testImages[i], sep=""))

## Append package test images

packageTestImages = list.files(system.file("images", package="EBImage"), full.names=TRUE)
names(packageTestImages) =  list.files(system.file("images", package="EBImage"))
names(testImages) = testImages
testImages = c(testImages, packageTestImages)

## Actual tests

tempdir = file.path(tempdir(),"copies")
dir.create(tempdir)

for (i in seq_along(testImages)) {
  original = readImage(testImages[i])
  filename = names(testImages)[i]
  tempfile = file.path(tempdir, filename)
  writeImage(original, tempfile)
  copy = readImage(tempfile)
  cat(filename, rep(" ", 40-nchar(filename)), if(identical(original, copy)) "PASS" else "FAIL", "\n", sep="")	
}

unlink(tempdir, recursive=TRUE)

## ----setup, echo=FALSE--------------------------------------------------------
library(knitr)
.dpi = 100
set.seed(0)
opts_chunk$set(comment=NA, fig.align="center", dpi=.dpi)
knit_hooks$set(crop=NULL)
.output = output()
switch(.output,
        html = opts_chunk$set(fig.retina=1),
        latex = opts_chunk$set(out.width=".5\\textwidth")
)

.dev = switch(.output, html="svg", latex="pdf")
options(EBImage.display = "raster")

## ----installation, eval=FALSE-------------------------------------------------
# install.packages("BiocManager")
# BiocManager::install("EBImage")

## ----library, message=FALSE---------------------------------------------------
library("EBImage")

## ----readImage----------------------------------------------------------------
f = system.file("images", "sample.png", package="EBImage")
img = readImage(f)

## ----display------------------------------------------------------------------
display(img, method="browser")

## ----display-raster, fig.width=dim(img)[1L]/.dpi, fig.height=dim(img)[2L]/.dpi, dpi=.dpi/2----
display(img, method="raster")
text(x = 20, y = 20, label = "Parrots", adj = c(0,1), col = "orange", cex = 2)

## ----dev-print, eval=FALSE----------------------------------------------------
# filename = "parrots.jpg"
# dev.print(jpeg, filename = filename , width = dim(img)[1], height = dim(img)[2])

## ----dev-print-pre, echo=FALSE, fig.show='hide', fig.width=dim(img)[1L]/.dpi, fig.height=dim(img)[2L]/.dpi, dpi=.dpi/2----
display(img, method="raster")
text(x = 20, y = 20, label = "Parrots", adj = c(0,1), col = "orange", cex = 2)
filename = "parrots.jpg"
dev.print(jpeg, filename = filename , width = dim(img)[1], height = dim(img)[2])

## ----filesize-----------------------------------------------------------------
file.info(filename)$size

## ----dev-print3, echo=FALSE, sfig.width=dim(img)[1L]/.dpi, fig.height=dim(img)[2L]/.dpi, dpi=.dpi/2----
invisible(file.remove(filename))

## ----readImageColor-pre, echo=FALSE-------------------------------------------
imgcol = readImage(system.file("images", "sample-color.png", package="EBImage"))

## ----readImageColor, eval=FALSE-----------------------------------------------
# imgcol = readImage(system.file("images", "sample-color.png", package="EBImage"))
# display(imgcol)

## ----readImageColor-post, echo=FALSE, fig.width=dim(imgcol)[1L]/.dpi, fig.height=dim(imgcol)[2L]/.dpi, dpi=.dpi/2----
display(imgcol)

## ----readImageMulti-pre, include=FALSE----------------------------------------
nuc = readImage(system.file("images", "nuclei.tif", package="EBImage"))

## ----readImageMulti, eval=FALSE-----------------------------------------------
# nuc = readImage(system.file("images", "nuclei.tif", package="EBImage"))
# display(nuc, method = "raster", all = TRUE)

## ----readImageMulti-post, echo=FALSE, fig.width=dim(nuc)[1L]/.dpi, fig.height=dim(nuc)[2L]/.dpi, dpi=.dpi----
display(nuc, method = "raster", all = TRUE)

## ----displayFrame, echo=FALSE, fig.width=dim(nuc)[1L]/.dpi, fig.height=dim(nuc)[2L]/.dpi, dpi=.dpi/2----
display(nuc, method = "raster", frame = 2)

## ----writeImage, eval=FALSE---------------------------------------------------
# writeImage(imgcol, "sample.jpeg", quality = 85)

## ----str----------------------------------------------------------------------
str(img)

## ----dim----------------------------------------------------------------------
dim(img)

## ----imageData----------------------------------------------------------------
imageData(img)[1:3, 1:6]

## ----as.array-----------------------------------------------------------------
is.Image( as.array(img) )

## ----hist, fig.width=6, fig.height=6, dev=.dev--------------------------------
hist(img)
range(img)

## ----show---------------------------------------------------------------------
img

## ----print--------------------------------------------------------------------
print(img, short=TRUE)

## ----printcol-----------------------------------------------------------------
print(imgcol, short=TRUE)

## ----numberOfFrames-----------------------------------------------------------
numberOfFrames(imgcol, type = "render")
numberOfFrames(imgcol, type = "total")

## ----nuc----------------------------------------------------------------------
nuc

## ----colorMode, fig.width=dim(imgcol)[1L]/.dpi, fig.height=dim(imgcol)[2L]/.dpi, dpi=.dpi----
colorMode(imgcol) = Grayscale
display(imgcol, all=TRUE)

## ----Image-character, fig.width=7/.dpi, fig.height=7/.dpi, dpi=10*.dpi--------
colorMat = matrix(rep(c("red","green", "#0000ff"), 25), 5, 5)
colorImg = Image(colorMat)
colorImg
display(colorImg, interpolate=FALSE)

## ----negative, fig.width=dim(img)[1L]/.dpi, fig.height=dim(img)[2L]/.dpi, dpi=.dpi/2----
img_neg = max(img) - img
display( img_neg )

## ----arithmetic, fig.width=dim(img)[1L]/.dpi, fig.height=dim(img)[2L]/.dpi, dpi=.dpi/2----
img_comb = combine(
  img,
  img + 0.3,
  img * 2,
  img ^ 0.5
)

display(img_comb, all=TRUE)

## ----cropthreshold-pre, echo=FALSE--------------------------------------------
img_crop = img[366:749, 58:441]
img_thresh = img_crop > .5

## ----cropthreshold, eval=FALSE------------------------------------------------
# img_crop = img[366:749, 58:441]
# img_thresh = img_crop > .5
# display(img_thresh)

## ----cropthreshold-post, echo=FALSE, fig.width=dim(img_thresh)[1L]/.dpi, fig.height=dim(img_thresh)[2L]/.dpi, dpi=.dpi/2----
display(img_thresh)

## ----img_thresh---------------------------------------------------------------
img_thresh

## ----transpose, fig.width=dim(img)[2L]/.dpi, fig.height=dim(img)[1L]/.dpi, dpi=.dpi/2----
img_t = transpose(img)
display( img_t )

## ----translate, fig.width=dim(img)[1L]/.dpi, fig.height=dim(img)[2L]/.dpi, dpi=.dpi/2----
img_translate = translate(img, c(100,-50))
display(img_translate)

## ----rotate-pre, echo=FALSE---------------------------------------------------
img_rotate = rotate(img, 30, bg.col = "white")

## ----rotate, eval=FALSE-------------------------------------------------------
# img_rotate = rotate(img, 30, bg.col = "white")
# display(img_rotate)

## ----rotate-post, echo=FALSE, fig.width=dim(img_rotate)[1L]/.dpi, fig.height=dim(img_rotate)[2L]/.dpi, dpi=.dpi/2----
display(img_rotate)

## ----resize-pre, echo=FALSE---------------------------------------------------
img_resize = resize(img, w=256, h=256)

## ----resize, eval=FALSE-------------------------------------------------------
# img_resize = resize(img, w=256, h=256)
# display(img_resize )

## ----resize-post, echo=FALSE, fig.width=dim(img_resize)[1L]/.dpi, fig.height=dim(img_resize)[2L]/.dpi, dpi=.dpi/2----
display(img_resize)

## ----flipflop, fig.width=2*dim(img)[1L]/.dpi, fig.height=dim(img)[2L]/.dpi, dpi=.dpi/2----
img_flip = flip(img)
img_flop = flop(img)

display(combine(img_flip, img_flop), all=TRUE)

## ----affine, fig.width=dim(img)[1L]/.dpi, fig.height=dim(img)[2L]/.dpi, dpi=.dpi/2----
m =  matrix(c(1, -.5, 128, 0, 1, 0), nrow=3, ncol=2)
img_affine = affine(img, m)
display( img_affine )

## ----makeBrush, fig.width=6, fig.height=6, dev=.dev---------------------------
w = makeBrush(size = 31, shape = 'gaussian', sigma = 5)
plot(w[(nrow(w)+1)/2, ], ylab = "w", xlab = "", cex = 0.7)

## ----lopass, fig.width=dim(img)[1L]/.dpi, fig.height=dim(img)[2L]/.dpi, dpi=.dpi/2----
img_flo = filter2(img, w)
display(img_flo)

## ----gblur, fig.width=dim(nuc)[1L]/.dpi, fig.height=dim(nuc)[2L]/.dpi, dpi=.dpi----
nuc_gblur = gblur(nuc, sigma = 5)
display(nuc_gblur, all=TRUE )

## ----highpass, fig.width=dim(img)[1L]/.dpi, fig.height=dim(img)[2L]/.dpi, dpi=.dpi/2----
fhi = matrix(1, nrow = 3, ncol = 3)
fhi[2, 2] = -8
img_fhi = filter2(img, fhi)
display(img_fhi)

## ----medianFilter, fig.width=dim(img)[1L]/.dpi, fig.height=dim(img)[2L]/.dpi, dpi=.dpi/2----
l = length(img)
n = l/10
pixels = sample(l, n)
img_noisy = img
img_noisy[pixels] = runif(n, min=0, max=1)
display(img_noisy)
img_median = medianFilter(img_noisy, 1)
display(img_median)

## ----logo-pre, echo=FALSE-----------------------------------------------------
shapes = readImage(system.file('images', 'shapes.png', package='EBImage'))
logo = shapes[110:512,1:130]

## ----logo, eval=FALSE---------------------------------------------------------
# shapes = readImage(system.file('images', 'shapes.png', package='EBImage'))
# logo = shapes[110:512,1:130]
# display(logo)

## ----logo-post, echo=FALSE, fig.width=dim(logo)[1L]/.dpi, fig.height=dim(logo)[2L]/.dpi, dpi=.dpi----
display(logo)

## ----kern, fig.width=7/.dpi, fig.height=7/.dpi, dpi=10*.dpi-------------------
kern = makeBrush(5, shape='diamond')
display(kern, interpolate=FALSE)

## ----morph, fig.width=2*dim(logo)[1L]/.dpi, fig.height=dim(logo)[2L]/.dpi, dpi=.dpi----
logo_erode= erode(logo, kern)
logo_dilate = dilate(logo, kern)

display(combine(logo_erode, logo_dilate), all=TRUE)

## ----otsu, fig.width=dim(nuc)[1L]/.dpi, fig.height=dim(nuc)[2L]/.dpi, dpi=.dpi----
threshold = otsu(nuc)
threshold
nuc_th = combine( mapply(function(frame, th) frame > th, getFrames(nuc), threshold, SIMPLIFY=FALSE) )
display(nuc_th, all=TRUE)

## ----filter2thresh, fig.width=dim(nuc)[1L]/.dpi, fig.height=dim(nuc)[2L]/.dpi, dpi=.dpi----
disc = makeBrush(31, "disc")
disc = disc / sum(disc)
offset = 0.05
nuc_bg = filter2( nuc, disc )
nuc_th = nuc > nuc_bg + offset
display(nuc_th, all=TRUE)

## ----thresh, fig.width=dim(nuc)[1L]/.dpi, fig.height=dim(nuc)[2L]/.dpi, dpi=.dpi----
display( thresh(nuc, w=15, h=15, offset=0.05), all=TRUE )

## ----bwlabel------------------------------------------------------------------
logo_label = bwlabel(logo)
table(logo_label)

## ----max_logolabel------------------------------------------------------------
max(logo_label)

## ----displaybw, fig.width=dim(logo)[1L]/.dpi, fig.height=dim(logo)[2L]/.dpi, dpi=.dpi----
display( normalize(logo_label) )

## ----colorCode, fig.width=dim(logo)[1L]/.dpi, fig.height=dim(logo)[2L]/.dpi, dpi=.dpi----
display( colorLabels(logo_label) )

## ----watershed, fig.width=dim(nuc)[1L]/.dpi, fig.height=dim(nuc)[2L]/.dpi, dpi=.dpi----
nmask = watershed( distmap(nuc_th), 2 )
display(colorLabels(nmask), all=TRUE)

## ----voronoiExample, fig.width=dim(nuc)[1L]/.dpi, fig.height=dim(nuc)[2L]/.dpi, dpi=.dpi----
voronoiExamp = propagate(seeds = nmask, x = nmask, lambda = 100)
voronoiPaint = colorLabels (voronoiExamp)
display(voronoiPaint)

## ----rmObjects, fig.width=2*dim(logo)[1L]/.dpi, fig.height=dim(logo)[2L]/.dpi, dpi=.dpi----
objects = list(
    seq.int(from = 2, to = max(logo_label), by = 2),
    seq.int(from = 1, to = max(logo_label), by = 2)
    )
logos = combine(logo_label, logo_label)
z = rmObjects(logos, objects, reenumerate=FALSE)
display(z, all=TRUE)

## ----uniqueIDs----------------------------------------------------------------
showIds = function(image) lapply(getFrames(image), function(frame) unique(as.vector(frame)))

showIds(z)

## ----reenumeratedIDs----------------------------------------------------------
showIds( reenumerate(z) )

## ----fillHull, fig.width=dim(logo)[1L]/.dpi, fig.height=dim(logo)[2L]/.dpi, dpi=.dpi----
filled_logo = fillHull(logo)
display(filled_logo)

## ----floodFill-logo, fig.width=dim(logo)[1L]/.dpi, fig.height=dim(logo)[2L]/.dpi, dpi=.dpi----
rgblogo = toRGB(logo)
points = rbind(c(50, 50), c(100, 50), c(150, 50))
colors = c("red", "green", "blue")
rgblogo = floodFill(rgblogo, points, colors)
display( rgblogo )

## ----floodFill-img, fig.width=dim(img)[1L]/.dpi, fig.height=dim(img)[2L]/.dpi, dpi=.dpi/2----
display(floodFill(img, rbind(c(200, 300), c(444, 222)), col=0.2, tolerance=0.2))

## ----paintObjects, fig.width=dim(img)[1L]/.dpi, fig.height=dim(img)[2L]/.dpi, dpi=.dpi/2----
d1 = dim(img)[1:2]
overlay = Image(dim=d1)
d2 = dim(logo_label)-1

offset = (d1-d2) %/% 2

overlay[offset[1]:(offset[1]+d2[1]), offset[2]:(offset[2]+d2[2])] = logo_label

img_logo = paintObjects(overlay, toRGB(img), col=c("red", "yellow"), opac=c(1, 0.3), thick=TRUE)

display( img_logo )

## ----load, fig.height=dim(nuc)[2L]/.dpi, fig.width=dim(nuc)[1L]/.dpi, warning=FALSE, dpi=.dpi----
nuc = readImage(system.file('images', 'nuclei.tif', package='EBImage'))
cel = readImage(system.file('images', 'cells.tif', package='EBImage'))

cells = rgbImage(green=1.5*cel, blue=nuc)
display(cells, all = TRUE)

## ----nmask, fig.width=dim(nuc)[1L]/.dpi, fig.height=dim(nuc)[2L]/.dpi, dpi=.dpi----
nmask = thresh(nuc, w=10, h=10, offset=0.05)
nmask = opening(nmask, makeBrush(5, shape='disc'))
nmask = fillHull(nmask)
nmask = bwlabel(nmask)

display(nmask, all=TRUE)

## ----ctmask, fig.width=dim(nuc)[1L]/.dpi, fig.height=dim(nuc)[2L]/.dpi, dpi=.dpi----
ctmask = opening(cel>0.1, makeBrush(5, shape='disc'))
cmask = propagate(cel, seeds=nmask, mask=ctmask)

display(ctmask, all=TRUE)

## ----res, fig.width=dim(nuc)[1L]/.dpi, fig.height=dim(nuc)[2L]/.dpi, dpi=.dpi----
segmented = paintObjects(cmask, cells, col='#ff00ff')
segmented = paintObjects(nmask, segmented, col='#ffff00')

display(segmented, all=TRUE)

## ----sessionInfo--------------------------------------------------------------
sessionInfo()

# Package tools

# Copyright (c) 2006 Oleg Sklyar

# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public License
# as published by the Free Software Foundation; either version 2.1
# of the License, or (at your option) any later version.

# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.

# See the GNU Lesser General Public License for more details.
# LGPL license wording: http://www.gnu.org/licenses/lgpl.html

## checks whether 'x' is a suitable image
validImage = function(x) {
  z = validImageObject(x)
  if (isTRUE(z))
    TRUE
  else 
    stop(z)
}

## if required changes the storage.mode of 'x' to 'double' 
castImage = function(x) {
  if (!is.double(x)) storage.mode(x) = 'double'
  x
}

## clip pixel data to the specified range
clipImage = function(x, range = c(0, 1)) {
  normalize(x, ft = NULL, inputRange = range)
}

## check if x (indexing image) and ref (image) are compatible
checkCompatibleImages=function(x, ref, type='total') {
  xn = deparse(substitute(x), width.cutoff = 500L, nlines = 1)
  refn = deparse(substitute(ref), width.cutoff = 500L, nlines = 1)
  validImage(x)
  if (!missing(ref)) {
    validImage(ref)
    if ( numberOfFrames(x, type) != numberOfFrames(ref, type) ) stop(sprintf("'%s' and '%s' must have the same number of '%s' frames", xn, refn, type))
    if ( any(dim(x)[1:2] != dim(ref)[1:2]) ) stop(sprintf("'%s' and '%s' must have the same spatial 2D dimensions", xn, refn, type))
  }
}

localCurvature = function(x, h=100, maxk=10000) {
 # check that x is of proper type
 if (!is.matrix(x) & !is.data.frame(x)) stop("'x' should be a data frame or a matrix")
 # if data frame convert to matrix to standardize processing
 if (is.data.frame(x)) x = as.matrix(x)
 # check that x has proper dimensions
 if (length(dim(x))!=2L || dim(x)[2L]!=2L) stop("'x' should have dimensions of N x 2")

 # compute arc length
 px = c(0, cumsum(sqrt(rowSums(diff(x)^2))))

 # smooth and calculate derivatives
 poly = lp(px, deg=2, h=h)
 derivatives = apply(x, 2, function(v) {
  d1 = locfit(v ~ poly, deriv=1, maxk=maxk)
  d2 = locfit(v ~ poly, deriv=c(1,1), maxk=maxk)
  list(predict(d1, newdata=data.frame(px=px)),
  predict(d2, newdata=data.frame(px=px)))
 })
 
 # calculate signed curvature
 curvature = function(d) (d[[1]][[1]]*d[[2]][[2]] - d[[2]][[1]]*d[[1]][[2]]) / (d[[1]][[1]]^2+d[[2]][[1]]^2)^1.5

 return(list(contour=x, curvature=curvature(derivatives), length=max(px, na.rm=TRUE)))
}

thresh = function (x, w=5, h=5, offset=0.01) {
  validImage(x)
  if (w<1 || h<1) 
    stop ("filter width 'w' and height 'h' must be larger than 1")
  
  d = dim(x)[1:2]
  d = (d-1)/2
  if (w > d[1L])
    stop ("filter width exceeds picture width")
  if (h > d[2L])
    stop ("filter height exceeds picture height")
  
  return (.Call(C_thresh, x, as.numeric(c(w, h, offset))))
}

distmap = function (x, metric=c('euclidean', 'manhattan')) {
  validImage(x)
  if (any(is.na(x))) stop("'x' shouldn't contain any NAs")
  metric = switch(match.arg(metric), euclidean=0L, manhattan=1L) 
  return (.Call(C_distmap, x, metric))
}

makeBrush = function(size, shape=c('box', 'disc', 'diamond', 'Gaussian', 'line'), step=TRUE, sigma=0.3, angle=45) {
  if(! (is.numeric(size) && (length(size)==1L) && (size>=1)) ) stop("'size' must be an odd integer.")
  shape = match.arg(arg = tolower(shape), choices = c('box', 'disc', 'diamond', 'gaussian', 'line'))

  if(size %% 2 == 0){
    size = size + 1
    warning(paste("'size' was rounded to the next odd number: ", size))
  }
  
  if (shape=='box') z = matrix(1L, size, size)
  else if (shape == 'line') {
    angle = angle %% 180
    angle.radians = angle * pi / 180;
    tg = tan(angle.radians)
    sizeh = (size-1)/2
    if ( angle < 45 || angle > 135) {
      z.x = sizeh
      z.y = round(sizeh*tg)
    }
    else {
      z.y = sizeh
      z.x = round(sizeh/tg)
    }
    z = array(0L, dim=2*c(z.x, z.y)+1);
    for (i in -sizeh:sizeh) {
      if ( angle < 45 || angle > 135) {
        ## scan horizontally
        i.x = i
        i.y = round(i*tg)
      }
      else {
        ## scan vertically
        i.y = i
        i.x = round(i/tg) 
      }
      z[i.x+z.x+1, i.y+z.y+1] = 1L
    }
  }
  else if (shape=='gaussian') {
    x = seq(-(size-1)/2, (size-1)/2, length.out=size)
    x = matrix(x, size, size)
    z = exp(- (x^2 + t(x)^2) / (2*sigma^2))
    z = z / sum(z)
  } else {
    ## pixel center coordinates
    x = 1:size -((size+1)/2)
    
    ## for each pixel, compute the distance from its center to the origin, using L1 norm ('diamond') or L2 norm ('disc')
    if (shape=='disc') {
      z = outer(x, x, FUN=function(X,Y) (X*X+Y*Y))
      mz = (size/2)^2
      z = (mz - z)/mz
      z = sqrt(ifelse(z>0, z, 0))
    } else {
      z = outer(x, x, FUN=function(X,Y) (abs(X)+abs(Y)))
      mz = (size/2)
      z = (mz - z)/mz
      z = ifelse(z>0, z, 0)
    }

    if (step) z = ifelse(z>0, 1L, 0L)
  }
  z
}

dilate = function (x, kern=makeBrush(5, shape='diamond')) {
  validImage(x)
  return (.Call(C_morphology, x, kern, 0L))
}

erode = function (x, kern=makeBrush(5, shape='diamond')) {
  validImage(x)
  return (.Call(C_morphology, x, kern, 1L))
}

opening = function (x, kern=makeBrush(5, shape='diamond')) {
  validImage(x)
  return (.Call(C_morphology, x, kern, 2L))
}

closing = function (x, kern=makeBrush(5, shape='diamond')) {
  validImage(x)
  return (.Call(C_morphology, x, kern, 3L))
}

whiteTopHat = function (x, kern=makeBrush(5, shape='diamond')) {
  validImage(x)
  return (.Call(C_morphology, x, kern, 4L))
}

blackTopHat = function (x, kern=makeBrush(5, shape='diamond')) {
  validImage(x)
  return (.Call(C_morphology, x, kern, 5L))
}

selfComplementaryTopHat = function (x, kern=makeBrush(5, shape='diamond')) {
  validImage(x)
  return (.Call(C_morphology, x, kern, 6L))
}

# Copyright (c) 2005-2007 Oleg Sklyar

# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public License
# as published by the Free Software Foundation; either version 2.1
# of the License, or (at your option) any later version.

# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.

# See the GNU Lesser General Public License for more details.
# LGPL license wording: http://www.gnu.org/licenses/lgpl.html

## - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
paintObjects = function (x, tgt, opac=c(1, 1), col=c('red', NA), thick=FALSE, closed=FALSE) {
  checkCompatibleImages(x, tgt, 'render')
  if (colorMode(x)!=Grayscale) stop("'", deparse(substitute(x), width.cutoff = 500L, nlines = 1L), "' must be in 'Grayscale' color mode")
  
  if ((l=length(col))>2L) col[1:2] else col = switch(l+1L, c(NA, NA), c(col, NA), col)
  if ((l=length(opac))>2L) opac[1:2] else opac = switch(l+1L, c(1, 1), c(opac, 1), opac)
  
  zcol = is.na(col)
  col[zcol] = 'white'
  opac[zcol] = 0
  
  opac = as.numeric(opac)
  if (any(opac < 0) || any(opac > 1)) stop("all opacity values must be in the range [0,1]" )
  
  ## behavior at image borders
  if ( isTRUE(closed) ) {
    col[3L] = col[1L] 
    opac[3L] = opac[1L]
  }
  else {
    col[3L] = col[2L] 
    opac[3L] = opac[2L]
  }
    
  .Call(C_paintObjects, castImage(x), castImage(tgt), opac, Image(col, colormode = colorMode(tgt)), isTRUE(thick))
}


## - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
stackObjects = function (x, ref, combine=TRUE, bg.col='black', ext) {
  ## check arguments
  if (colorMode(x) != Grayscale) stop("'x' must be an image in 'Grayscale' color mode or an array")
  checkCompatibleImages(x, ref, 'render')
  nz = numberOfFrames(x, 'total')
  
  ## uses 'computeFeatures.moment' to get centers and theta
  hf = lapply(getFrames(x), computeFeatures.moment)

  if (missing(ext)) {
    ext = unlist(sapply(hf, function(h) h[, 'm.majoraxis']))/2
    ext = quantile(ext, 0.98, na.rm=TRUE)
  }
  xy = lapply(hf, function(h) h[,c('m.cx', 'm.cy'), drop=FALSE])
  if (nz==1L) xy = xy[[1L]]
  
  bg.img = Image(bg.col, colormode=colorMode(ref))
  
  res = .Call(C_stackObjects, castImage(x), castImage(ref), castImage(bg.img), xy, as.numeric(ext))
  if (!combine || !is.list(res)) res
  else combine(res)
}

## - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
rmObjects = function (x, index, reenumerate=TRUE) {
  validImage(x)
  if ( storage.mode(x)!="integer" ) stop("storage mode of object mask should be integer")
  if (!is.list(index)) index = list(index)
  index = lapply (index, as.integer)
  .Call(C_rmObjects, x, index, isTRUE(reenumerate))
}

## - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
reenumerate = function(x) {
  validImage(x)
  if ( storage.mode(x)!="integer" ) stop("storage mode of object mask should be integer")
  .Call(C_rmObjects, x, NULL, TRUE)
}

## - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
display = function(x, method, ...) {
  validImage(x)
  
  if ( missing(method) )
    method = getOption("EBImage.display", if ( interactiveMode() ) "browser" else "raster")
  method = match.arg(method, c("browser", "raster"))
  
  switch(method,
         browser = displayWidget(x, ...),
         raster  = displayRaster(x, ...)
  ) 
}

## test whether run interactively and not in `rmarkdown::render`
## https://github.com/yihui/knitr/issues/926#issuecomment-68503962
interactiveMode = function() interactive() && !isTRUE(getOption('knitr.in.progress', FALSE))

## remove any args to fun
filterDotsArgs = function(fun, ...) {
  dotsArgs <- list(...)
  dotsArgs[which(names(dotsArgs) %in% names(formals(fun)))] <- NULL
  dotsArgs
}

## displays an image using R graphics
## - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
displayRaster = function(image, frame, all = FALSE, drawGrid = isTRUE(spacing==0),
                         nx, spacing = 0, margin = 0, interpolate = TRUE, ...) {
  ## remove any args to "browser" method
  dotsArgs <- filterDotsArgs(displayWidget, ...)
  
  all = isTRUE(all)
  nf = numberOfFrames(image, type="render")
  
  ## display all frames in a grid-like environment
  if ( all ) {
    if ( missing(nx) )
      ncol = ceiling(sqrt(nf))
    else {
      ncol = as.integer(nx[1L])
      if ( ncol==0L ) stop( "'nx' must be coercible to a non-zero integer" )
    }
    
    ## negative values are interpreted as number of rows
    if ( ncol<0 ) {
      nrow = -ncol
      ncol = ceiling(nf/nrow)
    } else {
      nrow = ceiling(nf/ncol)
    }
  }
  ## display a single frame only
  else {
    if ( missing(frame) ) {
      frame = 1L
      if ( nf > 1L ) message("Only the first frame of the image stack is displayed.\nTo display all frames use 'all = TRUE'.")
    }
    else {
      frame = as.integer(frame[1L])
      if ( frame<1L || frame>nf ) stop( "Frame index out of range: 'frame' must be between 1 and ", nf)
    }
    ncol = nrow = 1L
  }
  
  d <- dim(image)[1:2]
  xdim <- d[1L]
  ydim <- d[2L]
  
  ## returns a pair of horizontal and vertical pixel dimensions
  asPixelDims <- function(x, d) {
    x <- as.numeric(x)
    x[ is.na(x) | x < 0] <- 0L
    
    ## values smaller than one are interpreted as fractions of image dimensions
    frac = which( x>0 & x<1 )
    x[frac] <- (d*x)[frac]
    
    ## single fraction is taken against the maximum dimension
    if ( identical(frac, 1L) )
      x = x * max(d)/d
    
    ## round to integer pixel values
    x = as.integer(round(x))
    x = rep(x, length.out=2L)
  }
  
  spacing <- asPixelDims(spacing, d)
  margin  <- asPixelDims(margin, d)
  
  ## draw grid unless spacing is set
  if ( missing(drawGrid) )
    drawGrid = if ( any(spacing)>0 ) FALSE else TRUE
  
  xmar <- margin[1L]
  ymar <- margin[2L]
  
  xsep <- spacing[1L]
  ysep <- spacing[2L]
  
  xran = c(0, ncol*xdim + (ncol-1)*xsep) + .5
  yran = c(0, nrow*ydim + (nrow-1)*ysep) + .5
  
  xranm = xran + c(-xmar, xmar)
  yranm = yran + c(-ymar, ymar)
  
  ## set graphical parameters
  user <- do.call(par, c(list(bty="n", mai=c(0,0,0,0), xaxs="i", yaxs="i", xaxt="n", yaxt="n", col = "white"), dotsArgs))
  on.exit(par(user))
  plot(xranm, yranm, type="n", xlab="", ylab="", asp=1, ylim=rev(yranm))
      
  for(r in seq_len(nrow)) {
    for(c in seq_len(ncol)) {
      f = if(all) (r-1)*ncol + c else frame
      if ( f <= nf )
        rasterImage(as.nativeRaster(getFrame(image, f, type="render")),
                    (c-1)*(xdim+xsep) + .5,
                    r*(ydim+ysep)-ysep + .5,
                    c*(xdim+xsep)-xsep + .5,
                    (r-1)*(ydim+ysep) +.5,
                    interpolate = interpolate)
      else
        break
    }
  }    
  
  ## draw grid lines
  if ( isTRUE(drawGrid) && all && nf>1 ) {
    clip(xran[1L], xran[2L], yran[1L], yran[2L])
    abline(h = seq_len(nrow-1)*(ydim + ysep) - ysep/2 + .5,
           v = seq_len(ncol-1)*(xdim + xsep) - xsep/2 + .5)
  }
  
  invisible()
}

## displays an image using JavaScript
## - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
displayInBrowser = function(x, title, ...) {
  if ( missing(title) )
    title = deparse(substitute(x, parent.frame()), width.cutoff = 500L, nlines = 1L)
  
  ## template and script files
  templateFile = system.file("viewer","display.template", package = "EBImage")
  cssFile = system.file("viewer","display.css", package = "EBImage")
  scriptFile = system.file("viewer","viewer.js", package = "EBImage")
  tempDir = tempfile("",,"")
  htmlFile = "display.html"
  imageFile = tempfile("",tempDir,".png")

  if(!dir.create(tempDir))
    stop("Error creating temporary directory.")

  ## read the template file
  f = file(templateFile, "r")
  a = readLines(f)
  close(f)

  ## get image parameters
  d = dim(x)
  if ( length(d)==2L ) d = c(d, 1L)
  nf = numberOfFrames(x, "render")

  ## fill-in in the template
  a = sub("HEIGHT",d[2L], sub("WIDTH",d[1L], sub("FRAMES",nf, sub("IMAGE",basename(imageFile), sub("TITLE", title, a)))))

  ## temporarily switch to tempdir and write the files
  wd = setwd(tempDir)
  
  ## fill missing channels
  if ( isTRUE(colorMode(x) == Color && d[3L] < 3L) ) {
    fd = d
    fd[3L] = 3L - d[3L]
    imageData(x) = abind::abind(x, array(0, fd), along = 3L)
  }
  
  writeImage(x, imageFile)
  cat(a, file=htmlFile, sep="\n")
  file.copy(scriptFile, tempDir)
  file.copy(cssFile, tempDir)
  setwd(wd)

  ## create browser query
  query = paste0("file://", normalizePath(file.path(tempDir,"display.html")))

  browseURL(query)
}

plot.Image = function(x, ...) displayRaster(x, ...)

as.nativeRaster = function(x) .Call(C_nativeRaster, castImage(x))

## Display Widget

displayWidget <- function(x, title, embed = !interactiveMode(), tempDir = tempfile(""), ...) {
  ## remove any args to "raster" method
  dotsArgs <- filterDotsArgs(displayRaster, ...)

  ## get image parameters
  d = dim(x)
  if ( length(d)==2L ) d = c(d, 1L)
  
  ## fill missing channels
  if ( isTRUE(colorMode(x) == Color && d[3L] < 3L) ) {
    fd = d
    fd[3L] = 3L - d[3L]
    imageData(x) = abind::abind(x, array(0, fd), along = 3L)
  }
  
  x = clipImage(x) ## clip the image and change storage mode to double
  x = transpose(x)
  
  frames = seq_len(numberOfFrames(x, type='render'))
  dependencies = NULL
  
  if ( isTRUE(embed) ) {
    
    data <- sapply(frames, function(i) base64Encode(writePNG(getFrame(x, i, 'render'))))
    data <- sprintf("data:image/png;base64,%s", data)
    
  } else {
    if ( !dir.exists(tempDir) && !dir.create(tempDir, recursive=TRUE) )
        stop("Error creating temporary directory.")
    
    files = file.path(tempDir, sprintf("frame%.3d.png", frames))
    
    ## store image frames into individual files
    for (i in frames)
      writePNG(getFrame(x, i, 'render'), files[i])
    
    dependencies = htmlDependency(
      name = basename(tempDir),
      version = "0",
      src = list(tempDir)
    )
    
    filePath = file.path(sprintf("%s-%s", dependencies$name, dependencies$version), basename(files))
    
    ## set libdir unless run in shiny

    if ( !isNamespaceLoaded("shiny") || is.null(shiny::getDefaultReactiveDomain()))
      filePath = file.path("lib", filePath)
    
    data = filePath
  }
  
  # widget options
  opts = list(
    data = data,
    width = d[1L],
    height = d[2L]
  )
  
  # create widget
  do.call(createWidget, c(list(
    name = 'displayWidget',
    package = 'EBImage',
    x = opts,
    sizingPolicy = sizingPolicy(padding = 0, browser.fill = TRUE),
    dependencies = dependencies),
    dotsArgs)
  )
}

## Shiny bindings for displayWidget

displayOutput <- function(outputId, width = '100%', height = '500px'){
  htmlwidgets::shinyWidgetOutput(outputId, 'displayWidget', width, height, package = 'EBImage')
}

renderDisplay <- function(expr, env = parent.frame(), quoted = FALSE) {
  if (!quoted) { expr <- substitute(expr) } # force quoted
  htmlwidgets::shinyRenderWidget(expr, displayOutput, env, quoted = TRUE)
}

# defined outside of `setMethod` in order to subsequently call `formals` on it
.abind.Image <- function(...) {
  arglist = list(...)
  
  if (is.list(arglist[[1L]]))
    arglist <- arglist[[1L]]
  
  x <- arglist[[1L]]
  
  cm = colorMode(x)
  
  if (!all(cm==vapply(arglist, colorMode, integer(1L), USE.NAMES=FALSE)))
    stop("images have different color modes")
  
  y <- callNextMethod()
  
  ## don't introduce unnecessary dimnames
  if (all(vapply(arglist, function(x) is.null(dimnames(x)), logical(1L), USE.NAMES=FALSE)))
    dimnames(y) <- NULL
  
  ## use first object as template which might not be correct for Image subclasses
  imageData(x) <- y
  
  x
}


## neccesary for performing correct dispatch on object lists
.abind.list <- function(...) {
  cl <- match.call(expand.dots = FALSE)
  
  ## only a single list-valued argument for ... is supported
  if (length(cl$`...`)!=1L) {
    callNextMethod()
  }
  else {
    dots <- list(...)[[1L]]
    
    cl$`...` <- quote(dots)
  
    ## choose the apropriate method based on object classes
    cls <- unique(unlist(lapply(dots, class)))
    cl[[1L]] <- quote(selectMethod(abind, cls))
  
    eval(cl)
  }
}

# avoid "Undefined global functions or variables" NOTE during R CMD check
N <- NULL

# have the same formal arguments as the original function
formals(.abind.Image) <- formals(abind::abind)
formals(.abind.list) <- formals(abind::abind)

setMethod("abind", "Image", .abind.Image)
setMethod("abind", "list", .abind.list)

colormap <- function(x, palette = heat.colors(256L)) {
  validImage(x)
  if (colorMode(x) != Grayscale) 
    stop("Color mapping can be applied only to grayscale images")
  
  tmp <- round(1 + x * (length(palette)-1L))
  tmp <- Image(array(palette[tmp], dim(tmp)))
  
  if ( is.Image(x) ) {
    res <- x
    imageData(res) <- tmp
    colorMode(res) <- Color
  }
  else {
    res <- tmp
  }
  
  res
}

drawCircle <- function(img, x, y, radius, col, fill=FALSE, z=1) {
  validImage(img)
  if ( any(is.na(img)) ) stop("'img' shouldn't contain any NAs")

  toScalarInteger = function(x) suppressWarnings(as.integer(x)[1L])
  
  ## check whether parameters are OK
  r = toScalarInteger(radius)
  if ( !isTRUE(r>0L) ) stop("'radius' must be a positive integer")
  
  z = toScalarInteger(z)
  if ( !isTRUE(z>0L) || isTRUE(z>numberOfFrames(img, 'render')) ) stop("'z' must be a positive integer lower than the number of image frames")
  
  xy = c(toScalarInteger(x), toScalarInteger(y))
  if ( any(is.na(xy)) ) stop("'x', 'y' must be numeric scalars")
  
  fill = suppressWarnings(as.integer(isTRUE(as.logical(fill)[1L])))
  
  if (colorMode(img)==Color) {
    rgb = as.numeric(col2rgb(col)/255)
    if (length(rgb)!=3 || any(is.na(rgb))) stop("In Color mode, 'col' must be a valid color")
  } else {
    rgb = as.numeric(c(col, 0, 0))
    if (length(rgb)!=3 || any(is.na(rgb))) stop("In Grayscale mode, 'col' must be a scalar value")
  }
  
  invisible(.Call(C_drawCircle, castImage(img), c(xy-1L, z-1L, r), rgb, fill))
}

# Copyright (c) 2005-2007 Oleg Sklyar

# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public License
# as published by the Free Software Foundation; either version 2.1
# of the License, or (at your option) any later version.

# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.

# See the GNU Lesser General Public License for more details.
# LGPL license wording: http://www.gnu.org/licenses/lgpl.html

## - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
tile = function (x, nx=10, lwd=1, fg.col="#E4AF2B", bg.col="gray") {
  if (is.list(x)) lapply(x, tile, nx=nx, lwd=lwd, fg.col=fg.col, bg.col=bg.col)
  else {
    validImage(x)
    if ( nx < 1 || lwd < 0 || lwd > 100 ) stop( "wrong range of arguments, see help for range details" )
    
    hdr = Image(c(fg.col,bg.col), colormode = colorMode(x))
    
    .Call(C_tile, x, hdr, as.integer(c(nx, lwd)))
  }
}

## - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
untile = function (x, nim, lwd=1) {
  validImage(x)
  nim = as.integer(nim)
  lwd = as.integer(lwd)
  if (length(nim)<2) stop("'nim' must contain two values for the number of images in x and y directions")
  if (lwd<0) stop("wrong line width")
  .Call(C_untile, x, nim, lwd)
}

# Class Image, definition and methods

# Copyright (c) 2005-2007 Oleg Sklyar

# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public License
# as published by the Free Software Foundation; either version 2.1
# of the License, or (at your option) any later version.

# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.

# See the GNU Lesser General Public License for more details.
# LGPL license wording: http://www.gnu.org/licenses/lgpl.html

# - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
Grayscale = 0L
Color     = 2L

## - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
setClass ("Image",
  representation (colormode="integer"),
  prototype (colormode=Grayscale),
  contains = "array"
)

## - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
Image = function(data = array(0, dim=c(1,1)), dim, colormode) {
  if ( !missing(dim) && length(dim)<2 )
    stop("The number of dimensions dim must be at least 2")
  
  setdim = function(data) {
    if (is.array(data))
      base::dim(data)
    else 
      c(1L, length(data))
  }
  
  ## special character case
  if (is.character(data)) {
    colormode = 
      if (missing(colormode)) 
        Color
      else
        parseColorMode(colormode)
    
    if (missing(dim))
      dim = setdim(data)
      
    if ( colormode==Color )
      dim = c(dim[1:2], 3L, dim[-(1:2)])  
    
    dimnames = dimnames(data)
    data = col2rgb(data)/255
    
    if ( colormode==Color ) {
      channels = if (length(dim)<3L) 1L else seq_len(dim[3L])
      data = abind::abind(lapply(channels, function(ch) {
        array( if (ch>3L) 0 else data[ch,,drop=FALSE], dim[-3L], dimnames)
      }), along = 2.5)
      # replace a list of NULLs by the original NULL
      if(is.null(dimnames)) dimnames(data) = NULL
    }
    else
      data = array(data = (data[1,,drop=FALSE] + data[2,,drop=FALSE] + data[3,,drop=FALSE]) / 3, dim = dim, dimnames = dimnames)
  }
  ## default numeric case
  else {
    if (missing(dim))
      dim = setdim(data)
    
    colormode = 
      if (missing(colormode))
        if (is.Image(data))
          colorMode(data)
        else
          Grayscale
      else 
        parseColorMode(colormode)
  }
  
  return( new("Image", 
    .Data = 
      ## improve performance by not calling array constructor on well formed arrays
      if( is.array(data) && prod(dim)==length(data) ) {
        if( !identical(dim(data), dim) )
          dim(data) = dim
        data
      }
      else {
        array(data, dim = dim)
      },
    colormode = colormode
  ))    
}

## - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
is.Image <- function (x) is(x, "Image")

## - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
as.Image <- function(x) UseMethod("as.Image")

as.Image.Image = function(x) {
  ## true Image
  if ( class(x)=="Image" )
    x
  ## coerce subclasses to Image superclass
  else
    as(x, "Image")
}

as.Image.default = function(x) Image(x)

## - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
## define method for the S3 generic 'as.array'
as.array.Image = function(x, ...) x@.Data

## - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
colorMode = function (y) {
  if (is(y, 'Image')) y@colormode
  else Grayscale
}

`colorMode<-` = function(y, value) {
  if (is(y, 'Image')) {
    y@colormode = parseColorMode(value)
    validObject(y)
    y
  } 
  else warning('Color mode of an array cannot be changed, the array should be cast into an Image using \'Image\'')
}

## - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
imageData = function (y) {
  if (is(y, 'Image')) y@.Data
  else y
}

`imageData<-` = function (y, value) {
  if (is(y, 'Image')) {
    y@.Data = value
    validObject(y)
    y
  } 
  else value
}

## - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
validImageObject = function(object) {
  .Call(C_validImageObject, object) 
}
setValidity("Image", validImageObject)

## Overloading binary operators
# - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
setMethod("Ops", signature(e1="Image", e2="Image"),
	function(e1, e2) {
          e1@.Data = callGeneric(e1@.Data, e2@.Data)
          validObject(e1)
          return(e1)
	}
)
setMethod("Ops", signature(e1="Image", e2="numeric"),
	function(e1, e2) {
          e1@.Data = callGeneric(e1@.Data, e2)
          validObject(e1)
          return(e1)
	}
)
setMethod("Ops", signature(e1="numeric", e2="Image"),
	function(e1, e2) {
          e2@.Data = callGeneric(e1, e2@.Data)
          validObject(e2)
          return(e2)
	}
)

## legacy code addressing broken 'Math2' S4GroupGenerics in R 3.1.2
setMethod("Math2", "Image",
          function(x, digits) {
            x@.Data <- callGeneric(x = x@.Data, digits = digits)
            x
          }
)

## explicit method definition for 'log' needed because of the extra formal argument, see ?Math
setMethod("log", signature(x="Image"),
          function(x, base) {
            x@.Data <- callGeneric(x = x@.Data, base = base)
            x
          }
)

## private
## determines image type
## - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
determineFileType = function(files, type) {
  ## helper function 
  collapseTypeSynonyms = function(x){
    x = tolower(x)
    x[x=="tif"] = "tiff"
    x[x=="jpg"] = "jpeg"
    x
  }

  if (!(is.character(files)&&(length(files)>=1)))
    stop("Please supply at least one filename.")

  if (missing(type)) {
    type = unique(collapseTypeSynonyms(sapply(strsplit(files, split=".", fixed=TRUE), 
      function(x) {
        if (length(x)>1) 
          x[length(x)] 
        else if (length(x)==1)
          stop(sprintf("Unable to determine type of %s: Filename extension missing.", x)) 
        else 
          stop("Unable to determine type: Empty filename.") 
      }
    )))
    if (length(type)>1)
      stop(sprintf("File type implied by the file name extensions must be unique, but more than one type was found: %s.", paste(type, collapse=", ")))
  } else {
    if (!(is.character(type)&&(length(type)==1)))
      stop("'type' must be a character vector of length 1.")
    else 
      type = collapseTypeSynonyms(type)
  }
  return(type)
}

## - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
readImage = function(files, type, all=TRUE, names = sub("\\.[^.]*$", "", basename(files)), ...) {
  
  .readURL = function(url, buffer=2^24){
    f = try(file(url, "rb"), silent=TRUE)
    if (inherits(f,"try-error")) 
      stop(attr(f,"condition")$message)
    
    rawData = bufData = NULL;
    
    while( length(bufData <- readBin(f, 'raw', buffer)) > 0L )
      rawData = c(rawData, bufData)
    
    try(close(f), silent=TRUE)
    
    rawData
  }
  
  type = try (determineFileType(files, type), silent=TRUE)
  if (inherits(type,"try-error")) 
    stop(attr(type,"condition")$message)
  
  .readFun = switch(type,
                    tiff = function(x, ...) {
                      y = readTIFF(x, all = all, ...)
                      if ( (l=length(y)) > 1L) {
                        if (!is.null(names)) names(y) = seq_len(l)
                        # make sure all frames have the same dimensions
                        if(!all(duplicated.default(lapply(y, dim))[-1L]))
                          stop(sprintf("Frame dimensions of the '%s' file are not equal.", x))
                      }
                      y
                    },
                    jpeg = function(x, ...) readJPEG(x, ...),
                    png  = function(x, ...) readPNG(x, ...),
                    stop(sprintf("Invalid type: %s. Currently supported formats are JPEG, PNG, and TIFF.", type))
  )
  
  .loadFun = function(i, ...) {
    ## first look for local files
    if(!file.exists(i)){
      ## might still be a remote URL  
      w = options(warn=2)
      i = try(.readURL(i), silent = TRUE)
      options(w)
      ## is not URL
      if (inherits(i,"try-error")) {
        warning( sub("(converted from warning) ", "", attr(i,"condition")$message, fixed = TRUE) )
        return(NULL)
      }
    }
    ## ensure that the file is not a directory
    else if (file.info(i)$isdir){
      warning(sprintf("Cannot open %s: Is directory.", i))
      return(NULL)
    }
    
    return(.readFun(i, ...))
  }
  
  # flatten nested image list and remove null elements
  .flatten <- function(x) {
    while(any(vapply(x, is.list, logical(1L)))) {
      x <- lapply(x, function(x) if(is.list(x)) x else list(x))
      x <- unlist(x, recursive = FALSE) 
    }
    x[!vapply(x, is.null, logical(1L))]
  }
  
  # stratify processing for single and multiple files to increase performance
  
  # single file
  if(length(files) == 1L){
    y = .loadFun(files, ...)
  }
  
  #  multiple files
  else {
    y = lapply(files, .loadFun, ...)
    names(y) = names
    y = .flatten(y)   
  }
  
  if(is.list(y)){
    if(length(y)==0L) stop("Empty image stack.")
    
    # check whether image dimensions match
    if(!all(duplicated.default(lapply(y, dim))[-1L]))
      stop("Images have different dimensions")
    
    y1 = y[[1L]]
    channels = channelLayout(y1)
    if(length(y) == 1L)
      y = y1
    else
      y = vapply(y, identity, FUN.VALUE=y1)
    rm(y1)
  }
  else{
    channels = channelLayout(y)
  }
  
  y = transpose(y)
  
  new("Image", .Data = y, colormode = if(isTRUE(charmatch(channels,'G') == 1)) Grayscale else Color )
}

## private
## returns channel layout of a pixel array: G, GA, RGB, or RGBA
## - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
channelLayout = function(x){
  y = dim(x) 
  return( switch(if(length(y)==2) 1 else if (length(y)==3 && y[3]<=4) y[3] else 5, 'G', 'GA', 'RGB', 'RGBA', 'unknown') )
}

## private
## helper function used to check whether image data can be written on X bits without accuracy loss
## - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
isXbitImage = function(x, bits) {
  b = 2^bits - 1
  x = as.numeric(x)
  
  ## fast termination if not
  y = b * x[1] 
  if (trunc(y)!=y)
    FALSE
  else  {
    y = b * x
    all(trunc(y)==y)
  }
}

## - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
writeImage = function (x, files, type, quality=100L, bits.per.sample, compression='none', ...) {  
  validImage(x)
  
  type = try (determineFileType(files, type), silent=TRUE)
  if (inherits(type,"try-error")) 
    stop(attr(type,"condition")$message)
  
  ## automatic bits.per.sample guess
  if ( (type=='tiff') && missing(bits.per.sample) ) {
    if (isXbitImage(x, 8L)) 
      bits.per.sample = 8L 
    else 
      bits.per.sample = 16L
  }
  
  writeFun = switch(type,
                    tiff = function(x, file, ...) writeTIFF(x, file, bits.per.sample=bits.per.sample, compression=compression, ...),
                    jpeg = function(x, file, ...) writeJPEG(x, file, quality=quality/100, ...),
                    png  = function(x, file, ...) writePNG(x, file, ...),
                    stop(sprintf("Invalid type: %s. Currently supported formats are JPEG, PNG, and TIFF.", type))
  )
  
  if ((quality<1L) || (quality>100L))
    stop("'quality' must be a value between 1 and 100.")
  
  nf = numberOfFrames(x, type='render')
  lf = length(files)
  
  if ( (lf!=1) && (lf!=nf) )
    stop(sprintf("Image contains %g frame(s) which is different from the length of the file name list: %g. The number of files must be 1 or equal to the size of the image stack.", nf, lf))
  
  else {
    frames = seq_len(nf)
    
    x = clipImage(x) ## clip the image and change storage mode to double
    x = transpose(x)    
    
    if ( lf==1 && nf>1 ) {
      ## store all frames into a single TIFF file
      if (type=='tiff') {
        
        ## create list of image frames
        la = lapply(frames, function(i) getFrame(x, i, 'render'))
        
        if (nf==writeFun(la, files, ...))
          return(invisible(files))
        else
          stop(sprintf("Error writing file sequence to TIFF."))
      }
      ## generate file names for frames
      else {
        basename = unlist(strsplit(files, split=".", fixed=TRUE))
        prefix   = basename[-length(basename)]
        suffix   = basename[length(basename)]
        
        files = vapply(frames, function(i) paste0(paste0(prefix, collapse='.'), '-', i-1, '.', suffix), character(1))
      }
    }
    
    ## store image frames into individual files
    for (i in frames)
      writeFun(getFrame(x, i, 'render'), files[i], ...)
    return(invisible(files))
  }
}

## - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
setMethod ("[", "Image",
           function(x, i , j, ..., drop = TRUE) {
             ## list(...) doesn't work in this S4 method dispatch framework we
             ## are using the following trick: the current call is evaluated,
             ## but using x@.Data instead of x in the previous calling frame
             sc = sys.call()
             args = as.list(sc[-c(1L, 2L)])
             numIndices = length(args) - !is.null(args$drop)
             
             # when subsetting with single index treat as array
             if (numIndices == 1L) {
               callNextMethod()
             }
             else {
               # subset image array without dropping dimensions in order to
               # preserve spatial dimensions
               sc$drop = FALSE
               sc[[2L]] = call('slot', sc[[2L]], '.Data')
               y = eval.parent(sc)
                              
               # drop dims higher than 2 unless 'drop' explicitly set to FALSE
               if(!isTRUE(args$drop==FALSE) && length( (d = dim(y)) ) > 2L){
                 dims = which(d==1L)
                 y = adrop(y, drop = dims[dims>2L]) 
               }
               
               x@.Data = y
               validObject(x)
               x
             }
           })

## - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
getFrame = function(y, i, type = c('total', 'render')) {
    type = match.arg(type)
    
    n = numberOfFrames(y, type)
    if (i<1 || i>n) stop("'i' must belong between 1 and ", n)
    
    # return the argument if no subsetting neccessary
    ld = length(dim(y))
    fd = if (colorMode(y)==Color && type=='render' && ld>2L) 3L else 2L
    if (ld==fd) return(y)
    
    type = switch(type, total = 0L, render = 1L)
    
    .Call(C_getFrame, y, as.integer(i), type)
}

getFrames = function(y, i, type = c('total', 'render')) {
  type = match.arg(type)
  
  n = numberOfFrames(y, type)
  
  if ( missing(i) ) 
    i = seq_len(n)
  else {
    i = as.integer(i)
    if ( any( i<1L || i>n ) ) stop("'i' must be a vector of numbers ranging from 1 to ", n)
  }
  
  type = switch(type, total = 0L, render = 1L)
  
  .Call(C_getFrames, y, i, type)
}


## numberOfFrames
## If type='total', returns the total number of frames
## If type='render', return the number of frames to be rendered after color channel merging
## - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
numberOfFrames = function(y, type = c('total', 'render')) {
    type = match.arg(type)
    type = switch(type, total = 0L, render = 1L)
    .Call(C_numberOfFrames, y, type)
}

numberOfChannels = function(y, d = dim(y), cm = colorMode(y)) {
  if ( cm==Grayscale || is.na(d[3L]) ) 1L else d[3L]
}

## - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
showImage = function (object, short=FALSE) {
  nd = dim(object)
  ld = length(nd)
  dimorder = names(dimnames(object))
  
  cat(class(object)[1],'\n')
  
  cat('  colorMode    :',c('Grayscale', NA, 'Color')[1+colorMode(object)],'\n')
  cat('  storage.mode :',typeof(object),'\n')
  cat('  dim          :',nd,'\n')
  if ( !is.null(dimorder) )
  cat('  dimorder     :',dimorder,'\n')
  cat('  frames.total :',numberOfFrames(object,'total'),'\n')
  cat('  frames.render:',numberOfFrames(object,'render'),'\n')
  
  if ( !isTRUE(short) ) {
    if (nd[1]>5) nd[1] = 5
    if (nd[2]>6) nd[2] = 6
    if (ld>2) nd[3:ld] = 1
    
    ndl = lapply(nd, seq_len)
    
    nds = paste0('[1:',nd[1],',1:',nd[2],paste(rep(',1',ld-2),collapse=''),']')
    
    cat('\nimageData(object)', nds, '\n', sep='')
    print.default(asub(object@.Data, ndl))
  }
  invisible()
}

setMethod ("show", signature(object = "Image"), function(object) showImage(object))

print.Image <- function(x, ...) showImage(x, ...)

## - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
setMethod ("image", signature(x="Image"),
  function(x, i, xlab = "", ylab = "", axes = FALSE, col = gray((0:255) / 255), useRaster = TRUE, ...) {
    nf = numberOfFrames(x, type="total")
    
    if ( missing(i) ) {
      i = 1L
      if ( nf > 1L ) message( "Missing frame index for an image stack, assuming 'i = 1'")
    }
    else {
      i = as.integer(i[1L])
      if ( i<1L || i>nf ) stop( "Frame index out of range: 'i' must be between 1 and ", nf)
    }
    
    d <- dim(x)
    X <- 1:d[1L]
    Y <- 1:d[2L]
    Z <- getFrame(x, i, "total")
    image.default(x=X, y=Y, z=Z, asp=1, col=col, axes=axes, xlab=xlab, ylab=ylab, useRaster=useRaster, ...)
  }
)

## private function to select a channel from a Color image
## failsafe, will return a black image if the channel doesn't exist
selectChannel = function(x, i) {
  if (colorMode(x)==Grayscale) stop("in 'selectChannel', color mode must be 'Color'")
  
  dim = dim(x)
  y = NULL 
  
  if (length(dim) < 3) {
    if (i==1) y = x
  } 
  else {
    if (i <= dim[3]) y = asub(x, i, 3)
  }
  
  if (is.null(y)) 
    y = new("Image", .Data = array(0, dim[-3]), colormode = Grayscale)
  else
    colorMode(y) = Grayscale
  
  y
}

## - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
channel = function (x, mode) {
  if (!is.array(x)) x=Image(x)
  if (is.character(x)) x=Image(x)
  mode=tolower(mode)
  validObject(x)
  if (colorMode(x)==Grayscale) {
    return(switch(mode,
                  rgb=rgbImage(red=x,green=x,blue=x),
                  grey=,gray=,luminance=x,
                  r=,red=stop('invalid conversion mode, can\'t extract the red channel from a \'Grayscale\' image'),
                  g=,green=stop('invalid conversion mode, can\'t extract the green channel from a \'Grayscale\' image'),
                  b=,blue=stop('invalid conversion mode, can\'t extract the blue channel from a \'Grayscale\' image'),
                  asred=rgbImage(red=x),
                  asgreen=rgbImage(green=x),
                  asblue=rgbImage(blue=x),
                  x11=array(rgb(x,x,x),dim=dim(x)),
                  stop('invalid conversion mode')
                  ))
  } else {
    return(switch(mode,
                  rgb=x,
                  ## Color->Grayscale conversion is done using 1/3 uniform RGB weights
                  grey=,gray=(selectChannel(x,1)+selectChannel(x,2)+selectChannel(x,3))/3,
                  luminance=0.2126*selectChannel(x,1)+0.7152*selectChannel(x,2)+0.0722*selectChannel(x,3),
                  r=,red=selectChannel(x,1),
                  g=,green=selectChannel(x,2),
                  b=,blue=selectChannel(x,3),
                  asred=selectChannel(x,1),
                  asgreen=selectChannel(x,2),
                  asblue=selectChannel(x,3),
                  x11=array(rgb(selectChannel(x,1),selectChannel(x,2),selectChannel(x,3)),dim=dim(selectChannel(x,1))),
                  stop('invalid conversion mode')
                  ))
  }
}

toRGB = function(x) {
  channel(x, "rgb")
}

## - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
setMethod ("hist", signature(x="Image"),
  function (x, breaks=64L, rg=range(x, na.rm=TRUE), main=paste("Image histogram:", length(x), "pixels"), xlab="Intensity", ...) {
    if ( !is.numeric(rg) || length(rg) != 2 ) stop("'range' must be a numeric vector of length 2.")
    if ( colorMode(x)==Color ) {
      d3 = dim(x)[3L]
      nc = if ( is.na(d3) ) 1L else min(d3, 3L)
      colores = c("red", "green", "blue")[1:nc]
      y = sapply(colores, function(m) imageData(channel(x, m)), simplify = FALSE, USE.NAMES = TRUE)
    } else {
      y = list(black=imageData(x))
    }

    if(length(breaks)==1L) {
      bins = breaks
      breaks = seq(rg[1], rg[2], length.out=breaks+1L)
    } else {
      bins = length(breaks) - 1L
    }
    
    h = lapply(y, hist.default, breaks=breaks, plot=FALSE)
    xx = vapply(h, "[[", vector(mode = "double", length = bins+1L), "breaks")
    yy = vapply(h, "[[", vector(mode = "integer", length = bins), "counts")
    yy = rbind(yy, yy[bins,])
    matplot(xx, yy, type="s", lty=1L, main=main, xlab=xlab, col=names(y), ylab="counts", ...)
    
    if ( length(h)==1L && names(h)=="black")
      h = h$black
    
    invisible(h)
  }
)

## - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
combineImages = function (x, y, ...) {
  dx = dim(x)
  dy = dim(y)
  cx = colorMode(x)
  cy = colorMode(y)
  
  if ( dx[1] != dy[1] || dx[2] != dy[2] ) stop("images must have the same 2D frame size to be combined")
  if ( cx != cy ) stop("images must have the same color mode to be combined")
  
  ## merging along position guided by colorMode
  along = if (cx == Color) 4L else 3L
  
  ## add extra dimension in case of single frame Color Images
  if (along == 4L) {
    if (length(dx)==2L) dim(x) = c(dx, 1)
    if (length(dy)==2L) dim(y) = c(dy, 1)
  }
  
  z = abind::abind(x, y, along = along)
  ## don't introduce unnecessary dimnames
  if ( is.null(dimnames(x)) && is.null(dimnames(y)) ) dimnames(z) = NULL
  imageData(x) = z
    
  validObject(x)
  return (x)
}

## general method for the superclass of 'Image' and 'matrix'
setMethod("combine", signature("array", "array"), combineImages)
## explicit methods for subclasses of 'array'
setMethod("combine", signature("matrix", "matrix"), combineImages)
setMethod("combine", signature("Image", "Image"), combineImages)

## special case of combining a list of images
setMethod("combine", signature("list", "missing"), 
  function(x, y, ...) {
    if (!is.null(names(x))) names(x) <- NULL
    do.call(combine, x)
  }
)

## useful when combining image lists containing NULL elements
setMethod("combine", signature("ANY", "NULL"), function(x, y, ...) x)
setMethod("combine", signature("NULL", "ANY"), function(x, y, ...) y)
setMethod("combine", signature("NULL", "NULL"), function(x, y, ...) NULL)

## Median & quantile redefinition
## needed to overcome an isObject() test in median() which greatly slows down median()
## - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
median.Image <- function(x, ...) {
  median(x@.Data, ...)
}

quantile.Image <- function(x, ...) {
  quantile(x@.Data, ...)
}


## - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
rgbImage = function(red=NULL, green=NULL, blue=NULL) {
  classname = function(x) as.character(class(x))  
  
  set_check = function(prop, ch) {
    if ( !is.null(ch) ) {
      if ( is.null(prop) ) {
        prop = list(dim = dim(ch),
                    cls = classname(ch),
                    ref = substitute(ch),
                    dns = !is.null(dimnames(ch)))
      }
      else {
        if  ( !identical(prop$dim, dim(ch)) )
          stop('images must have the same 2D frame size and number of frames to be combined')
        if ( !identical(prop$cls, classname(ch)) )
          stop('images must be object of the same class in order to be combined')
        prop$dns = prop$dns || !is.null(dimnames(ch))
      }
    }
    prop
  }
  
  prop = set_check(NULL, red)
  prop = set_check(prop, green)
  prop = set_check(prop, blue)

  if (is.null(prop)) stop('at least one non-null array must be supplied')
  
  if (is.null(red)) red = array(0, dim=prop$dim)
  if (is.null(green)) green = array(0, dim=prop$dim)
  if (is.null(blue)) blue = array(0, dim=prop$dim)
  
  x = abind::abind(red, green, blue, along=2.5)
  ## don't introduce unnecessary dimnames
  if ( !prop$dns ) dimnames(x) = NULL
  
  if ( extends(prop$cls, "Image") ) {
    res = eval(prop$ref)
    imageData(res) <- x
    colorMode(res) <- Color
  } else {
    res = new("Image", .Data = x, colormode = Color)
  }
  
  res
}

## - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
parseColorMode = function(colormode) {
  icolormode=NA

  if (is.numeric(colormode)) {
    icolormode=colormode
  } else if (is.character(colormode)) {
    icolormode=pmatch(tolower(colormode),c('grayscale', NA, 'color'), duplicates.ok=TRUE,nomatch=NA)-1
  }

  as.integer(icolormode)
}

## - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
## returns the raster representation of an image (by default the first frame)
as.raster.Image = function(x, max = 1, i = 1L, ...) {
  y = getFrame(x, i, type = 'render')
  y = clipImage(y, range = c(0, max))
  y = imageData(y)
  d = dim(y)
  
  ## grayscale
  r <- if (colorMode(x) == Grayscale)
    rgb(y, y, y, maxColorValue = max)
  ## color
  else {
    if (length(d) == 2L)
      rgb(y, 0, 0, maxColorValue = max)
    else
      switch(min(d[3L], 4L),
             rgb(y[,,1L], 0, 0, maxColorValue = max),
             rgb(y[,,1L], y[,,2L], 0, maxColorValue = max),
             rgb(y[,,1L], y[,,2L], y[,,3L], maxColorValue = max),
             rgb(y[,,1L], y[,,2L], y[,,3L], y[,,4L], maxColorValue = max)
      )
  }
  
  attributes(r) <- list(dim = d[2:1], class = "raster")
  r
}

clahe = function(x, nx = 8, ny = nx, bins = 256, limit = 2, keep.range = FALSE) {
  validImage(x)
  .Call(C_clahe, castImage(x), as.integer(nx), as.integer(ny), as.integer(bins), as.double(limit), isTRUE(keep.range))
}

equalize <- function(x, range = c(0, 1), levels = 256){
  validImage(x)
  r = range(x)
  if ( r[1L] == r[2L] ) {
    warning("Image histogram is single-valued and cannot be equalized.")
  }
  else {
    if ( !is.numeric(range) || length(range) != 2L ) stop("'range' must be a numeric vector of length 2.")
    levels = as.integer(levels)
    if ( is.na(levels) || levels < 1L ) stop("Levels must be at least equal 1.")
    breaks = seq(range[1L], range[2L], length.out = levels + 1L)
    x = castImage(x)  # converts logical to numeric
    d = dim(x)
    n = d[1L] * d[2L]
    
    # equalize each frame separately
    .equalize = function(y) {
      h = hist.default(y, breaks = breaks, plot = FALSE)
      cdf = cumsum(h$counts)
      cdf_min = min(cdf[cdf>0])
      
      equalized = ( (cdf - cdf_min) / (prod(dim(y)) - cdf_min) * (range[2L] - range[1L]) ) + range[1L]
      bins = round( (y - range[1L]) / (range[2L] - range[1L]) * (levels-1L) ) + 1L
      
      res = equalized[bins]
    }
    
    res = if ( (ld = length(d)) > 2L ) apply(x, 3L:ld, .equalize)
    else .equalize(x)
    
    # setup output image
    dim(res) = d
    imageData(x) = res
  }
  x
}

clahe = function(x, nx = 8, ny = nx, bins = 256, limit = 2, keep.range = FALSE) {
  validImage(x)
  .Call(C_clahe, castImage(x), as.integer(nx), as.integer(ny), as.integer(bins), as.double(limit), isTRUE(keep.range))
}

## cat tests/test.R | R --vanilla &>tests/test.Rout.save
library("EBImage")

set.seed(0) # make random color permutations in 'colorLabels' reproducible
.digits = floor(-log10(.Machine$double.eps^.5) - 1)

## returns a hashcode given an object
hash = function (x) .Call(digest:::digest_impl, serialize(x, connection=NULL, ascii=FALSE, xdr=FALSE), 7L, -1L, 14L, 0L, 0L, PACKAGE="digest")

hash.old <- function(x) {
  if (is.list(x) && length(x)>0) {
    hash.old(sapply(x, hash.old, USE.NAMES=FALSE))
  }
  else {
    xd <- suppressWarnings(as.numeric(x))
    xd <- xd[!(is.nan(xd)|is.na(xd))]
    sx <- sum(xd*(1:length(xd)))
    if (is.matrix(xd))
      sx + 0.7 * hash.old(dim(xd))
    else
      sx - 0.1
  }
}

## try to evaluate fun(x,...) 
check <- function(fun, x, ..., capture.output=FALSE, suppressWarnings=FALSE, suppressMessages=FALSE, expectError=FALSE, round=FALSE, debug=FALSE) {
  passed <- TRUE

  cat(sprintf("checking \'%s\' %s ", fun, paste(rep(".", 35L-nchar(fun)), collapse = "")))
  
  expr = quote(do.call(fun,c(list(x),list(...))))
  if ( isTRUE(capture.output) ) expr = call("capture.output", expr)
  if ( isTRUE(suppressWarnings) ) expr = call("suppressWarnings", expr)
  if ( isTRUE(suppressMessages) ) expr = call("suppressMessages", expr)
  
  y <- try(eval(expr), silent=TRUE)
  
  if ( (inherits(y, "try-error") && !isTRUE(expectError)) || (is.Image(y) && !validObject(y)) ) {
    y <- NULL
    passed <- FALSE
  }

  if (passed) {
    if (isTRUE(round) && !inherits(y, "try-error"))
      y = if ( is.list(y) ) lapply(y, round, digits=.digits) else round(y, digits = .digits)
    cat("PASS (", hash(y), ") ", hash.old(y), "\n", sep="") 
  }
  else cat("FAIL\n")
  
  if ( isTRUE(debug) ) {
    if (is.Image(y)) show(y)
    if (is.array(y)) display(y, all=TRUE, interpolate=FALSE) 
    else if (is.character(y)) cat(y, sep="\n")
  }
  
  y
}

checkIO <- function(name) {
  cat("checking IO for \'", name, "\' ... ", sep="")
  x = get(name)
  y = FALSE
  if ( !is.null(x) ) {
    y <- try({
      xx <- readImage(writeImage(x, tempfile("", fileext = ".tif")))
      dimnames(xx) <- dimnames(x)
      identical(x, xx)
      }, silent=TRUE)
  }
  if ( isTRUE(y) ) cat("PASS\n") else cat("FAIL\n")
  invisible(y)
}

testIOFunctions <- function(...) invisible(lapply(list(...), function(y) checkIO(y)))

testEBImageFunctions <- function(x) {
  cat("new test (hash=", hash(x), ")\n", sep="")
  
  z <- check("show", x, capture.output=TRUE)
  z <- check("print", x, short=TRUE, capture.output=TRUE)
  if ( typeof(x)=="logical" )
    z <- check("hist", EBImage:::castImage(x), breaks = c(0, .5, 1))
  else
    z <- check("hist", x)
  
  ## pixel arithmetic
  z <- check(">", x, 0.5)
  z <- check("+", x, x)
  z <- check("/", x, 2)
  z <- check("*", 2L, x)
  z <- check("median", x)
  z <- check("quantile", x)

  ## image methods
  z <- check("Image", x, colormode="Color")
  z <- check("as.Image", x)
  z <- check("is.Image", x)
  z <- check("imageData", x)
  z <- check("imageData<-", x, z)
  z <- check("as.raster", x)
  z <- check("colorMode<-", x, Grayscale, suppressWarnings=TRUE)
  y <- check("numberOfFrames", x, type="render")
  z <- if ( y==1L ) check("getFrames", x, 1L, "render") else check("getFrames", x)
  z <- check("display", x, method = "browser", tempDir = file.path(dirname(tempdir()), "EBImage_tmp"))
  if ( y>2L ) {
    z <- check("display", x, method = "raster", all = TRUE)
    z <- check("image", x, i = 3L)
  }
  else {
    z <- if (y==1L) check("display", x, method = "raster") else check("display", x, method = "raster", frame = 2L, suppressMessages=TRUE)
    z <- check("image", x, suppressMessages=TRUE)
  }
  
  ## drawCircle
  d <- dim(x)
  c.x <- ceiling(d[1L]/2)
  c.y <- ceiling(d[2L]/2)
  radius <- max(c.x-1, 1)
  nf <- numberOfFrames(x, "render")
  fill <- nf > 1
  col <- if ( colorMode(x)==Color ) "yellow" else 1
  z <- check("drawCircle", x, c.x, c.x, radius, col, fill, nf)
  
  ## subset
  sub <- list(x, 1:min(10,d[1L]), 1:min(7,d[2L]))
  if (length(d)>2) sub <- c(sub, rep(TRUE, length(d)-2))
  z <- do.call("check", c("[", sub))

  ## spatial transform
  z <- check("resize", x, 137, 22)
  z <- check("rotate", x, 20)
  z <- check("flip", x)
  z <- check("flop", x)
  z <- check("translate", x, c(-7, 5), bg.col=1)
  z <- check("affine", x, matrix(c(-7, 5, 0.1, -0.2, 0.3, 1), ncol=2L))
  z <- check("transpose", x)

  ## segmentation
  fd = pmin(5, (d[1:2]-1)/2)
  z <- check("thresh", x, w=fd[1L], h=fd[2L])
  y <- check("channel", x, "luminance")
  z <- check("otsu", y)
  y <- suppressWarnings(normalize(y, separate=FALSE))
  y <- check("bwlabel", y > 0.5)
  z <- check("colorLabels", y, suppressWarnings=TRUE)
  z <- check("stackObjects", y, x)
  z <- check("stackObjects", Image(dim=dim(y)), x)
  cls <- if ( colorMode(x)==Color ) TRUE else FALSE
  z <- check("paintObjects", y, x, col=c("#ff00ff", "#ffff00"), opac=c(1.0, 0.5), thick=cls, closed=cls)  
  z <- check("rmObjects", y, as.list(seq_len(numberOfFrames(y))), cls)
  z <- check("reenumerate", z)
  z <- check("reenumerate", y)
  
  ## features
  x1 <- getFrame(x, 1)
  x2 <- list(x=x1, y=2*x1)
  y1 <- getFrame(y, 1)
  expandRef <- if ( min(dim(x1)) > 31L ) function(ref, refnames) standardExpandRef(ref, refnames, gblob(n=31L)) else NULL
  z <- check("computeFeatures", y1, x2, expandRef = expandRef, round = !is.null(expandRef))
  z <- check("computeFeatures", y1, x2, expandRef = expandRef, properties = TRUE)
  
  ## curvature
  y <- check("ocontour", x>0.5)
  if (length(y) > 0L ) z <- check("localCurvature", y[[1L]], round=TRUE)

  ## filtering
  z <- check("normalize", x, suppressWarnings=TRUE)
  z <- check("normalize", x, inputRange=c(0.1, 0.9), suppressWarnings=TRUE)
  z <- check("gblur", x, sigma=1, expectError=min(d)<7, round=TRUE)
  y <- if (cls) "replicate" else if (nf>1) 0.5 else "circular"
  z <- check("filter2", x, array(1, dim=c(5, 5)), y, round=TRUE)
  z <- check("medianFilter", x, 2)
  z <- check("equalize", x, suppressWarnings=TRUE)

  ## morphological operations
  y <- x > 0.5
  z <- check("erode", y)
  z <- check("dilate", y, makeBrush(5, 'disc'))
  z <- check("opening", y, makeBrush(7, 'line'))
  z <- check("closing", y, makeBrush(4, 'line', angle=0), suppressWarnings=TRUE)
  z <- check("whiteTopHat", x, makeBrush(3, 'box'))
  z <- check("blackTopHat", x, makeBrush(5, 'disc'))
  z <- check("selfComplementaryTopHat", x)
  z <- check("distmap", y)
  z <- check("watershed", z)
  z <- check('floodFill', x, c(5, 5), 0.5)
  z <- check('fillHull', y)

  ## propagate
  y <- thresh(x, w=fd[1L], h=fd[2L], offset=0.02)
  y <- fillHull(y)
  y <- bwlabel(y)
  z <- check("propagate", x, y, x>0.5)
  
  ## colorspace
  z <- check("toRGB", x)
  z <- check("rgbImage", x, x>0.5)
  z <- check("colormap", channel(x, "luminance"), heat.colors(16L))

  ## image stacking, combining, tiling
  y <- check("combine", list(NULL, x, x, NULL, NULL))
  z <- check("combine", x, y, y)
  y <- check("tile", z, nx=3)
  z <- check("untile", y, c(3, 2))
  y <- check("abind", list(x, x), along=length(d)+1L)
  cat("\n")
}


## check error handling
try.readImage <- function(...) tryCatch(suppressWarnings(readImage(...)), error = function(e) NULL)
mock <- try.readImage(system.file("images", package="EBImage"), type="png")
mock <- try.readImage("http://www.huber.embl.de/EBImage/missing.file ", type="png")

## single greyscale and color images
sample <- try.readImage(system.file("images","sample.png", package="EBImage"))
sample.color <- try.readImage(system.file("images","sample-color.png", package="EBImage"))
## multi-frame image stack
f = system.file("images","nuclei.tif", package="EBImage")
nuclei = try.readImage(c(f, f))
## test reading from URL
logo <- try.readImage("http://www.huber.embl.de/EBImage/logo.png")

## test: IO operations
testIOFunctions("sample", "sample.color", "nuclei", "logo")

## test: black image
testEBImageFunctions(Image(0, c(8, 8)))

## test: white image
testEBImageFunctions(Image(1L, c(5, 5)))

## test: 2D Grayscale 64x48
x <- nuclei[50:113,208:255,2]
testEBImageFunctions(as.array(x))

## test: 2D Color 32x48x1
x <- sample[1:32, 1:48]
testEBImageFunctions(Image(as.vector(x), dim(x), Color))

## test: 3D Color 65x17x3
x <- sample.color[1:65, 1:17,]
testEBImageFunctions(x)

## test: 3D Grayscale logical 32x32x2
x <- sample[32:63, 32:63]
x <- x > otsu(x)
x <- combine(x, x)
testEBImageFunctions(x)

## test: 4D Color 33x16x3x2
x <- sample.color[1:33, 1:16,]
x <- combine(x, x)
testEBImageFunctions(x)

## test: 4D Grayscale 16x33x2x3
colorMode(x) <- Grayscale
imageData(x) <- aperm(x, c(2L, 1L, 4L, 3L))
testEBImageFunctions(x)

## test: Image subclass
ImageSubclass <- setClass ("ImageSubclass", contains = "Image", slots = c(foo = "character"))
x <- ImageSubclass(x, foo="bar")
testEBImageFunctions(x)

# Flood fill for images and matrices

# Copyright (c) 2005 Oleg Sklyar

# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public License
# as published by the Free Software Foundation; either version 2.1
# of the License, or (at your option) any later version.

# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.

# See the GNU Lesser General Public License for more details.
# LGPL license wording: http://www.gnu.org/licenses/lgpl.html

## - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
floodFill = function(x, pts, col, tolerance=0) {
  validImage(x)
  nf = numberOfFrames(x, 'render')
  nc = numberOfChannels(x)
  
  ## make sure that `pt` and `col` are lists of length matching the number of frames
  format_as_list = function(x, nf) {
    if ( is.list(x) )
      if ( length(x)==nf )
        return(x)
      else if ( nf > 1L)
        stop(sprintf("length of '%s' must match the number of 'render' frames"), deparse(substitute(x)))
    rep(list(x), nf) 
  }
  pts = format_as_list(pts, nf)
  col = format_as_list(col, nf)
  
  for (i in seq_len(nf) ) {
    pti = pts[[i]]
    if ( is.list(pti) )
      if ( is.data.frame(pti) ) 
        pti = as.matrix(pti)
      else 
        pti = unlist(pti, use.names=FALSE)
    storage.mode(pti) = "integer"
    if ( !is.matrix(pti) || dim(pti)[2L]!=2L )
      pti = matrix(pti, nrow=length(pti)/2, ncol=2L, byrow=TRUE)
    np = dim(pti)[1L]
    if ( any( pti<1L ) || any( pti>rep(dim(x)[1:2], each=np) ) )
      stop("coordinates of starting point(s) 'pts' must be inside image boundaries")
    pts[[i]] = pti
    
    cli = col[[i]]
    cli = if ( is.character(cli) )
      unlist(lapply(cli, function(col) rep_len(col2rgb(col, alpha=TRUE)/255, nc)))
    else 
      rep(cli, each=nc)
    storage.mode(cli) = storage.mode(x) 
    cli = matrix(rep_len(cli, np * nc), nrow=np, ncol=nc, byrow=TRUE)
    col[[i]] = cli
  }
  
  return( .Call(C_floodFill, x, pts, col, as.numeric(tolerance)))
}

## - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
fillHull = function(x) {
  validImage(x)
  return(.Call(C_fillHull, x))
}

normalizeImage = function(object, separate = TRUE, ft = c(0, 1), inputRange) {
  validImage(object)
  
  if ( missing(inputRange) ) {
    inputRange = NULL;
  }
  
  if ( !is.null(inputRange) ) {
    inputRange <- as.numeric(inputRange)
    if ( diff(inputRange) == 0 ) stop("clipping range is 0")
    separate = FALSE
  }
   
  if ( !is.null(ft)) {
    ft <- as.numeric (ft)
    if ( diff(ft) == 0 ) stop("normalization range is 0")
  }
  
  if ( is.null(ft) && is.null(inputRange) )
    stop("please specify either normalization range or clipping range")
  
  .Call(C_normalize, castImage(object), isTRUE(separate), ft, inputRange)
}

## general method for the superclass of 'Image' and 'matrix'
setMethod("normalize", signature("array"), normalizeImage)

## explicit methods for subclasses of 'array'
setMethod("normalize", signature("matrix"), normalizeImage)
setMethod("normalize", signature("Image"), normalizeImage)

## tests
# nocov start
tests = function() {

  ## build labeled image x and reference y
  y = readImage(system.file("images", "nuclei.tif", package="EBImage"))[,,1]
  x = thresh(y, 10, 10, 0.05)
  x = opening(x, makeBrush(5, shape='disc'))
  x = bwlabel(x)
  display(x)
  display(y)

  ## standard call 88 features
  ft = computeFeatures(x, y)
  pft = computeFeatures(x, y, properties=TRUE)
  stopifnot(all(colnames(ft)==pft$name))
  
  ## call without expandRef 49 features
  ft = computeFeatures(x, y, expandRef=NULL)
  pft = computeFeatures(x, y, expandRef=NULL, properties=TRUE)
  stopifnot(all(colnames(ft)==pft$name))
  
  ## changing parameters
  ft = computeFeatures(x, y, basic.quantiles=c(0.2, 0.3), haralick.scales=c(1, 4, 8), expandRef=NULL)
  pft = computeFeatures(x, y, basic.quantiles=c(0.2, 0.3), haralick.scales=c(1, 4, 8), expandRef=NULL, properties=TRUE)
  stopifnot(all(colnames(ft)==pft$name))
  
  ## call with 3 reference images
  yc = list(d=y, t=flip(y), a=flop(y))
  ft = computeFeatures(x, yc)
  pft = computeFeatures(x, yc, properties=TRUE)
  stopifnot(all(colnames(ft)==pft$name))
  
  ## test standardExpandRef
  z = standardExpandRef(yc)
  #str(z)
  display(combine(z))
  
  ## image with one point
  x2 = array(0, dim=dim(x))
  x2[100, 100] = 1
  ft = computeFeatures(x2, y)
}
# nocov end

## main function
computeFeatures = function(x, ref, methods.noref=c("computeFeatures.moment", "computeFeatures.shape"),
  methods.ref=c("computeFeatures.basic", "computeFeatures.moment", "computeFeatures.haralick"),
  xname="x", refnames, properties=FALSE, expandRef=standardExpandRef, ...) {
  ## check arguments
  x = checkx(x)
  ref = convertRef(ref, refnames)
  refnames = names(ref)
  nref = length(ref)
 
  ## expand ref
  if (!is.null(expandRef)) {
    ref = expandRef(ref, refnames)
    refnames = names(ref)
    nref = length(ref)
  }
  
  ## compute features
  if ( !isTRUE(properties) ) {
    ## prepare data
    xs = splitObjects(x)
    if (length(xs)==0) return(NULL)
    
    ## compute features without reference
    features.noref = do.call(cbind, lapply(methods.noref, do.call, list(x=x, properties=FALSE, xs=xs, ...)))
    
    ## compute features with reference, for each channel
    features.ref = lapply(1:nref, function(i) {
      do.call(cbind, lapply(methods.ref, do.call, list(x=x, ref=ref[[i]], properties=FALSE, xs=xs, ...)))
    })
    names(features.ref) = refnames

    ## sum up data into a single matrix
    features = c("0"=list(features.noref), features.ref)
    features = features[!sapply(features, is.null)]
    for (i in 1:length(features)) colnames(features[[i]]) = paste(names(features)[i], colnames(features[[i]]), sep=".")
    features = do.call(cbind, features)
    colnames(features) = paste(xname, colnames(features), sep=".")
    features
  } else {
    ## feature properties
    pfeatures.noref = do.call(rbind, lapply(methods.noref, do.call, list(properties=TRUE, ...)))
    pfeatures.ref = do.call(rbind, lapply(methods.ref, do.call, list(properties=TRUE, ...)))
    pfeatures.ref = lapply(1:length(refnames), function(z) pfeatures.ref)
    names(pfeatures.ref) = refnames

    ## sum up data
    pfeatures = c("0"=list(pfeatures.noref), pfeatures.ref)
    pfeatures = pfeatures[!sapply(pfeatures, is.null)]
    for (i in 1:length(pfeatures)) pfeatures[[i]]$name = paste(names(pfeatures)[i], pfeatures[[i]]$name, sep=".")
    pfeatures = do.call(rbind, pfeatures)
    pfeatures$name = paste(xname, pfeatures$name, sep=".")
    rownames(pfeatures) = NULL
    pfeatures
  }
}

## standard reference expansion
standardExpandRef = function(ref, refnames, filter = gblob()) {
  ## check arguments
  ref = convertRef(ref, refnames)
  nref = length(ref)
  refnames = names(ref)
    
  ## adding joint channels
  if (nref>1) {
    for (i in 1:(nref-1)) {
      for (j in (i+1):nref) {
        refi = ref[[i]]
        refj = ref[[j]]
        jref0 = list( (refi - mean(refi)) * (refj - mean(refj)) )
        names(jref0) = paste0(refnames[i], refnames[j])
        ref = c(ref, jref0)
      }
    }
  }

  ## adding granulometry by blob transform
  bref = lapply(ref, function(r) filter2(r, filter)/2)
  names(bref) = paste0("B", names(ref))
  c(ref, bref)
}

## basic pixel-independant statistics
computeFeatures.basic = function(x, ref, properties=FALSE, basic.quantiles=c(0.01, 0.05, 0.5, 0.95, 0.99), xs, ...) {
  qnames = paste0('b.q', gsub('\\.', '', as.character(basic.quantiles)))
  if ( !isTRUE(properties) ) {
    ## check arguments
    x = checkx(x)
    if (missing(xs)) xs = splitObjects(x)
    if (length(xs)==0) return(NULL)
    ref = convertRef(ref)[[1]]
    
    ## compute features
    features = do.call(rbind, lapply(xs, function(z) {
      z = ref[z]
      q = quantile(z, basic.quantiles)
      names(q) = qnames
      c(b.mean=mean(z), b.sd=sd(z), b.mad=mad(z), q)
    }))

    ## special processing for single points
    z = sapply(xs, length)==1
    features[,'b.sd'] = ifelse(z, 0, features[,'b.sd'])
    features
  }
  else {
    ## feature properties
    data.frame(name=c("b.mean", "b.sd", "b.mad", qnames),
               translation.invariant=TRUE,
               rotation.invariant=TRUE)
  }
}

## shape features
computeFeatures.shape = function(x, properties=FALSE, xs, ...) {
  if ( !isTRUE(properties) ) {
    ## check arguments
    x = checkx(x)
    if (missing(xs)) xs = splitObjects(x)
    if (length(xs)==0) return(NULL)
    
    contours = ocontour(x)
    ## compute features
    features = do.call(rbind, lapply(contours, function(z) {
      cz = apply(z, 2, mean)
      radius = sqrt(rowSums((z - rep(cz, each=nrow(z)))^2))
      radius.mean = mean(radius)
      radius.sd = sqrt(mean((radius - radius.mean)^2))
      
      c(s.perimeter=nrow(z), s.radius.mean=radius.mean, s.radius.sd=radius.sd, s.radius.min=min(radius),
        s.radius.max=max(radius))
    }))
    cbind(s.area=sapply(xs, length), features)
  } else {
    ## feature properties
    data.frame(name=c("s.area", "s.perimeter", "s.radius.mean", "s.radius.sd", "s.radius.min", "s.radius.max"),
               translation.invariant = c(TRUE, TRUE, TRUE, TRUE, TRUE, TRUE),
               rotation.invariant = c(TRUE, TRUE, TRUE, TRUE, TRUE, TRUE))
  }
}

## image moments
computeFeatures.moment = function(x, ref, properties=FALSE, xs, ...) {
  if ( !isTRUE(properties) ) {
    ## check arguments
    x = checkx(x)
    if (missing(xs)) xs = splitObjects(x)
    if (length(xs)==0) return(NULL)
    if (missing(ref)) ref = array(1, dim=dim(x))
    ref = convertRef(ref)[[1]]
    
    ## image moments: computing m{pq} = sum_{x^p*y^q*f(x, y)}
    m00 = sapply(xs, function(z) sum(ref[z]))
    m10 = {w = row(ref)*ref ; sapply(xs, function(z) sum(w[z]))}
    m01 = {w = col(ref)*ref ; sapply(xs, function(z) sum(w[z]))}
    m20 = {w = row(ref)^2*ref ; sapply(xs, function(z) sum(w[z]))}
    m02 = {w = col(ref)^2*ref ; sapply(xs, function(z) sum(w[z]))}
    m11 = {w = col(ref)*row(ref)*ref ; sapply(xs, function(z) sum(w[z]))}
    
    ## derive interesting geometric quantities from image moments
    ## done by diagonalisation of the centered image moment matrix
    ## see http://en.wikipedia.org/wiki/Image_moment
    suppressWarnings({
      cx = m10/m00 ## center of mass x
      cy = m01/m00 ## center of mass y
      mu20 = m20/m00 - cx^2 ## temporary quantity
      mu02 = m02/m00 - cy^2 ## temporary quantity
      mu11 = m11/m00 - cx*cy ## temporary quantity
      det = sqrt(4*mu11^2 + (mu20 - mu02)^2) ## temporary quantity
      theta = atan2(2*mu11, (mu20 - mu02))/2 ## angle
      l1 = sqrt((mu20 + mu02 + det)/2)*4 ## major axis
      l2 = sqrt((mu20 + mu02 - det)/2)*4 ## minor axis
      eccentricity = sqrt(1-l2^2/l1^2) ## eccentricity
    })

    ## special processing for single points
    features = cbind(m.cx=cx, m.cy=cy, m.majoraxis=l1, m.eccentricity=eccentricity, m.theta=theta)
    features[is.na(features) | is.nan(features)] = 0
    features
  } else {
    ## feature properties
    data.frame(name=c("m.cx", "m.cy", "m.majoraxis", "m.eccentricity", "m.theta"),
      translation.invariant = c(FALSE, FALSE, TRUE, TRUE, TRUE),
      rotation.invariant = c(TRUE, TRUE, TRUE, TRUE, FALSE))
  }
}

## haralick features
## h.*: haralick features
computeFeatures.haralick = function(x, ref, properties=FALSE, haralick.nbins=32, haralick.scales=c(1, 2), xs, ...) {
  snames = paste0("s", haralick.scales)
  if ( !isTRUE(properties) ) {
    ## check arguments
    x = checkx(x)
    if (missing(xs)) xs = splitObjects(x)
    if (length(xs)==0) return(NULL)
    ref = convertRef(ref)[[1]]
  
    ## clip data
    ref[ref>1] = 1
    ref[ref<0] = 0
    features = lapply(haralick.scales, function(scale) {
      if (scale>1) {
        combx = seq(1, nrow(x), by=scale)
        comby = seq(1, ncol(x), by=scale)
        xscaled = x[combx, comby]
        refscaled = ref[combx, comby]
      } else {
        xscaled = x
        refscaled = ref
      }
      hf = .haralickFeatures(xscaled, refscaled, nc=haralick.nbins)
      ## after scaling, xscaled may not contain the original object labels
      ## fortunately, .haralickFeatures() leave 0-row for them, therefore, the data
      ## is kept synceda and it is enough to fill up hf with 0-row to level up to
      ## the original number of objects
      rbind(hf, matrix(0, nrow=max(x)-max(xscaled), ncol=ncol(hf)))
    })
    for (i in 1:length(features)) colnames(features[[i]]) = paste(colnames(features[[i]]), snames[i], sep=".")
    do.call(cbind, features)
  } else {
    ## feature properties
    hnames = paste("h", c("asm", "con", "cor", "var", "idm", "sav", "sva", "sen", "ent", "dva", "den", "f12", "f13"), sep=".")
    names = paste(hnames, rep(snames, each=length(hnames)), sep=".")
    data.frame(name=names, translation.invariant=TRUE, rotation.invariant=TRUE)
  }
}

## make a blob
gblob = function(x0=15, n=49, alpha=0.8, beta=1.2) {
  xx = seq(-x0, x0, length.out=n)
  xx = matrix(xx, nrow=length(xx), ncol=length(xx))
  xx = sqrt(xx^2+t(xx)^2)
  z = dnorm(xx, mean=0, sd=alpha) - 0.65*dnorm(xx, mean=0, sd=beta)
  z/sum(z)
}

## convert ref into a list of images, for fast processing
convertRef = function(ref, refnames) {
  if (!is.array(ref) && !is.list(ref)) stop("'ref' must be an array or a list containing the reference images")
  if (is.array(ref)) {
    nref = numberOfFrames(ref, 'total')
    if (inherits(ref, "Image")) ref = imageData(ref)
    ndim = length(dim(ref))
    if (ndim==2) ref = list(ref)
    else if (ndim==3) ref = lapply(1:nref, function(i) ref[,,i])
    else stop ("'ref' must be a 2D or 3D array")
  }
  else if (is.list(ref)) {
    nref = length(ref)
    if (missing(refnames) && !is.null(names(ref))) refnames = names(ref)
    ## sanity check
    ref = lapply(ref, function(r) {
      ndim = length(dim(r))
      if (inherits(r, "Image")) r = imageData(r)
      if (ndim<2 || ndim>3) stop ("'ref' must contain only 2D arrays")
      else if (ndim==3) {
        if (dim(r)[3]>1) stop ("'ref' must contain only 2D arrays")
        else r = r[,,1]
      }
      r
    })
  }
  if (missing(refnames)) refnames = letters[1:nref]
  if (length(refnames)!=nref) stop ("'refnames' must have the same length as 'ref'")
  names(ref) = refnames
  ref
}

## check x
checkx = function(x) {
  if (!is.array(x))  stop("'x' must be a 2D array")
  if (inherits(x, "Image")) x = imageData(x)
  ndim = length(dim(x))
  if (ndim<2||ndim>3)  stop("'x' must be a 2D array")
  if (ndim==3) {
    if (dim(x)[3]>1) stop("'x' must be a 2D array")
    else x = x[,,1]
  }
  x
}

## split an labelled image into list of points
splitObjects = function(x) {
  z = which(as.integer(x)>0L)
  split(z, x[z])
}

.haralickMatrix <- function(x, ref, nc=32) {
  checkCompatibleImages(x, ref)
  rref <- range.default(ref)
  if ( rref[1] < 0 || rref[2] > 1 ) {
    ref[ref<0] = 0
    ref[ref>1] = 1
    warning( "Values in 'ref' have been limited to the range [0,1]" )
  }
  
  res <- .Call(C_haralickMatrix, castImage(x), castImage(ref), as.integer(nc))
  return( res )
}

.haralickFeatures <- function(x, ref, nc=32) {
  validImage(x)
  hm <- .haralickMatrix(x=x, ref=ref, nc=nc)
  if ( is.null(hm) || !(is.array(hm) || is.list(hm)) ) return( NULL )
  do.features <- function(m) {
    if (dim(m)[3]>0) res <- .Call(C_haralickFeatures, m)
    else res = matrix(0, nrow=0, ncol=13) ## no objects
    if ( is.matrix(res) )
      colnames(res) <- c("h.asm", "h.con", "h.cor", "h.var", "h.idm", "h.sav", "h.sva", 
                         "h.sen", "h.ent", "h.dva", "h.den", "h.f12", "h.f13")
    res
  }
  if ( !is.list(hm) ) return( do.features(hm) )
  lapply( hm, do.features )
} 

# Declarations of all generic methods used in the package

# Copyright (c) 2005-2007 Oleg Sklyar

# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public License
# as published by the Free Software Foundation; either version 2.1
# of the License, or (at your option) any later version.

# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.

# See the GNU Lesser General Public License for more details.
# LGPL license wording: http://www.gnu.org/licenses/lgpl.html

# - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -

## image IO, display
setGeneric ("image", function (x, ...) standardGeneric("image") )

## statistics
setGeneric ("hist", function (x, ...) standardGeneric("hist") )

# explicit signature definition to avoid dispatching on arguments other than `...`
setGeneric("abind", signature="...")

medianFilter = function(x, size, cacheSize=512L) {
  size = as.integer(size)
  if (size < 1L) stop("filter radius must be >= 1")
  if (min(dim(x)[1:2]) < 2L * size + 1L) stop("filter radius must not exceed half of the x or y dimension")
  if (any(is.na(x))) stop("'x' shouldn't contain any NAs")
  validImage(x)
  
  .Call(C_medianFilter, castImage(x), size, as.integer(cacheSize))
}

flip <- function (x) {
  validImage(x)
  asub(x, seq.int(dim(x)[2],1), 2)
}

flop <- function (x) {
  validImage(x)
  asub(x, seq.int(dim(x)[1],1), 1)
}

## performs an affine transform on a set of images
affine <- function (x, m, filter = c("bilinear", "none"), output.dim, bg.col = "black", antialias = TRUE) {
  ## check arguments
  validImage(x)
  if ( !is.matrix(m) || !identical(dim(m), c(3L, 2L)) ) stop("'m' must be a 3x2 matrix")
  if ( any(is.na(m)) ) stop("'m' shouldn't contain any NAs")
  
  filter <- switch(match.arg(filter), none=0L, bilinear=1L)
  
  # dimensions of output image
  d = dim(x)  
  if (!missing(output.dim)) {
    if( length(output.dim)!=2L || !is.numeric(output.dim) ) stop("'output.dim' must be a numeric vector of length 2")
    d[1:2] = as.integer(round(output.dim))
  }
  
  ## inverse of the transformation matrix
  ## this is needed because in C we iterate over (x',y') for which we calculate (x,y)
  m <- solve(cbind(m, c(0, 0, 1)))
  
  ## image background
  nf = numberOfFrames(x, "render")
  
  bg <- Image(rep_len(bg.col, nf), c(1L, 1L, d[-c(1,2)]), colorMode(x))
  
  .Call(C_affine, castImage(x), d, castImage(bg), m, filter, isTRUE(antialias))
}

# images are represented in a cooridinate system with the y-axis oriented
# downwards; therefore we perform "counter-clockwise" rotation, which is
# equivalent to clockwise rotation in a regular coordinate system
rotate = function(x, angle, filter = "bilinear", output.dim, output.origin, ...) {
  ## check arguments
  if ( length(angle)!=1L || !is.numeric(angle) ) stop("'angle' must be a number")
  if ( !missing(output.dim) )
    if ( length(output.dim)!=2L || !is.numeric(output.dim) ) stop("'output.dim' must be a numeric vector of length 2")

  ## allow lossless rotation
  if ( (angle %% 90) == 0 ) filter = "none"
  
  angle = angle * pi / 180
  d  = dim(x)[1:2]
  dx = d[1]
  dy = d[2]
  cos = cos(angle)
  sin = sin(angle)
  
  ## if no origin is specified, rotate about center of output image
  if ( missing(output.origin) ) {
    ## calculate new bounding box size
    newdim = c(
      dx * abs(cos) + dy * abs(sin),
      dx * abs(sin) + dy * abs(cos)
    )
    
    ## offset needed to stay within the bounding box
    offset = c(
      dx * max(0, -cos) + dy * max(0,  sin),
      dx * max(0, -sin) + dy * max(0, -cos)
    )
    
    if ( missing(output.dim) )
      output.dim = newdim
    else
      offset = offset + (output.dim - newdim) / 2
    
  } else {
    if ( length(output.origin)!=2L || !is.numeric(output.origin) )
      stop("'output.origin' must be a numeric vector of length 2")
    offset = c(
      output.origin[1L] * (1 - cos) + output.origin[2L] * sin,
      output.origin[2L] * (1 - cos) - output.origin[1L] * sin
    )
  }
  
  m <- matrix(c(cos, -sin, offset[1], 
                sin,  cos, offset[2]), 3L, 2L) 
  
  affine(x = x, m = m, filter = filter, output.dim = output.dim, ...)
}

translate <- function(x, v, filter = "none", ...) {
  ## check arguments
  if ( length(v)!=2L || !is.numeric(v) ) stop("'v' must be a numeric vector of length 2")
  
  m <- matrix(c(1, 0, v[1],
                0, 1, v[2]), 3L, 2L)
  
  affine(x = x, m = m, filter = filter, ...)
}

resize <- function(x, w, h, output.dim = c(w, h), output.origin = c(0, 0), antialias = FALSE, ...) {
  ## check arguments
  if ( missing(h) && missing(w) ) stop("either 'w' or 'h' must be specified")
  if ( length(output.origin)!=2L || !is.numeric(output.origin) ) stop("'output.origin' must be a numeric vector of length 2")
  
  d = dim(x)[1:2]
  if ( missing(w) ) w <- round(h*d[1]/d[2])
  if ( missing(h) ) h <- round(w*d[2]/d[1])
  if ( missing(output.dim) ) output.dim <- c(w, h)
  ratio <- c(w, h)/d
  
  m <- matrix(c(ratio[1], 0, (1-ratio[1]) * output.origin[1],
                0, ratio[2], (1-ratio[2]) * output.origin[2]), 3L, 2L)
  
  affine(x = x, m = m, output.dim = output.dim, antialias = antialias, ...)
}

## transpose spatial dimensions
transpose <- function(x) {
    .Call(C_transpose, x)
}

otsu <- function(x, range = c(0, 1), levels = 256){
  validImage(x)
  if ( colorMode(x) != Grayscale ) stop("Only thresholding of Grayscale images is supported.")
  if ( !is.numeric(range) || length(range) != 2 ) stop("'range' must be a numeric vector of length 2.")
  levels = as.integer(levels)
  if ( is.na(levels) || levels < 1 ) stop("Levels must be at least equal 1.")
  breaks = seq(range[1], range[2], length.out = levels+1)
  x = castImage(x)  # converts logical to numeric
  
# prepare 3D array for the 'apply' function
  dim(x) = c(dim(x)[seq_len(2)], numberOfFrames(x, 'total'))
  
# threshold each frame separately
  apply(x, 3, function(y) {
    h = hist.default(y, breaks = breaks, plot = FALSE)
    counts = as.double(h$counts)
    mids = as.double(h$mids)
    len = length(counts)
    w1 = cumsum(counts)
    w2 = w1[len] + counts - w1
    cm = counts * mids
    m1 = cumsum(cm)
    m2 = m1[len] + cm - m1
    var = w1 * w2 * (m2/w2 - m1/w1)^2
    # find the left- and right-most maximum and return the threshold value in between
    maxi = which(var == max(var, na.rm = TRUE))
    (mids[maxi[1]] + mids[maxi[length(maxi)]] ) /2
  })
}

# 2D convolution-based linear filter for images and matrix data

# Copyright (c) 2007-2015, Andrzej Oleś, Gregoire Pau, Oleg Sklyar

# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public License
# as published by the Free Software Foundation; either version 2.1
# of the License, or (at your option) any later version.

# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.

# See the GNU Lesser General Public License for more details.
# LGPL license wording: http://www.gnu.org/licenses/lgpl.html

## - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -

filter2 = function(x, filter, boundary = c("circular", "replicate")) {
  if ( is.numeric(boundary) ) {
    val = boundary
    boundary = "linear"
  }
  else
    boundary = match.arg(boundary)
  
  validObject(x)
  validObject(filter)
  
  d = dx = dim(x)
  df = dim(filter)
  dnames = dimnames(x)
  
  ## do not enforce odd filter dimensions in the special case when filter size
  ## equals image size
  if ( any(dx[1:2]!=df) || boundary!="circular" ) {
    if (any(df%%2==0)) stop("dimensions of 'filter' matrix must be odd")
    if (any(dx[1:2]<df)) stop("'filter' dimensions cannot exceed dimensions of 'x'")
  }
  
  cf = df%/%2
  of = df%%2
  
  res = x
  
  switch(boundary,
         ## default mode just wraps around edges
         circular = {
           x = imageData(x)
         },
         ## pad with a given value
         linear = {
           dx[1:2] = dx[1:2] + cf[1:2]
           
           if ( length(dx)>2 && length(val)==prod(dx[-(1:2)]) ) {
             # Higher dim array with matching linear boundry values
             xpad = array(rep(val, each=prod(dx[1:2])), dim = dx)
           } else {
             if ( length(val)>1 ) {
               warning('The boundary value length does not match the number of frames, only the first element of boundary will be used')
             }
             xpad = array(val[1], dx)
           }
           # The do.call and arguments as a list of dimensions is to take are of arrays of unknown dimension
           x = do.call("[<-", c(quote(xpad), lapply(d, function(x) enquote(1:x)), quote(x)) )
         },
         replicate = {
           x = imageData(x)
           
           dx[1:2] = dx[1:2] + df[1:2] - 1L
           
           rep.dim <- function(x, dim, index, times) {
             xs <- asub(x, idx=index, dims=dim, drop=FALSE)
             ds <- dim(xs)
             ds[dim] <- times
             if ( dim==2L ) {
               xs <- split(xs, ceiling(seq_along(xs)/ds[1L]))
               xs <- sapply(xs, FUN=rep, times=times)
             } else {
               xs <- rep(xs, each=times)
             }
             array(xs, dim=ds)
           }
           
           # Add left and right colums
           lc <- rep.dim(x, 2L, 1L, cf[2L])
           rc <- rep.dim(x, 2L, d[2L], cf[2L])
           x <- abind::abind(x, rc, lc, along=2L)
           # Add top and bottom rows
           tr <- rep.dim(x, 1L, 1L, cf[1L])
           br <- rep.dim(x, 1L, d[1L], cf[1L])
           x <- abind::abind(x, br, tr, along=1L)
         }
  )
  
  ## create fft filter matrix
  wf = matrix(0.0, nrow=dx[1L], ncol=dx[2L])
  
  wf[c(if (cf[1L]>0L) (dx[1L]-cf[1L]+1L):dx[1L] else NULL, 1L:(cf[1L]+of[1L])), 
     c(if (cf[2L]>0L) (dx[2L]-cf[2L]+1L):dx[2L] else NULL, 1L:(cf[2L]+of[2L]))] = filter
  
  wf = fftw2d(wf)
  
  pdx = prod(dx[1:2])
  
  .filter = function(xx) Re(fftw2d(fftw2d(xx)*wf, inverse=1)/pdx)
  
  ## convert to a frame-based 3D array
  if ( length(dx) > 2L ) {    
    y = apply(x, 3:length(dx), .filter)
    dim(y) = dx
  }
  else {
    y = .filter(x)
  }
  
  if ( boundary!="circular" ) {
    y = asub(y, list(1:d[1L], 1:d[2L]), 1:2)
  }
  
  dimnames(y) = dnames
  
  imageData(res) <- y
  res
}

## - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
watershed = function (x, tolerance=1, ext=1) {
  validImage(x)
  tolerance = as.numeric(tolerance)
  if (tolerance<0) stop( "'tolerance' must be non-negative" )
  ext = as.integer(ext)
  if (ext<1) stop( "'ext' must be a positive integer" )
  .Call(C_watershed, castImage(x), tolerance, ext)
}

## - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
propagate = function (x, seeds, mask=NULL, lambda=1e-4) {
  validImage(x)
  checkCompatibleImages(x, seeds)
  
  if (!is.integer(seeds))
    storage.mode(seeds) = 'integer'

  if (!is.null(mask)) {
    checkCompatibleImages(x, mask)
    if (!is.integer(mask))
      storage.mode(mask) = 'integer'
  }

  lambda = as.numeric(lambda)
  if (lambda<0.0) stop("'lambda' must be positive" )

  return(.Call(C_propagate, castImage(x), seeds, mask, lambda))
}

## - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
ocontour = function(x) {
  validImage(x)
  if(!is.integer(x)) storage.mode(x) = 'integer'
  y = .Call(C_ocontour, x)
  names(y) = seq_along(y)
  y = y[sapply(y, length)>0] # remove missing objects
}

## - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
bwlabel = function(x) {
  validImage(x)
  .Call(C_bwlabel, x)
}

## - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
colorLabels = function(x, normalize = TRUE){
  len = length( (d = dim(x)) )
  
  res <- x
  
  # linearize image data for convenient processing
  dim(res) = c(prod(d[1:2]), if(len>2) prod(d[3:len]) else 1)
  
  f = function(y, m) {
    idx <- y > 0
    y[idx] <- sample(m)[y[idx]]
    y
  }
  
  tmp = apply(res, 2, function(y) {
    m = max(y)
    replicate(3, f(y, m))
  })
  
  # restore proper dimensions
  dim(tmp) = c(d[1:2], 3, if(len>2) d[3:len] else NULL)
  
  if ( is.Image(x) ) {
    imageData(res) <- tmp
    colorMode(res) <- Color
  }
  else {
    res = new("Image", .Data = tmp, colormode = Color)
  }
  
  if (normalize) normalize(res) else res
}

\name{floodFill}

\alias{floodFill}

\title{Region filling}

\description{
  Fill regions in images.
}

\usage{
floodFill(x, pts, col, tolerance=0)
}

\arguments{
  \item{x}{An \code{Image} object or an array.}

  \item{pts}{Coordinates of the start filling points given as either of the following: a vector of the form \code{c(x1, y1, x2, y2, ...)}, a list of points, a matrix or data frame where rows represent points and columns are the x and y coordinates. For image stacks different points for each frame can be specified by providing them in a list of length matching the number of 'render' frames.}

  \item{col}{Fill color. This argument should be a numeric for Grayscale images and an R color for Color images. Values are recycled such that their length matches the number of points in \code{pts}. Can be a list of length matching the number of 'render' frames similarly as \code{pts}.}

  \item{tolerance}{Color tolerance used during the fill.}
}

\value{
  An \code{Image} object or an array, containing the transformed version
  of \code{x}.
}

\details{
  Flood fill is performed using the fast scan line algorithm. Filling
  starts at \code{pts} and grows in connected areas where the absolute
  difference of the pixels intensities (or colors) remains below
  \code{tolerance}.
}

\author{
  Gregoire Pau, Oleg Sklyar; 2007
}

\examples{
  x = readImage(system.file("images", "shapes.png", package="EBImage"))

  ## fill a shape with 50% shade of gray
  y = floodFill(x, c(67, 146), 0.5)
  display(y)
  
  ## fill with color
  y = toRGB(y)
  y = floodFill(y, c(48, 78), 'orange')	
  display(y)
  
  ## fill multiple shapes with different colors
  y = y[110:512,1:130,]
  points = rbind(c(50, 50), c(100, 50), c(150, 50))
  colors = c("red", "green", "blue")
  y = floodFill(y, points, colors)
  display(y)
  
  ## area fill
  x = readImage(system.file("images", "sample.png", package="EBImage"))
  y = floodFill(x, rbind(c(200, 400), c(200, 325)), 1, tolerance=0.1)
  display(y)
  
  ## application to image stacks
  f = system.file("images", "nuclei.tif", package="EBImage")
  x = readImage(f)[1:250,1:250,]
  x = opening(thresh(x, 12, 12), makeBrush(5, shape='disc'))
  xy = lapply(getFrames(bwlabel(x)), function(x) computeFeatures.moment(x)[,1:2])
  y = floodFill(toRGB(x), xy, c("red", "green", "blue"))
  display(y)
}

\name{thresh}

\alias{thresh}

\title{Adaptive thresholding}

\description{
  Thresholds an image using a moving rectangular window.
}

\usage{
thresh(x, w=5, h=5, offset=0.01)
}

\arguments{
  \item{x}{An \code{Image} object or an array.}
  
  \item{w, h}{Half width and height of the moving rectangular window.} 
  
  \item{offset}{Thresholding offset from the averaged value.}
}

\value{
  An \code{Image} object or an array, containing the transformed version
  of \code{x}.
}

\details{
  This function returns the binary image resulting from the comparison
  between an image and its filtered version with a rectangular window.
  It is equivalent of doing 
  \code{\{
  f = matrix(1, nc=2*w+1, nr=2*h+1); 
  f = f/sum(f); 
  x > (filter2(x, f, boundary="replicate") + offset)
  \}} 
  but faster. The function \code{filter2} provides hence more 
  flexibility than \code{thresh}.
}

\seealso{
  \code{filter2}
}

\author{
  Oleg Sklyar, \email{osklyar@ebi.ac.uk},  2005-2007
}

\examples{
  x = readImage(system.file('images', 'nuclei.tif', package='EBImage'))
  display(x)
  y = thresh(x, 10, 10, 0.05)
  display(y)
}

\keyword{manip}

\name{channel}

\alias{channel}
\alias{rgbImage}
\alias{toRGB}

\concept{color conversion}
\concept{color mode}

\title{Color and image color mode conversions}

\description{
  \code{channel} handles color space conversions between image modes.
  \code{rgbImage} combines \code{Grayscale} images into a \code{Color} one.
  \code{toRGB} is a wrapper function for convenient grayscale to RGB color space conversion; the call \code{toRGB(x)} returns the result of \code{channel(x, 'rgb')}.
}

\usage{ 
channel(x, mode)
rgbImage(red, green, blue)
toRGB(x)
}

\arguments{
  \item{x}{An \code{Image} object or an array.}
  
  \item{mode}{ A character value specifying the target mode for
    conversion. See Details.}
  
  \item{red, green, blue}{\code{Image} objects in \code{Grayscale}
    color mode or arrays of the same dimension. If missing, a
    black image will be used.} 
}

\value{
  An \code{Image} object or an array.
}

\details{
  Conversion modes:
  \describe{
    \item{\code{rgb}}{Converts a \code{Grayscale} image or an array
    into a \code{Color} image, replicating RGB channels.}

  \item{\code{gray, grey}}{Converts a \code{Color} image into a
    \code{Grayscale} image, using uniform 1/3 RGB weights.}
    
  \item{\code{luminance}}{Luminance-preserving \code{Color} to \code{Grayscale} conversion
  using CIE 1931 luminance weights: 0.2126 * R + 0.7152 * G + 0.0722 * B.}

  \item{\code{red, green, blue}}{Extracts the \code{red}, \code{green} or
    \code{blue} channel from a \code{Color} image. Returns a
    \code{Grayscale} image.}

  \item{\code{asred, asgreen, asblue}}{Converts a \code{Grayscale}
    image or an array into a \code{Color} image of the specified hue.}
}

NOTE: \code{channel} changes the pixel intensities, unlike \code{colorMode}
which just changes the way that EBImage renders an image.
}

\seealso{
  \code{\link{colorMode}}
} 

\examples{
 x = readImage(system.file("images", "shapes.png", package="EBImage"))
 display(x)
 y = channel(x, 'asgreen')
 display(y)

 ## rgbImage
 x = readImage(system.file('images', 'nuclei.tif', package='EBImage'))
 y = readImage(system.file('images', 'cells.tif', package='EBImage'))
 display(x, title='Cell nuclei')
 display(y, title='Cell bodies')

 cells = rgbImage(green=1.5*y, blue=x)
 display(cells, title='Cells')
}

\author{
  Oleg Sklyar, \email{osklyar@ebi.ac.uk}
}

\name{transpose}

\alias{transpose}

\title{Image Transposition}

\description{
  Transposes an image by swapping its spatial dimensions.
}

\usage{
transpose(x)
}

\arguments{
  \item{x}{an \code{Image} object or an array.}
}

\value{
  A transformed version of \code{x} with its first two dimensions transposed.
}

\details{
The transposition of an image is performed by swapping the X and Y indices of its array representation.
}

\note{
The function is implemented using an efficient cash-oblivious algorithm which is typically faster than R's \code{aperm} and \code{t} functions.
}

\author{
  Andrzej Oles, \email{andrzej.oles@embl.de}, 2012-2017
}

\seealso{
  \code{\link{flip}}, \code{\link{flop}}, \code{\link{rotate}}
}

\examples{
  x = readImage(system.file("images", "sample-color.png", package="EBImage"))
  y = transpose(x)
  
  display(x, title='Original')
  display(y, title='Transposed')

  ## performing the transposition of an image twice should result in the original image
  z = transpose(y)
  identical(x, z)
}

\name{EBImage-defunct}

\alias{EBImage-defunct}
\alias{animate}
\alias{blur}
\alias{drawtext}
\alias{drawfont}
\alias{getFeatures}
\alias{getNumberOfFrames}
\alias{hullFeatures}
\alias{zernikeMoments}
\alias{edgeProfile}
\alias{edgeFeatures}
\alias{haralickFeatures}
\alias{haralickMatrix}
\alias{moments}
\alias{rmoments}
\alias{smoments}
\alias{cmoments}
\alias{dilateGreyScale}
\alias{erodeGreyScale}
\alias{openingGreyScale}
\alias{closingGreyScale}
\alias{whiteTopHatGreyScale}
\alias{blackTopHatGreyScale}
\alias{selfcomplementaryTopHatGreyScale}

\title{Defunct functions in package \sQuote{EBImage}}

\description{These functions are defunct and no longer available.}

\details{
  The following functions are defunct and no longer available; use
  the replacement indicated below.
  
  \itemize{
    \item{\code{animate}: \code{\link{display}}}
    \item{\code{blur}: \code{\link{gblur}}}
    \item{\code{drawtext}: see package vignette for documentation on how to add text labels to images}
    \item{\code{drawfont}: see package vignette for documentation on how to add text labels to images}
    \item{\code{getFeatures}: \code{\link{computeFeatures}}}
    \item{\code{getNumberOfFrames}: \code{\link{numberOfFrames}}}
    \item{\code{hullFeatures}: \code{\link{computeFeatures.shape}}}
    \item{\code{zernikeMoments}: \code{\link{computeFeatures}}}
    \item{\code{edgeProfile}: \code{\link{computeFeatures}}}
    \item{\code{edgeFeatures}: \code{\link{computeFeatures.shape}}}
    \item{\code{haralickFeatures}: \code{\link{computeFeatures}}}
    \item{\code{haralickMatrix}: \code{\link{computeFeatures}}}
    \item{\code{moments}: \code{\link{computeFeatures.moment}}}
    \item{\code{cmoments}: \code{\link{computeFeatures.moment}}}
    \item{\code{rmoments}: \code{\link{computeFeatures.moment}}}
    \item{\code{smoments}: \code{\link{computeFeatures.moment}}}
    \item{\code{dilateGreyScale}: \code{\link{dilate}}}
    \item{\code{erodeGreyScale}: \code{\link{erode}}}
    \item{\code{openingGreyScale}: \code{\link{opening}}}
    \item{\code{closingGreyScale}: \code{\link{closing}}}
    \item{\code{whiteTopHatGreyScale}: \code{\link{whiteTopHat}}}
    \item{\code{blackTopHatGreyScale}: \code{\link{blackTopHat}}}
    \item{\code{selfcomplementaryTopHatGreyScale}: \code{\link{selfComplementaryTopHat}}}
  }
}

\keyword{package}

\name{bwlabel}

\alias{bwlabel}

\title{Binary segmentation}

\description{
  Labels connected (connected sets) objects in a binary image.
}

\usage{
bwlabel(x)
}

\arguments{
  \item{x}{An \code{Image} object or an array. \code{x} is considered as
    a binary image, whose pixels of value 0 are considered as background
    ones and other pixels as foreground ones.}
}

\value{
  A \code{Grayscale} \code{Image} object or an array, containing the
  labelled version of \code{x}.
}

\details{
  All pixels for each connected set of foreground (non-zero) pixels
  in \code{x} are set to an unique increasing integer, starting from 1.
  Hence, \code{max(x)} gives the number of connected objects in \code{x}.
}

\author{
  Gregoire Pau, 2009
}

\seealso{
  \code{\link{computeFeatures}}, \code{\link{propagate}}, \code{\link{watershed}}, \code{\link{paintObjects}}, \code{\link{colorLabels}}
}

\examples{
  ## simple example
  x = readImage(system.file('images', 'shapes.png', package='EBImage'))
  x = x[110:512,1:130]
  display(x, title='Binary')
  y = bwlabel(x)
  display(normalize(y), title='Segmented')

  ## read nuclei images
  x = readImage(system.file('images', 'nuclei.tif', package='EBImage'))
  display(x)

  ## computes binary mask
  y = thresh(x, 10, 10, 0.05)
  y = opening(y, makeBrush(5, shape='disc'))
  display(y, title='Cell nuclei binary mask')

  ## bwlabel
  z = bwlabel(y)
  display(normalize(z), title='Cell nuclei')
  nbnuclei = apply(z, 3, max)
  cat('Number of nuclei=', paste(nbnuclei, collapse=','),'\n')

  ## paint nuclei in color
  cols = c('black', sample(rainbow(max(z))))
  zrainbow = Image(cols[1+z], dim=dim(z))
  display(zrainbow, title='Cell nuclei (recolored)')
}

\name{propagate}

\alias{propagate}

\concept{object detection}

\title{Voronoi-based segmentation on image manifolds}

\description{
Find boundaries between adjacent regions in an image, where seeds
have been already identified in the individual regions to be
segmented. The method finds the Voronoi region of each seed on
a manifold with a metric controlled by local image properties.
The method is motivated by the problem of finding the borders of
cells in microscopy images, given a labelling of the nuclei
in the images.

Algorithm and implementation are from Jones et al. [1].
}

\usage{
propagate(x, seeds, mask=NULL, lambda=1e-4)
}

\arguments{
   \item{x}{An \code{Image} object or an array, containing the image to segment.}

   \item{seeds}{An \code{Image} object or an array, containing the seeding objects of
   the already identified regions.}	

   \item{mask}{An optional \code{Image} object or an array, containing
   the binary image mask of the regions that can be segmented. If missing, 
   the whole image is segmented.}

   \item{lambda}{A numeric value. The regularization parameter used in the
   metric, determining the trade-off between the Euclidean distance in the
   image plane and the contribution of the gradient of \code{x}. See details.}
}

\value{
  An \code{Image} object or an array, containing the labelled objects.
}

\details{
  The method operates by computing a discretized approximation of the
  Voronoi regions for given seed points on a Riemann manifold with a
  metric controlled by local image features.

  Under this metric, the infinitesimal distance d between points
  v and v+dv is defined by: 
  \preformatted{d^2 = ( (t(dv)*g)^2 + lambda*t(dv)*dv )/(lambda + 1) },
  where g is the gradient of image \code{x} at point v.

  \code{lambda} controls the weight of the Euclidean distance term. 
  When \code{lambda} tends to infinity, d tends to the Euclidean
  distance. When \code{lambda} tends to 0, d tends to the intensity
  gradient of the image.

  The gradient is
  computed on a neighborhood of 3x3 pixels.

  Segmentation of the Voronoi regions in the vicinity of flat areas
  (having a null gradient) with small values of \code{lambda} can
  suffer from artifacts coming from the metric approximation.
}

\seealso{ \code{\link{bwlabel}}, \code{\link{watershed}}
}

\examples{
  ## a paraboloid mountain in a plane
  n = 400
  x = (n/4)^2 - matrix(
	(rep(1:n, times=n) - n/2)^2 + (rep(1:n, each=n) - n/2)^2,
	nrow=n, ncol=n)
  x = normalize(x)

  ## 4 seeds
  seeds = array(0, dim=c(n,n))
  seeds[51:55, 301:305] = 1
  seeds[301:305, 101:105] = 2
  seeds[201:205, 141:145] = 3
  seeds[331:335, 351:355] = 4

  lambda = 10^seq(-8, -1, by=1)
  segmented = Image(dim=c(dim(x), length(lambda)))

  for(i in seq_along(lambda)) {
    prop = propagate(x, seeds, lambda=lambda[i])
    prop = prop/max(prop)
    segmented[,,i] = prop
  }

  display(x, title='Image')
  display(seeds/max(seeds), title='Seeds')
  display(segmented, title="Voronoi regions", all=TRUE)
  
}

\author{
  The original CellProfiler code is from Anne Carpenter <carpenter@wi.mit.edu>,
  Thouis Jones <thouis@csail.mit.edu>, In Han Kang <inthek@mit.edu>.
  Responsible for this implementation: Greg Pau.
}

\section{License}{
  The implementation is based on CellProfiler C++ source code [2, 3].
  An LGPL license was granted by Thouis Jones to use this part of
  CellProfiler's code for the \code{propagate} function.
}

\references{
   [1] T. Jones, A. Carpenter and P. Golland,
    "Voronoi-Based Segmentation of Cells on Image Manifolds",
    CVBIA05 (535-543), 2005

   [2] A. Carpenter, T.R. Jones, M.R. Lamprecht, C. Clarke, I.H. Kang,
    O. Friman, D. Guertin, J.H. Chang, R.A. Lindquist, J. Moffat,
    P. Golland and D.M. Sabatini, "CellProfiler: image analysis software
    for identifying and quantifying cell phenotypes", Genome Biology 2006, 7:R100

   [3] CellProfiler: http://www.cellprofiler.org
}

\name{stackObjects}

\alias{stackObjects}

\concept{image manipulation}

\title{Places detected objects into an image stack}

\description{
  Places detected objects into an image stack.
}

\usage{
stackObjects(x, ref, combine=TRUE, bg.col='black', ext)
}

\arguments{
  \item{x}{An \code{Image} object or an array containing object masks.
    Object masks are sets of pixels with the same unique integer value.}
    
  \item{ref}{An \code{Image} object or an array, containing the
    intensity values of the objects.}

  \item{combine}{If \code{x} contains multiple images, specifies if the resulting
    list of image stacks with individual objects should be combined
    using \code{combine} into a single image stack.}

  \item{bg.col}{Background pixel color.}

  \item{ext}{A numeric controlling the size of the output image. 
   If missing, \code{ext} is estimated from data. See details.}
}

\value{
  An \code{Image} object containing the stacked objects contained in
  \code{x}. If \code{x} contains multiple images and if \code{combine}
  is \code{TRUE}, \code{stackObjects} returns a list of \code{Image}
  objects.  
}

\details{
  \code{stackObjects} creates a set of \code{n} images of size
  (\code{2*ext+1}, \code{2*ext+1}), where \code{n} is the number of objects
  in \code{x}, and places each object of \code{x} in this set.
  
  If not specified, \code{ext} is estimated using the 98\% quantile of
  m.majoraxis/2, where \code{m.majoraxis} is the semi-major axis
  descriptor extracted from \code{computeFeatures.moment}, taken over
  all the objects of the image \code{x}.
}

\seealso{ \code{\link{combine}}, \code{\link{tile}}, \code{\link{computeFeatures.moment}} 
}

\examples{
  ## simple example
  x = readImage(system.file('images', 'shapes.png', package='EBImage'))
  x = x[110:512,1:130]
  y = bwlabel(x)
  display(normalize(y), title='Objects')
  z = stackObjects(y, normalize(y))
  display(z, title='Stacked objects')

  ## load images
  nuc = readImage(system.file('images', 'nuclei.tif', package='EBImage'))
  cel = readImage(system.file('images', 'cells.tif', package='EBImage'))
  img = rgbImage(green=cel, blue=nuc)
  display(img, title='Cells')

  ## segment nuclei
  nmask = thresh(nuc, 10, 10, 0.05)
  nmask = opening(nmask, makeBrush(5, shape='disc'))
  nmask = fillHull(bwlabel(nmask))

  ## segment cells, using propagate and nuclei as 'seeds'
  ctmask = opening(cel>0.1, makeBrush(5, shape='disc'))
  cmask = propagate(cel, nmask, ctmask)

  ## using paintObjects to highlight objects
  res = paintObjects(cmask, img, col='#ff00ff')
  res = paintObjects(nmask, res, col='#ffff00')
  display(res, title='Segmented cells')

  ## stacked cells
  st = stackObjects(cmask, img)
  display(st, title='Stacked objects')
}

\author{
  Oleg Sklyar, \email{osklyar@ebi.ac.uk}, 2006-2007
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/displayWidget.R
\name{display-shiny}
\alias{display-shiny}
\alias{displayOutput}
\alias{renderDisplay}
\title{Shiny Bindings for display}
\usage{
displayOutput(outputId, width = "100\%", height = "500px")

renderDisplay(expr, env = parent.frame(), quoted = FALSE)
}
\arguments{
\item{outputId}{output variable to read from}

\item{width, height}{Must be a valid CSS unit (like \code{'100\%'},
\code{'400px'}, \code{'auto'}) or a number, which will be coerced to a
string and have \code{'px'} appended.}

\item{expr}{An expression that generates the image viewer (typicall through a call to \code{display})}

\item{env}{The environment in which to evaluate \code{expr}.}

\item{quoted}{Is \code{expr} a quoted expression (with \code{quote()})? This
is useful if you want to save an expression in a variable.}
}
\description{
Output and render functions for using the interactive image viewer within Shiny
applications and interactive R Markdown documents.
}

\seealso{
  \code{\link{display}}
}

\examples{
# Only run this example in interactive R sessions
if (interactive()) {
  options(device.ask.default = FALSE)

  require("shiny")
  
  ui <- fluidPage(
  
    # Application title
    titlePanel("Image display"),
  
    # Sidebar with a select input for the image
    sidebarLayout(
      sidebarPanel(
        selectInput("image", "Sample image:", list.files(system.file("images", package="EBImage")))
      ),
  
      # Show a plot of the generated distribution
      mainPanel(
        tabsetPanel(
          tabPanel("Static raster", plotOutput("raster")),
          tabPanel("Interactive browser", displayOutput("widget"))
        )
      )
    )
  
  )
  
  server <- function(input, output) {
  
    img <- reactive({
      f = system.file("images", input$image, package="EBImage")
      readImage(f)
    })
  
    output$widget <- renderDisplay({
      display(img())
    })
  
    output$raster <- renderPlot({
      plot(img(), all=TRUE)
    })
  
  }
  
  # Run the application
  shinyApp(ui = ui, server = server)
}
}

\name{clahe}

\alias{clahe}

\title{Contrast Limited Adaptive Histogram Equalization}

\description{
  Improve contrast locally by performing adaptive histogram equalization.
}

\usage{
clahe(x, nx = 8, ny = nx, bins = 256, limit = 2, keep.range = FALSE) 
}

\arguments{
  \item{x}{an \code{Image} object or an array.}
  \item{nx}{integer, number of contextual regions in the X direction (min 2, max 256)}
  \item{ny}{integer, number of contextual regions in the Y direction (min 2, max 256)}
  \item{bins}{integer, number of greybins for histogram ("dynamic range"). Smaller values (eg. 128) speed up processing while still producing good quality output.}
  \item{limit}{double, normalized clip limit (higher values give more contrast).  A clip limit smaller than 0 results in standard (non-contrast limited) AHE.}
  \item{keep.range}{logical, retain image minimum and maximum values rather then use the full available range}
}

\value{
  An \code{Image} object or an array, containing the filtered version
  of \code{x}.
}

\details{
  Adaptive histogram equalization (AHE) is a contrast enhancement 
  technique which overcomes the limitations of standard histogram equalization.
  Unlike ordinary histogram equalization the adaptive method redistributes the lightness values 
  of the image based on several histograms, each corresponding to a distinct section of the image.
  It is therefore useful for improving the local contrast and enhancing the definitions of edges in each region of an image.
  However, AHE has a tendency to overamplify noise in relatively homogeneous 
  regions of an image. Contrast limited adaptive histogram equalization (CLAHE) prevents this by limiting the amplification.
  
  The function is based on the implementation by Karel Zuiderveld [1].
  This implementation assumes that the X- and Y image dimensions are an integer
  multiple of the X- and Y sizes of the contextual regions.
  The input image \code{x} should contain pixel values in the range from 0 to 1, 
  inclusive; values lower than 0 or higher than 1 are clipped before applying 
  the filter. Internal processing is performed in 16-bit precision.
  If the image contains multiple channels or frames, 
  the filter is applied to each one of them separately.
}

\note{
  The interpolation step of the original implementation by Karel Zuiderveld [1] 
  was modified to use double precision arithmetic in order to make the filter 
  rotationally invariant for even-sized contextual regions, and the result is 
  properly rounded rather than truncated towards 0 in order to avoid a 
  systematic shift of pixel values.
}

\seealso{
  \code{\link{equalize}}
}

\author{
  Andrzej Oleś, \email{andrzej.oles@embl.de}, 2017
}

\examples{
  x = readImage(system.file("images", "sample-color.png", package="EBImage"))
  y = clahe(x)  
  display(y)
}

\references{
  [1] K. Zuiderveld: Contrast Limited Adaptive Histogram Equalization. In: P. Heckbert: Graphics Gems IV, Academic Press 1994
}

\name{equalize}

\alias{equalize}

\concept{equalization}

\title{Histogram Equalization}

\description{
  Equalize the image histogram to a specified range and number of levels.
}

\usage{
equalize(x, range = c(0, 1), levels = 256)
}

\arguments{
  \item{x}{an \code{Image} object or an array}

  \item{range}{numeric vector of length 2, the output range of the equalized histogram}

  \item{levels}{number of grayscale levels (Grayscale images) or intensity levels of each channel (Color images)}
}

\value{
  An \code{Image} object or an array, containing the transformed version
  of \code{x}.
}

\details{
 Histogram equalization is an adaptive image contrast adjustment method. It flattens the image histogram by performing linearization of the cumulative distribution function of pixel intensities.

 Individual channels of \code{Color} images and frames of image stacks are equalized separately.
}

\seealso{
  \code{\link{clahe}}
}

\author{
  Andrzej Oles, \email{andrzej.oles@embl.de}, 2014
}

\examples{
  x = readImage(system.file('images', 'cells.tif', package='EBImage'))
  hist(x)
  y = equalize(x)
  hist(y)
  display(y, title='Equalized Grayscale Image')

  x = readImage(system.file('images', 'sample-color.png', package='EBImage'))
  hist(x)
  y = equalize(x)
  hist(y)
  display(y, title='Equalized Grayscale Image')
}

\name{combine}

\alias{combine}
\alias{combine,array,array-method}
\alias{combine,matrix,matrix-method}
\alias{combine,Image,Image-method}
\alias{combine,list,missing-method}
\alias{combine,ANY,NULL-method}
\alias{combine,NULL,ANY-method}
\alias{combine,NULL,NULL-method}

\title{Combine images}

\description{
  Merges images to create image sequences.
}

\usage{
combine(x, y, ...)
}


\arguments{
   \item{x}{An \code{Image} object, an array, or a list containing \code{Image} objects and arrays.}
   \item{y}{An \code{Image} object or an array.}
   \item{...}{\code{Image} objects or arrays.}
}

\value{
  An \code{Image} object or an array.
}

\details{
  The function \code{combine} uses \code{\link[abind]{abind}} to merge multi-dimensional
  arrays along the dimension depending on the
  color mode of \code{x}. If \code{x} is a \code{Grayscale} image or an array,
  image objects are combined along the third dimension, whereas when
  \code{x} is a \code{Color} image they are combined along the fourth dimension, leaving room on the third dimension for color
  channels.
}

\seealso{
  The method \code{\link{abind}} provides a more flexible interface which allows to specify the dimension along which to combine the images.
}

\author{
  Gregoire Pau, Andrzej Oles, 2013
}

\examples{
  ## combination of color images
  img = readImage(system.file("images", "sample-color.png", package="EBImage"))[257:768,,]
  x = combine(img, flip(img), flop(img))
  display(x, all=TRUE)

  ## Blurred images
  x = resize(img, 128, 128)
  xt = list()
  for (t in seq(0.1, 5, length.out=9)) xt=c(xt, list(gblur(x, s=t)))
  xt = combine(xt)
  display(xt, title='Blurred images', all=TRUE)
}

\name{drawCircle}

\alias{drawCircle}

\title{Draw a circle on an image.}

\description{
  Draw a circle on an image.
}

\usage{
drawCircle(img, x, y, radius, col, fill=FALSE, z=1)
}

\arguments{
\item{img}{An \code{Image} object or an array.}  
\item{x, y, radius}{numerics indicating the center and the radius
    of the circle.}
  
\item{col}{A numeric or a character string specifying the color
    of the circle.}
  
\item{fill}{A logical indicating whether the circle should be filled.
    Default is \code{FALSE}.}
  
\item{z}{A numeric indicating on which frame of the image the circle
    should be drawn. Default is 1.}
}

\value{
  An \code{Image} object or an array, containing the transformed version
  of \code{img}.
}

\author{
  Gregoire Pau, 2010
}

\examples{
  ## Simple white circle
  x = matrix(0, nrow=300, ncol=300)
  y = drawCircle(x, 100, 200, 47, col=1)
  display(y)
  
  ## Simple filled yellow circle
  x = channel(y, 'rgb')
  y = drawCircle(x, 200, 140, 57, col='yellow', fill=TRUE)
  display(y)
}

\name{otsu}

\alias{otsu}

\title{Calculate Otsu's threshold}

\description{
Returns a threshold value based on Otsu's method, which can be then used to reduce the grayscale image to a binary image.
}

\usage{
otsu(x, range = c(0, 1), levels = 256)
}

\arguments{
  \item{x}{A \code{Grayscale} \code{Image} object or an array.}
  
  \item{range}{Numeric vector of length 2 specifying the histogram range used for thresholding.}
  
  \item{levels}{Number of grayscale levels.}
}

\value{
A vector of length equal to the total number of frames in \code{x}. Each vector element contains the Otsu's threshold value calculated for the corresponding image frame.
}

\details{
Otsu's thresholding method [1] is useful to automatically perform clustering-based image thresholding. The algorithm assumes that the distribution of image pixel intensities follows a bi-modal histogram, and separates those pixels into two classes (e.g. foreground and background). 
The optimal threshold value is determined by minimizing the combined intra-class variance.

The threshold value is calculated for each image frame separately resulting in a output vector of length equal to the total number of frames in the image.

The default number of \code{levels} corresponds to the number of gray levels of an 8bit image. It is recommended to adjust this value according to the bit depth of the processed data, i.e. set \code{levels} to 2^16 = 65536 when working with 16bit images.
}

\seealso{
  \code{\link{thresh}}
}

\references{
   [1] Nobuyuki Otsu, "A threshold selection method from gray-level histograms". IEEE Trans. Sys., Man., Cyber. 9 (1): 62-66. doi:10.1109/TSMC.1979.4310076 (1979)
}

\author{
  Philip A. Marais \email{philipmarais@gmail.com}, Andrzej Oles \email{andrzej.oles@embl.de}, 2014
}

\examples{
  x = readImage(system.file("images", "sample.png", package="EBImage"))
  display(x)
  
  ## threshold using Otsu's method
  y = x > otsu(x)
  display(y)
}

\keyword{manip}

\name{normalize}

\alias{normalize}
\alias{normalize,array-method}
\alias{normalize,matrix-method}
\alias{normalize,Image-method}

\concept{normalization}

\title{Intensity values linear scaling}

\description{
  Linearly scale the intensity values of an image to a specified range.
}

\usage{
\S4method{normalize}{Image}(object, separate=TRUE, ft=c(0,1), inputRange)%

\S4method{normalize}{array}(object, separate=TRUE, ft=c(0,1), inputRange)
}

\arguments{
  \item{object}{an \code{Image} object or an array}

  \item{separate}{if \code{TRUE}, normalizes each frame separately}

  \item{ft}{a numeric vector of 2 values, target minimum and maximum 
    intensity values after normalization}
    
  \item{inputRange}{a numeric vector of 2 values, sets the range of the input
    intensity values; values exceeding this range are clipped}
}

\value{
  An \code{Image} object or an array, containing the transformed version
  of \code{object}.
}

\details{
  \code{normalize} performs linear interpolation of the intensity values of an image to the specified range \code{ft}. If \code{inputRange} is not set the whole dynamic range of the image is used as input. By specifying \code{inputRange} the input intensity range of the image can be limited to [min, max]. Values exceeding this range are clipped, i.e. intensities lower/higher than \code{min}/\code{max} are set to \code{min}/\code{max}.
}

\author{
  Oleg Sklyar, \email{osklyar@ebi.ac.uk}, 2006-2007
  Andrzej Oles, \email{andrzej.oles@embl.de}, 2013
}

\examples{
  x = readImage(system.file('images', 'shapes.png', package='EBImage'))
  x = x[110:512,1:130]
  y = bwlabel(x)
  display(x, title='Original')

  print(range(y))
  y = normalize(y)
  print(range(y))
  display(y, title='Segmented')
}

\name{resize}

\alias{flip}
\alias{flop}
\alias{resize}
\alias{rotate}
\alias{translate}
\alias{affine}

\concept{transformation}
\concept{rotation}
\concept{resize}
\concept{mirror}

\title{Spatial linear transformations}

\description{
  The following functions perform all spatial linear transforms: reflection, rotation,
  translation, resizing, and general affine transform.
}

\usage{
flip(x)
flop(x)
rotate(x, angle, filter = "bilinear", output.dim, output.origin, ...)
translate(x, v, filter = "none", ...)
resize(x, w, h, output.dim = c(w, h), output.origin = c(0, 0), antialias = FALSE, ...)

affine(x, m, filter = c("bilinear", "none"), output.dim, bg.col = "black", antialias = TRUE)
}

\arguments{
  \item{x}{An \code{Image} object or an array.}
  \item{angle}{A numeric specifying the image rotation angle in degrees.}
  \item{v}{A vector of 2 numbers denoting the translation vector in pixels.}
  \item{w, h}{Width and height of the resized image. One of these arguments
    can be missing to enable proportional resizing.}  
  \item{filter}{A character string indicating the interpolating sampling filter.
                Valid values are 'none' or 'bilinear'. See Details.}
  \item{output.dim}{A vector of 2 numbers indicating the dimension of the output image. 
                    For \code{affine} and \code{translate} the default is \code{dim(x)}, for \code{resize} it equals \code{c(w, h)}, and for \code{rotate} it defaults to the bounding box size of the rotated image.}
  \item{output.origin}{A vector of 2 numbers indicating the output coordinates of the origin in pixels.}
  
  \item{m}{A 3x2 matrix describing the affine transformation. See Details.}
  \item{bg.col}{Color used to fill the background pixels, defaults to "black". In the case of multi-frame images the value is recycled, and individual background for each frame can be specified by providing a vector.}
  \item{antialias}{If \code{TRUE}, perform bilinear sampling at image edges using \code{bg.col}.}
  \item{...}{Arguments to be passed to \code{affine}, such as \code{filter}, \code{output.dim}, \code{bg.col} or \code{antialias}.}  
}
 
\value{
  An \code{Image} object or an array, containing the transformed version
  of \code{x}.
}

\details{
  \code{flip} mirrors \code{x} around the image horizontal axis (vertical reflection).
  
  \code{flop} mirrors \code{x} around the image vertical axis (horizontal reflection).

  \code{rotate} rotates the image clockwise by the given angle around the 
  origin specified in \code{output.origin}. If no \code{output.origin} is 
  provided, the result will be centered in a recalculated bounding box unless
  \code{output.dim} is provided.
 
  \code{resize} scales the image \code{x} to the desired dimensions.
  The transformation origin can be specified in \code{output.origin}. 
  For example, zooming about the \code{output.origin} can be achieved by setting
  \code{output.dim} to a value different from \code{c(w, h)}.

  \code{affine} returns the affine transformation of \code{x}, where
  pixels coordinates, denoted by the matrix \code{px}, are
  transformed to \code{cbind(px, 1)\%*\%m}. 

  All spatial transformations except \code{flip} and \code{flop} are based on the 
  general \code{affine} transformation. Spatial interpolation can be either
  \code{none}, also called nearest neighbor, where the resulting pixel value equals to
  the closest pixel value, or \code{bilinear}, where the new 
  pixel value is computed by bilinear approximation of the 4 neighboring pixels. The
  \code{bilinear} filter gives smoother results.
}

\seealso{ 
  \code{\link{transpose}}
}

\author{
  Gregoire Pau, 2012
}

\examples{
  x <- readImage(system.file("images", "sample.png", package="EBImage"))
  display(x)

  display( flip(x) )
  display( flop(x) ) 
  display( resize(x, 128) )
  display( rotate(x, 30) )
  display( translate(x, c(120, -20)) )

  m <- matrix(c(0.6, 0.2, 0, -0.2, 0.3, 300), nrow=3)
  display( affine(x, m) )
}

\name{fillHull}

\alias{fillHull}

\title{Fill holes in objects}

\description{
  Fill holes in objects.
}

\usage{
fillHull(x)
}

\arguments{
  \item{x}{An \code{Image} object or an array.}
}

\value{
  An \code{Image} object or an array, containing the transformed version
  of \code{x}.
}

\details{
  \code{fillHull} fills holes in the objects defined in \code{x}, where
   objects are sets of pixels with the same unique integer value.
}

\seealso{
  \code{\link{bwlabel}}
}

\author{
  Gregoire Pau, Oleg Sklyar; 2007
}

\examples{
  x = readImage(system.file('images', 'nuclei.tif', package='EBImage'))
  display(x)

  y = thresh(x, 10, 10, 0.05)
  display(y, title='Cell nuclei')

  y = fillHull(y)
  display(y, title='Cell nuclei without holes')
}

\name{colormap}

\alias{colormap}

\title{Map a Greyscale Image to Color}

\description{
  Maps a greyscale image to color using a color palette.
}

\usage{
colormap(x, palette = heat.colors(256L))
}

\arguments{
  \item{x}{an \code{Image} object of color mode \code{Grayscale}, or an array}

  \item{palette}{character vector containing the color palette}
}

\details{
  The \code{colormap} function first linearly maps the pixel intensity values
  of \code{x} to the integer range \code{1:length(palette)}. It then 
  uses these values as indices to the provided color palette to create
  a color version of the original image.
  
  The default palette contains 256 colors, which is the typical number of
  different shades in a 8bit grayscale image.
}

\value{
  An \code{Image} object of color mode \code{Color}, containing the color-mapped version
  of \code{x}.
}

\author{
  Andrzej Oleś, \email{andrzej.oles@embl.de}, 2016
}

\examples{
  x = readImage(system.file("images", "sample.png", package="EBImage"))

  ## posterize an image using the topo.colors palette
  y = colormap(x, topo.colors(8))

  display(y, method="raster")

  ## mimic MatLab's 'jet.colors' colormap
  jet.colors = colorRampPalette(c("#00007F", "blue", "#007FFF", "cyan", "#7FFF7F", "yellow", "#FF7F00", "red", "#7F0000"))
  y = colormap(x, jet.colors(256))

  display(y, method="raster")
}

\name{io}

\alias{readImage}
\alias{writeImage}

\concept{image IO}

\title{Image I/O}

\description{
  Read images from files and URLs, and write images to files.
}

\usage{
readImage(files, type, all = TRUE, names = sub("\\\\.[^.]*$", "", basename(files)), \dots)
writeImage(x, files, type, quality = 100, bits.per.sample, compression = "none", \dots)
}

\arguments{
  \item{files}{a character vector of file names or URLs.}
  \item{type}{image type (optional). Supported values are: \code{jpeg}, \code{png}, and \code{tiff}. If missing, file format is automatically determined by file name extension.}
  \item{all}{logical: when the file contains more than one image should all frames be read, or only the first one?}
  \item{names}{a character vector used for frame names. Should have the same length as files.}
  \item{x}{an \code{Image} object or an array.} 
  \item{bits.per.sample}{a numeric scalar specifying the number of bits per sample (only for \code{tiff} files). Supported values are 8 and 16.}
  \item{compression}{the desired compression algorithm (only for \code{tiff} files). For a list of supported values consult the documentation of the \code{\link{writeTIFF}} function from the \pkg{tiff} package.}
  \item{quality}{a numeric ranging from 1 to 100 (default) controlling the quality of the JPEG output.}
  \item{\dots}{arguments passed to the corresponding functions from the \pkg{jpeg}, \pkg{png}, and \pkg{tiff} packages.}
}

\value{
  \code{readImage} returns a new \code{Image} object.

  \code{writeImage} returns an invisible vector of file names.
}

\details{
\code{readImage} loads all images from the \code{files} vector and returns them stacked into a single \code{Image} object containing an array of doubles ranging from 0 (black) to 1 (white). All images need to be of the same \code{type} and have the same dimensions and color mode. If \code{type} is missing, the appropriate file format is determined from file name extension. Color mode is determined automatically based on the number of channels. When the function fails to read an image it skips to the next element of the \code{files} vector issuing a warning message. Non-local files can be read directly from a valid URL.

  \code{writeImage} writes images into files specified by \code{files}, were the number of \code{files} needs to be equal 1 or the number of frames. Given an image containing multiple frames and a single file name either the whole stack is written into a single TIFF file, or each frame is saved to an individual JPEG/PNG file (for \code{files = "image.*"} frames are saved into \code{image-X.*} files, where \code{X} equals the frame number less one; for an image containing \code{n} frames this results in file names numbered from 0 to \code{n-1}). 

  When writing JPEG files the compression quality can be specified using \code{quality}. Valid values range from 100 (highest quality) to 1 (lowest quality). For TIFF files additional information about the desired number of bits per sample (\code{bits.per.sample}) and the compression algorithm (\code{compression}) can be provided. For a complete list of supported values please consult the documentation of the \pkg{tiff} package.
}

\note{
  Image formats have a limited dynamic range (e.g. JPEG: 8 bit, TIFF: 16 bit) and \code{writeImage} may cause some loss of accuracy. In specific, writing 16 bit image data to formats other than TIFF will strip the 8 LSB. When writing TIFF files a dynamic range check is performed and an appropriate value of \code{bits.per.sample} is set automatically.
}

\seealso{
  \code{\link{Image}}, \code{\link{display}}, \code{\link{readJPEG}}/\code{\link{writeJPEG}},  \code{\link{readPNG}}/\code{\link{writePNG}}, \code{\link{readTIFF}}/\code{\link{writeTIFF}}
}

\author{
  Andrzej Oles, \email{andrzej.oles@embl.de}, 2012
}

\examples{
  ## Read and display an image
  f = system.file("images", "sample-color.png", package="EBImage")
  x = readImage(f)
  display(x)

  ## Read and display a multi-frame TIFF
  y = readImage(system.file("images", "nuclei.tif", package="EBImage"))
  display(y)

  ## Read an image directly from a remote location by specifying its URL
  try({
    im = readImage("http://www-huber.embl.de/EBImage/ExampleImages/berlin.tif")
    display(im, title = "Berlin Impressions")
  })

  ## Convert a PNG file into JPEG
  tempfile = tempfile("", , ".jpeg")
  writeImage(x, tempfile, quality = 85)
  cat("Converted '", f, "' into '", tempfile, "'.\n", sep="")

  ## Save a frame sequence
  files = writeImage(y, tempfile("", , ".jpeg"), quality = 85)
  cat("Files created: ", files, sep="\n")
}

\name{abind}

\docType{methods}

\alias{abind}
\alias{abind-methods}
\alias{abind,ANY-method}
\alias{abind,Image-method}
\alias{abind,list-method}

\title{Combine Image Arrays}

\description{
  Methods for function \code{\link[abind]{abind}} from package \pkg{abind} useful for combining \code{Image} arrays.
}

\section{Usage}{
\code{abind(...)}
}

\section{Arguments}{
  \describe{
    \item{\code{\ldots}}{Arguments to \code{\link[abind]{abind}}}
  }
}

\section{Methods}{
\describe{
\item{\code{signature(... = "Image")}}{
 This method is defined primarily for the sake of preserving the class of the combined \code{Image} objects. Unlike the original \code{\link[abind]{abind}} function, if \code{dimnames} for all combined objects are \code{NULL} it does not introduce a list of empty \code{dimnames} for each dimension.
}
\item{\code{signature(... = "ANY")}}{
  Dispatches to the original \code{\link[abind]{abind}} function.
}
}}

\value{
  An \code{Image} object or an array, containing the combined data arrays of the input objects.
}

\seealso{
   \code{\link{combine}} provides a more convenient interface to merging images into an image sequence. Use \code{\link{tile}} to lay out images next to each other in a regular grid.
}

\author{
  Andrzej Oleś, \email{andrzej.oles@embl.de}, 2017
}

\examples{
  f = system.file("images", "sample-color.png", package="EBImage")
  x = readImage(f)
  
  ## combine images horizontally
  y = abind(x, x, along=1)
  display(y)
  
  ## stack images one on top of the other
  z = abind(x, x, along=2)
  display(z)
}

\keyword{methods}

\name{paintObjects}

\alias{paintObjects}

\title{Mark objects in images}

\description{
  Higlight objects in images by outlining and/or painting them.
}

\usage{
paintObjects(x, tgt, opac=c(1, 1), col=c('red', NA), thick=FALSE, closed=FALSE)
}

\arguments{
   \item{x}{An \code{Image} object in \code{Grayscale} color mode or an
    array containing object masks. Object masks are sets of pixels
    with the same unique integer value.}
    
  \item{tgt}{An \code{Image} object or an array, containing the
    intensity values of the objects.}

  \item{opac}{A numeric vector of two opacity values for drawing object
    boundaries and object bodies. Opacity ranges from 0 to 1, with 0
    being fully transparent and 1 fully opaque.}

  \item{col}{A character vector of two R colors for drawing object
    boundaries and object bodies. By default, object boundaries are
    painted in \code{red} while object bodies are not painted.}
    
  \item{thick}{A logical indicating whether to use thick boundary contours. Default is \code{FALSE}.}
  
  \item{closed}{A logical indicating whether object contours should be closed along image edges or remain open.}
}

\value{
  An \code{Image} object or an array, containing the painted version of \code{tgt}.
}

\seealso{
  \code{\link{bwlabel}}, \code{\link{watershed}}, \code{\link{computeFeatures}}, \code{\link{colorLabels}}
}

\examples{
  ## load images
  nuc = readImage(system.file('images', 'nuclei.tif', package='EBImage'))
  cel = readImage(system.file('images', 'cells.tif', package='EBImage'))
  img = rgbImage(green=cel, blue=nuc)
  display(img, title='Cells')

  ## segment nuclei
  nmask = thresh(nuc, 10, 10, 0.05)
  nmask = opening(nmask, makeBrush(5, shape='disc'))
  nmask = fillHull(nmask)
  nmask = bwlabel(nmask)
  display(normalize(nmask), title='Cell nuclei mask')

  ## segment cells, using propagate and nuclei as 'seeds'
  ctmask = opening(cel>0.1, makeBrush(5, shape='disc'))
  cmask = propagate(cel, nmask, ctmask)
  display(normalize(cmask), title='Cell mask')

  ## using paintObjects to highlight objects
  res = paintObjects(cmask, img, col='#ff00ff')
  res = paintObjects(nmask, res, col='#ffff00')
  display(res, title='Segmented cells')
}

\author{
  Oleg Sklyar, \email{osklyar@ebi.ac.uk}, 2006-2007
  Andrzej Oles, \email{andrzej.oles@embl.de}, 2015
}

\name{display}

\alias{display}
\alias{plot.Image}

\concept{display}

\title{Image Display}

\description{
  Display images in an interactive JavaScript viewer or using R's built-in graphics capabilities.
}

\usage{
display(x, method, ...)

\method{plot}{Image}(x, ...)
}

\arguments{
  \item{x}{an \code{Image} object or an array.}
  \item{method}{the way of displaying images. Defaults to \code{"browser"} when R is used interactively, and to \code{"raster"} otherwise. The default behavior can be overridden by setting \code{options("EBImage.display")}. See Details.}
  \item{\dots}{arguments to be passed to the specialized display functions; for details see the sections on individual display methods.}
}

\details{
The default \code{method} used for displaying images depends on whether called from and interactive R session. If \code{interactive()} is \code{TRUE} images are displayed with the \code{"browser"} method, otherwise the \code{"raster"} method is used. This dynamic behavior can be overridden by setting \code{options("EBImage.display")} to either \code{"browser"} or \code{"raster"}.

\code{plot.Image} S3 method is a wrapper for \code{display(..., method="raster")}
}

\section{"browser" method}{
The \code{"browser"} method runs an interactive JavaScript image viewer. A list of available features along with corresponding mouse and keyboard actions is shown by pressing 'h'. This method takes the following additional arguments.
\describe{
  \item{\code{embed}}{logical(1), include images in the document as data URIs. Defaults to \code{TRUE} in non-interactive context (e.g. static R Markdown documents), otherwise to \code{FALSE}.}
  
  \item{\code{tempDir}}{character(1), file path for storing any temporary image files. Defaults to \code{tempfile("")}}
  
  \item{\code{\dots}}{arguments passed to \code{\link{createWidget}}, such as fixed \code{width} and \code{height} (in CSS units), \code{elementId}, or \code{preRenderHook}.}
  }
}

\section{"raster" method}{
The \code{"raster"} method displays images as R raster graphics. The user coordinates of the plotting region are set to the image pixel coordinates with the origin \code{(0, 0)} in the upper left corner.

By default only the first frame of an image stack is shown; a different \code{frame} can also be specified. When \code{all=TRUE} the whole image stack is rendered and the frames are automatically positioned next to each other in a grid. The grid layout can be modified through \code{nx} and \code{spacing} and \code{margin}.

This method provides to following additional arguments to \code{display}.
  \describe{
    \item{\code{interpolate}}{a logical vector (or scalar) indicating whether to apply linear interpolation to the image when drawing.}
    \item{\code{frame}}{a numeric indicating the frame number to display; only effective when \code{all = FALSE}.}
    \item{\code{all}}{logical, defaulting to \code{FALSE}. If set to \code{TRUE}, all frames of a stacked image are displayed arranged in a grid, otherwise (default) just a single frame specified in \code{frame} is displayed. The grid layout can be controlled by \code{nx}, \code{spacing} and \code{margin}.}
    \item{\code{drawGrid}}{a logical indicating whether to draw grid lines between individual frames. Defaults to \code{TRUE} unless \code{spacing} is non-zero. Line color, type and width can be specified through graphical parameters \code{col}, \code{lty} and \code{lwd}, respectively; see \code{\link{par}} for details.}
    \item{\code{nx}}{integer. Specifies the number images in a row. Negative numbers are interpreted as the number of images in a column, e.g. use \code{-1} to display a single row containing all the frames.}
    \item{\code{spacing}}{numeric. Specifies the separation between frames as a fraction of frame dimensions (positive numbers <1) or in pixels (numbers >=1). It can be either a single number or a vector of length 2, in which case its elements correspond to the horizontal and vertical spacing, respectively.}
    \item{\code{margin}}{numeric. Specifies the outer margins around the image, or a grid of images. Similarly as for \code{spacing}, different horizontal and vertical margins can be defined by providing a vector.}
    \item{\code{\dots}}{graphical parameters passed to \code{\link{par}}}
  }
}

\author{
  Andrzej Oles, \email{andrzej.oles@embl.de}, 2012-2017
}

\seealso{
  \code{\link{display-shiny}}
}

\examples{
## Display a single image
x = readImage(system.file("images", "sample-color.png", package="EBImage"))[257:768,,]
display(x)

## Display a thresholded sequence ...
y = readImage(system.file("images", "sample.png", package="EBImage"))[366:749, 58:441]
z = lapply(seq(from=0.5, to=5, length.out=6),
  function(s) gblur(y, s, boundary="replicate")
)
z = combine(z)

## ... using the interactive viewer ...
display(z)

## ... or using R's build-in raster device
display(z, method = "raster", all = TRUE)

## Display the last frame 
display(z, method = "raster", frame = numberOfFrames(z, type = "render"))

## Customize grid appearance
display(z, method = "raster", all = TRUE,
        nx = 2, spacing = 0.05, margin = 20, bg = "black")
}

\name{rmObjects}

\alias{rmObjects}
\alias{reenumerate}

\concept{delete objects}

\title{Object removal and re-indexation}

\description{
  The \code{rmObjects} functions deletes objects from an image
  by setting their pixel intensity values to 0.
  \code{reenumerate} re-enumerates all objects in an image from 0 (background)
  to the actual number of objects.
}

\usage{
  rmObjects(x, index, reenumerate = TRUE)

  reenumerate(x)
}

\arguments{
 \item{x}{An \code{Image} object in \code{Grayscale} color mode or an
    array containing object masks. Object masks are sets of pixels
    with the same unique integer value.}

  \item{index}{A numeric vector (or a list of vectors if \code{x} contains multiple frames) 
    containing the indexes of objects to remove in the frame.}
    
  \item{reenumerate}{Logical, should all the objects in the image be re-indexed afterwards (default).}
}

\value{
 An \code{Image} object or an array, containing the new objects.
}

\seealso{ 
  \code{\link{bwlabel}}, \code{\link{watershed}}
}

\examples{
  ## make objects
  x = readImage(system.file('images', 'shapes.png', package='EBImage'))
  x = x[110:512,1:130]
  y = bwlabel(x)
  
  ## number of objects found
  max(y)
  
  display(normalize(y), title='Objects')
  
  ## remove every second letter
  objects = list(
    seq.int(from = 2, to = max(y), by = 2),
    seq.int(from = 1, to = max(y), by = 2)
    )
  z = rmObjects(combine(y, y), objects)
  
  display(normalize(z), title='Object removal')
  
  ## the number of objects left in each image
  apply(z, 3, max)
  
  ## perform object removal without re-enumerating
  z = rmObjects(y, objects, reenumerate = FALSE)
  
  ## labels of objects left
  unique(as.vector(z))[-1L]
  
  ## re-index objects
  z = reenumerate(z)
  unique(as.vector(z))[-1L]
}

\author{
  Oleg Sklyar, \email{osklyar@ebi.ac.uk}, 2006-2007
}

\name{Image}

\alias{Image}
\alias{Image-class}
\alias{Grayscale}
\alias{Color}
\alias{is.Image}
\alias{as.Image}
\alias{as.array.Image}
\alias{as.raster.Image}
\alias{print.Image}
\alias{median.Image}
\alias{quantile.Image}
\alias{colorMode}
\alias{colorMode<-}
\alias{getFrame}
\alias{getFrames}
\alias{numberOfFrames}
\alias{imageData}
\alias{imageData<-}
\alias{colormode}
\alias{Ops,Image,Image-method}
\alias{Ops,numeric,Image-method}
\alias{Ops,Image,numeric-method}
\alias{Math2,Image-method}
\alias{[,Image,ANY,ANY,ANY-method}
\alias{[,Image-method}
\alias{show,Image-method}
\alias{image,Image-method}
\alias{hist,Image-method}
\alias{log,Image-method}

\title{Image class}

\description{
  \code{EBImage} uses the \code{Image} class to store and process
  images. Images are stored as multi-dimensional arrays containing the pixel
  intensities. \code{Image} extends the base class \code{array} and
  uses the \code{colormode} slot to store how the color information of
  the multi-dimensional data is handled.

  The \code{colormode} slot can be either \code{Grayscale} or \code{Color}.
  In either mode, the first two dimensions of the underlying array are understood to be the spatial dimensions of the image.
  In the \code{Grayscale} mode the remaining dimensions contain other image frames.
  In the \code{Color} mode, the third dimension contains color channels of the image, while higher dimensions contain  image frames.
  The number of channels is not limited and can be any number >= 1; these can be, for instance, the red, green, blue and, possibly, alpha channel.
  Note that grayscale images containing an alpha channel are stored with \code{colormode=Color}.
  
  All methods from the \code{EBImage} package work either with \code{Image} objects or
  multi-dimensional arrays. In the latter case, the color mode is assumed to be \code{Grayscale}.
}

\usage{
Image(data, dim, colormode)
as.Image(x)
is.Image(x)

\method{as.array}{Image}(x, \ldots)
\method{as.raster}{Image}(x, max = 1, i = 1L, \ldots)

colorMode(y)
colorMode(y) <- value

imageData(y)
imageData(y) <- value

getFrame(y, i, type = c('total', 'render'))
getFrames(y, i, type = c('total', 'render'))
numberOfFrames(y, type = c('total', 'render'))
}

\arguments{
  \item{data}{A vector or array containing the pixel intensities of an image. If missing, the default 1x1
  zero-filled array is used.}

  \item{dim}{A vector containing the final dimensions of an \code{Image} object. If missing, equals to
  \code{dim(data)}.}

  \item{colormode}{A numeric or a character string containing the color mode which can be
  either \code{Grayscale} or \code{Color}. If missing, equals to \code{Grayscale}.}

  \item{x}{An R object.}

  \item{y}{An \code{Image} object or an array.}
  
  \item{max}{Number giving the maximum of the color values range.}

  \item{i}{Number(s) of frame(s). A single number in case of \code{getFrame}, or a vector of frame numbers for \code{getFrames}. If missing all frames are returned.}

  \item{value}{For \code{colorMode}, a numeric or a character string containing the color mode which
    can be either \code{Grayscale} or \code{Color}. For \code{imageData}, an \code{Image} object or an array.}

  \item{type}{A character string containing \code{total} or \code{render}. Default is \code{total}.}
  
  \item{\dots}{further arguments passed to or from other methods.}
}

\value{
  \code{Image} and \code{as.Image} return a new \code{Image} object.

  \code{is.Image} returns TRUE if \code{x} is an \code{Image} object and FALSE otherwise.

  \code{as.raster} coerces an Image object to its raster representation. For stacked images the \code{i}-th frame is returned (by default the first one).

  \code{colorMode} returns the color mode of \code{y} and \code{colorMode<-} changes the color mode
  of \code{y}.

  \code{imageData} returns the array contained in an \code{Image} object.
}

\details{
  Depending on \code{type}, \code{numberOfFrames} returns the total number of frames contained
  in the object \code{y} or the number of rendered frames. The total number of frames is independent
  of the color mode and equals to the product of all the dimensions except the two first ones. The
  number of rendered frames is equal to the total number of frames in the \code{Grayscale} color mode, or
  to the product of all the dimensions except the three first ones in the \code{Color} color mode.

  \code{getFrame} returns the i-th frame contained in the image \code{y}. If \code{type} is \code{total}, the
  function is unaware of the color mode and returns an xy-plane. For \code{type=render}, the function returns the
  i-th image as shown by the \code{display} function.
}

\seealso{
  \code{\link{readImage}}, \code{\link{writeImage}}, \code{\link{display}}
}

\author{
  Oleg Sklyar, \email{osklyar@ebi.ac.uk}, 2005-2007
}

\examples{
  s1 = exp(12i*pi*seq(-1, 1, length.out=300)^2)
  y = Image(outer(Im(s1), Re(s1)))
  display(normalize(y))

  x = Image(rnorm(300*300*3),dim=c(300,300,3), colormode='Color')
  display(x)

  w = matrix(seq(0, 1, length.out=300), nc=300, nr=300)
  m = abind::abind(w, t(w), along=3)
  z = Image(m, colormode='Color')
  display(normalize(z))

  y = Image(matrix(c('red', 'violet', '#ff51a5', 'yellow'), nrow=10, ncol=10))
  display(y, interpolate=FALSE)

  ## colorMode example
  x = readImage(system.file('images', 'nuclei.tif', package='EBImage'))
  x = x[,,1:3]
  display(x, title='Cell nuclei')
  colorMode(x) = Color
  display(x, title='Cell nuclei in RGB')
}

\name{medianFilter}

\alias{medianFilter}

\title{2D constant time median filtering}

\description{
  Process an image using Perreault's modern constant-time median filtering algorithm [1, 2].
}

\usage{
medianFilter(x, size, cacheSize=512)
}

\arguments{
  \item{x}{an \code{Image} object or an array.}
  \item{size}{integer, median filter radius.}
  \item{cacheSize}{integer, the L2 cache size of the system CPU in kB.}
}

\value{
  An \code{Image} object or an array, containing the filtered version
  of \code{x}.
}

\details{
  Median filtering is useful as a smoothing technique, e.g. in the removal
  of speckling noise.
  
  For a filter of radius \code{size}, the median kernel is a \code{2*size+1}
  times \code{2*size+1} square.
  
  The input image \code{x} should contain pixel values in the range from 0 to 1, 
  inclusive; values lower than 0 or higher than 1 are clipped before applying 
  the filter. Internal processing is performed using 16-bit precision. The 
  behavior at image boundaries is such as the source image has been padded with
  pixels whose values equal the nearest border pixel value.
  
  If the image contains multiple channels or frames, 
  the filter is applied to each one of them separately.
}

\seealso{
  \code{\link{gblur}}
}

\author{
  Joseph Barry, \email{joseph.barry@embl.de}, 2012

  Andrzej Oleś, \email{andrzej.oles@embl.de}, 2016
}

\examples{
  x = readImage(system.file("images", "nuclei.tif", package="EBImage"))
  display(x, title='Nuclei')
  y = medianFilter(x, 5)  
  display(y, title='Filtered nuclei')
}

\references{
  [1] S. Perreault and P. Hebert, "Median Filtering in Constant Time", IEEE Trans Image Process 16(9), 2389-2394, 2007
  
  [2] \url{http://nomis80.org/ctmf.html}
}

\name{filter2}

\alias{filter2}

\title{2D Convolution Filter}

\description{
  Filters an image using the fast 2D FFT convolution product.
}

\usage{
filter2(x, filter, boundary = c("circular", "replicate"))
}

\arguments{
  \item{x}{An \code{Image} object or an array.}

  \item{filter}{An \code{Image} object or an array, with odd spatial
    dimensions. Must contain only one frame.}
    
  \item{boundary}{Behaviour at image borders. The default is to wrap the image around borders. For other modes see details.}
}

\value{
  An \code{Image} object or an array, containing the filtered version
  of \code{x}.
}

\details{
  Linear filtering is useful to perform low-pass filtering (to blur
  images, remove noise...) and high-pass filtering (to detect
  edges, sharpen images). The function \code{makeBrush} is useful to
  generate filters.

  The default \code{"circular"} behaviour at boundaries is to wrap the image around borders.
  In the \code{"replicate"} mode pixels outside the bounds of the image are assumed to equal the nearest border pixel value.
  Numeric values of \code{boundary} yield linear convolution by padding the image with the given value(s).
  
  If \code{x} contains multiple frames, the filter is applied separately to each frame.
  
}

\seealso{
  \code{\link{makeBrush}}, \code{\link{convolve}}, \code{\link{fft}}, \code{\link{blur}}
}

\author{
  Andrzej Oleś, Gregoire Pau
}

\examples{
  x = readImage(system.file("images", "sample-color.png", package="EBImage"))
  display(x, title='Sample')

  ## Low-pass disc-shaped filter
  f = makeBrush(21, shape='disc', step=FALSE)
  display(f, title='Disc filter')
  f = f/sum(f)
  y = filter2(x, f)
  display(y, title='Filtered image')

  ## Low-pass filter with linear padded boundary
  y = filter2(x, f, boundary=c(0,.5,1))
  display(y, title='Filtered image with linear padded boundary')

  ## High-pass Laplacian filter
  la = matrix(1, nc=3, nr=3)
  la[2,2] = -8
  y = filter2(x, la)
  display(y, title='Filtered image')
  
  ## High-pass Laplacian filter with replicated boundary
  y = filter2(x, la, boundary='replicate')
  display(y, title='Filtered image with replicated boundary')
}

\name{distmap}

\alias{distmap}

\title{Distance map transform}

\description{
  Computes the distance map transform of a binary image.
  The distance map is a matrix which contains for each pixel
  the distance to its nearest background pixel.
}

\usage{
distmap(x, metric=c('euclidean', 'manhattan'))
}

\arguments{
  \item{x}{An \code{Image} object or an array.
    \code{x} is considered as a binary image, whose pixels of value 0
    are considered as background ones and other pixels as
    foreground ones.}

  \item{metric}{A character indicating which metric to use, L1 distance (\code{manhattan}) 
  or L2 distance (\code{euclidean}). Default is \code{euclidean}.} 
}

\value{
  An \code{Image} object or an array, with pixels
  containing the distances to the nearest background points. 
}

\details{
  A fast algorithm of complexity O(M*N*log(max(M,N))), where (M,N) are the
  dimensions of \code{x}, is used to compute the distance map.
}

\references{M. N. Kolountzakis, K. N. Kutulakos. Fast Computation of the Euclidean
  Distance Map for Binary Images, Infor. Proc. Letters 43 (1992).}

\author{
  Gregoire Pau, \email{gpau@ebi.ac.uk}, 2008
}

\examples{
  x = readImage(system.file("images", "shapes.png", package="EBImage"))
  display(x)
  dx = distmap(x)
  display(dx/10, title='Distance map of x')
}

\name{gblur}

\alias{gblur}

\title{Low-pass Gaussian filter	}

\description{
  Filters an image with a low-pass Gaussian filter.	
}

\usage{
gblur(x, sigma, radius = 2 * ceiling(3 * sigma) + 1, ...)
}

\arguments{
  \item{x}{An \code{Image} object or an array.}

  \item{sigma}{A numeric denoting the standard deviation of the Gaussian filter
    used for blurring.}

  \item{radius}{The radius of the filter in pixels. Default is \code{2*ceiling(3*sigma)+1)}.}
  
  \item{...}{Arguments passed to \code{\link{filter2}}.}
}

\value{
  An \code{Image} object or an array, containing the filtered version
  of \code{x}.
}

\details{
The Gaussian filter is created with the function \code{makeBrush}.	
}

\seealso{
  \code{\link{filter2}},  \code{\link{makeBrush}}
}

\author{
  Oleg Sklyar, \email{osklyar@ebi.ac.uk}, 2005-2007
}

\examples{
  x = readImage(system.file("images", "sample.png", package="EBImage"))
  display(x)

  y = gblur(x, sigma=8)
  display(y, title='gblur(x, sigma=8)')
}

\name{localCurvature}

\alias{localCurvature}

\title{Local Curvature}

\description{
  Computes signed curvature along a line.
}

\usage{
localCurvature(x, h, maxk)
}

\arguments{
  \item{x}{A data frame or matrix of dimensions N x 2 containing the coordinates 
    of the line, where N is the number of points. The points should be ordered according 
    to their position on the line. The columns should contain the x and y coordinates. 
    The curvature calculation is unaffected by any permutation of the columns.
    Directly accepts a list element from \code{ocontour}.}
  \item{h}{Specifies the length of the smoothing window. See \code{locfit::lp} for more details.}
  \item{maxk}{See \code{locfit::locfit.raw} for details.}
}

\value{
  Returns a \code{list} containing the contour coordinates \code{x}, the signed curvature at each point \code{curvature} 
  and the arc length of the contour \code{length}.
}

\details{
 \code{localCurvature} fits a local non-parametric smoothing line (polynomial of degree 2) 
 at each point along the line segment, and computes the curvature locally using numerical derivatives.
}

\author{
  Joseph Barry, Wolfgang Huber, 2013
}

\seealso{
  \code{\link{ocontour}}
}

\examples{
  ## curvature goes as the inverse of the radius of a circle
  range=seq(3.5,33.5,by=2)
  plotRange=seq(0.5,33.5,length.out=100)
  circleRes=array(dim=length(range))
  names(circleRes)=range
  for (i in  seq_along(1:length(range))) {
   y=as.Image(makeBrush('disc', size=2*range[i]))
   y=ocontour(y)[[1]]
   circleRes[i]=abs(mean(localCurvature(x=y,h=range[i])$curvature, na.rm=TRUE))
  }
  plot(range, circleRes, ylim=c(0,max(circleRes, na.rm=TRUE)), xlab='Circle Radius', ylab='Curvature', type='p', xlim=range(plotRange))
  points(plotRange, 1/plotRange, type='l')

  ## Calculate curvature
  x = readImage(system.file("images", "shapes.png", package="EBImage"))[25:74, 60:109]
  x = resize(x, 200)
  y = gblur(x, 3) > .3
  display(y)
  
  contours = ocontour(bwlabel(y))
  c = localCurvature(x=contours[[1]], h=11)
  i = c$curvature >= 0
  pos = neg = array(0, dim(x))
  pos[c$contour[i,]+1]  = c$curvature[i]
  neg[c$contour[!i,]+1] = -c$curvature[!i]
  display(10*(rgbImage(pos, , neg)), title = "Image curvature")
}

\name{ocontour}

\alias{ocontour}
\title{Oriented contours}

\description{
  Computes the oriented contour of objects.
}

\usage{
ocontour(x)
}

\arguments{
  \item{x}{An \code{Image} object or an array, containing objects.
    Only integer values are considered. Pixels of value 0 constitute
    the background. Each object is a set of pixels with the same unique
    integer value. Objects are assumed connected.
  }
}

\value{
  A list of matrices, containing the coordinates of object oriented contours.
}

\author{
  Gregoire Pau, \email{gpau@ebi.ac.uk}, 2008
}

\examples{
  x = readImage(system.file("images", "shapes.png", package="EBImage"))
  x = x[1:120,50:120]
  display(x)
  oc = ocontour(x)
  plot(oc[[1]], type='l')
  points(oc[[1]], col=2)
}

\name{computeFeatures}

\alias{computeFeatures}
\alias{computeFeatures.basic}
\alias{computeFeatures.shape}
\alias{computeFeatures.moment}
\alias{computeFeatures.haralick}
\alias{standardExpandRef}
\alias{gblob}
  
\title{Compute object features}

\description{
  Computes morphological and texture features from image objects.
}

\usage{
computeFeatures(x, ref, methods.noref=c("computeFeatures.moment", "computeFeatures.shape"),
  methods.ref=c("computeFeatures.basic", "computeFeatures.moment", "computeFeatures.haralick"),
  xname="x", refnames, properties=FALSE, expandRef=standardExpandRef, ...)
  
computeFeatures.basic(x, ref, properties=FALSE, basic.quantiles=c(0.01, 0.05, 0.5, 0.95, 0.99), xs, ...)
computeFeatures.shape(x, properties=FALSE, xs, ...)
computeFeatures.moment(x, ref, properties=FALSE, xs, ...)
computeFeatures.haralick(x, ref , properties=FALSE, haralick.nbins=32, haralick.scales=c(1, 2), xs, ...)

standardExpandRef(ref, refnames, filter = gblob())
}

\arguments{
  \item{x}{An \code{Image} object or an array containing labelled objects.
    Labelled objects are pixel sets with the same unique integer value.}
  \item{ref}{A matrix or a list of matrices, containing the
    intensity values of the reference objects.}
  \item{methods.noref}{A character vector containing the function names
    to be called to compute features without reference intensities. Default is
    \code{computeFeatures.moment} and \code{computeFeatures.shape}.}
 \item{methods.ref}{A character vector containing the function names
   to be called to compute features with reference intensities. Default is
   \code{computeFeatures.basic}, \code{computeFeatures.moment} and
   \code{computeFeatures.haralick}.}
   \item{xname}{A character string naming the object layer. Default is
     \code{x}.}
   \item{refnames}{A character vector naming the reference intensity
     layers. Default are the names of \code{ref}, if present. If not,
     reference intensity layers are named using lower-case letters.}
   \item{properties}{A logical. If \code{FALSE}, the default, the
    function returns the feature matrix. If \code{TRUE}, the function
    returns feature properties.}
  \item{expandRef}{A function used to expand the reference
    images. Default is \code{standardExpandRef}. See Details.}
  \item{basic.quantiles}{A numerical vector indicating the quantiles to
    compute.}
  \item{haralick.nbins}{An integer indicating the number of bins using
    to compute the Haralick matrix. See Details.}
  \item{haralick.scales}{A integer vector indicating the number of
    scales to use to compute the Haralick features.}
  \item{xs}{An optional temporary object created by
    \code{computeFeatures} used for performance considerations.}
  \item{filter}{The filter applied to reference images using \code{\link{filter2}} in order to add granulometry.}
  \item{...}{Optional arguments passed to the feature computation functions.}
}

\value{
  If \code{properties} if \code{FALSE} (by default), \code{computeFeatures}
  returns a matrix of n cells times p features, where p depends of
  the options given to the function. Returns \code{NULL} if no object is
  present.

  If \code{properties} if \code{TRUE}, \code{computeFeatures}
  returns a matrix of p features times 2 properties (translation and
  rotation invariance). Feature properties are useful to filter out
  features that may not be needed for specific tasks, e.g. cell
  position when doing cell classification.
}

\details{
  Features are named x.y.f, where x is the object layer, y the reference
  image layer and f the feature name. Examples include \code{cell.dna.mean},
  indicating mean DNA intensity computed in the cell or
  \code{nucleus.tubulin.cx}, indicating the x center of mass of tubulin
  computed in the nucleus region.

  The function \code{computeFeatures} computes sets of
  features. Features are organized in 4 sets, each computed by a
  different function. The function \code{computeFeatures.basic}
  computes spatial-independent statistics on pixel intensities:
  \itemize{
    \item b.mean: mean intensity
    \item b.sd: standard deviation intensity
    \item b.mad: mad intensity
    \item b.q*: quantile intensity
  }
  
  The function \code{computeFeatures.shape} computes features that
  quantify object shape:
  \itemize{
    \item s.area: area size (in pixels)
    \item s.perimeter: perimeter (in pixels)
    \item s.radius.mean: mean radius (in pixels)
    \item s.radius.sd: standard deviation of the mean radius (in pixels)
    \item s.radius.max: max radius (in pixels)
    \item s.radius.min: min radius (in pixels)
  }
  
  The function \code{computeFeatures.moment} computes features
  related to object image moments, which can be computed with or without
  reference intensities:
  \itemize{
    \item m.cx: center of mass x (in pixels)
    \item m.cy: center of mass y (in pixels)
    \item m.majoraxis: elliptical fit major axis (in pixels)
    \item m.eccentricity: elliptical eccentricity defined by
    sqrt(1-minoraxis^2/majoraxis^2). Circle eccentricity is 0 and straight
    line eccentricity is 1.
    \item m.theta: object angle (in radians)
  }

  The function \code{computeFeatures.haralick} computes features
  that quantify pixel texture. Features are named according to
  Haralick's original paper.
}


\references{
  R. M. Haralick, K Shanmugam and Its'Hak Deinstein (1979). \emph{Textural Features for Image 
  Classification}. IEEE Transactions on Systems, Man and Cybernetics.
}

\seealso{
  \code{\link{bwlabel}}, \code{\link{propagate}}
}

\author{
  Gregoire Pau, \email{gregoire.pau@embl.de}, 2011
}

\examples{
  ## load and segment nucleus
  y = readImage(system.file("images", "nuclei.tif", package="EBImage"))[,,1]
  x = thresh(y, 10, 10, 0.05)
  x = opening(x, makeBrush(5, shape='disc'))
  x = bwlabel(x)
  display(y, title="Cell nuclei")
  display(x, title="Segmented nuclei")

  ## compute shape features
  fts = computeFeatures.shape(x)
  fts

  ## compute features
  ft = computeFeatures(x, y, xname="nucleus")
  cat("median features are:\n")
  apply(ft, 2, median)

  ## compute feature properties
  ftp = computeFeatures(x, y, properties=TRUE, xname="nucleus")
  ftp
}

\name{colorLabels}

\alias{colorLabels}

\title{Color Code Labels}

\description{
  Color codes the labels of object masks by a random permutation.
}

\usage{
colorLabels(x, normalize = TRUE)
}

\arguments{
  \item{x}{an \code{Image} object in \code{Grayscale} color mode or an array containing object masks. Object masks are sets of pixels with the same unique integer value}
  \item{normalize}{if TRUE normalizes the resulting color image}
}

\value{
  An \code{Image} object containing color coded objects of \code{x}.
}

\details{
Performs color coding of object masks, which are typically obtained using the \code{bwlabel} function. Each label from \code{x} is assigned an unique color. The colors are distributed among the labels using a random permutation. If \code{normalize} is set to \code{TRUE} the intensity values of the resulting image are mapped to the [0,1] range.
}

\author{
  Bernd Fischer, Andrzej Oles, 2013-2014
}

\seealso{
  \code{\link{bwlabel}},  \code{\link{normalize}}
}

\examples{
  x = readImage(system.file('images', 'shapes.png', package='EBImage'))
  x = x[110:512,1:130]
  y = bwlabel(x)
  z = colorLabels(y)
  display(z, title='Colored segmentation')
}

\name{watershed}

\alias{watershed}

\concept{watershed transformation}
\concept{watershed segmentation}

\title{Watershed transformation and watershed based object detection}

\description{
  Watershed transformation and watershed based object detection.
}

\usage{
watershed(x, tolerance=1, ext=1)
}

\arguments{

  \item{x}{An \code{Image} object or an array.}

  \item{tolerance}{ The minimum height of the object in the units of image
    intensity between its highest point (seed) and the point where it
    contacts another object (checked for every contact pixel). If the
    height is smaller than the tolerance, the object will be combined with
    one of its neighbors, which is the highest. Tolerance should be chosen
    according to the range of \code{x}. Default value is 1, which 
    is a reasonable value if \code{x} comes from \code{distmap}.}

  \item{ext}{Radius of the neighborhood in pixels for the detection
    of neighboring objects. Higher value smoothes out small objects. }
}

\value{
  An \code{Grayscale} \code{Image} object or an array, containing the
  labelled version of \code{x}.
}

\details{
  The algorithm identifies and separates objects that stand out of the
  background (zero). It inverts the image and uses water to fill
  the resulting valleys (pixels with high intensity in the source image)
  until another object or background is met.
  The deepest valleys become indexed first, starting from 1.

  The function \code{bwlabel} is a simpler, faster alternative to 
  segment connected objects from binary images.
}

\seealso{ 
  \code{\link{bwlabel}},  \code{\link{propagate}} 
}

\examples{
  x = readImage(system.file('images', 'shapes.png', package='EBImage'))
  x = x[110:512,1:130]
  display(x, title='Binary')
  y = distmap(x)
  display(normalize(y), title='Distance map')
  w = watershed(y)
  display(normalize(w), title='Watershed')
}

\author{
  Oleg Sklyar, \email{osklyar@ebi.ac.uk}, 2007
}

\keyword{manip}

\name{tile}

\alias{tile}
\alias{untile}

\title{Tiling/untiling images}

\description{
  Given a sequence of frames, \code{tile} generates a single image with frames tiled.
  \code{untile} is the inverse function and divides an image into a sequence of images.
}

\usage{
tile(x, nx=10, lwd=1, fg.col="#E4AF2B", bg.col="gray")
untile(x, nim, lwd=1)
}

\arguments{
  \item{x}{An \code{Image} object, an array or a list of these objects.} 

  \item{nx}{ The number of tiled images in a row. }

  \item{lwd}{ The width of the grid lines between tiled images, can be 0. }

  \item{fg.col}{The color of the grid lines.}

  \item{bg.col}{The color of the background for extra tiles.}

  \item{nim}{A numeric vector of 2 elements for the number of images in both directions.}
}

\value{
 An \code{Image} object or an array, containing the tiled/untiled version
 of \code{x}.
}

\details{
  After object segmentation, \code{tile} is a useful addition to \code{stackObjects}
  to have an overview of the segmented objects.
}

\seealso{ \code{\link{stackObjects}}
}

\examples{
  ## make a set of blurred images
  img = readImage(system.file("images", "sample-color.png", package="EBImage"))[257:768,,]
  x = resize(img, 128, 128)
  xt = list()
  for (t in seq(0.1, 5, length.out=9)) xt=c(xt, list(gblur(x, s=t)))
  xt = combine(xt)
  display(xt, title='Blurred images')

  ## tile
  xt = tile(xt, 3)
  display(xt, title='Tiles')

  ## untile
  xu = untile(img, c(3, 3))
  display(xu, title='Blocks')
}

\author{
  Oleg Sklyar, \email{osklyar@ebi.ac.uk}, 2006-2007
}

\keyword{manip}

\name{EBImage}

\alias{EBImage}

\docType{package}

\title{Package overview}

\description{
  \code{EBImage} is an image processing and analysis package for R. Its primary
  goal is to enable automated analysis of large sets of images such as those
  obtained in high throughput automated microscopy.

  \code{EBImage} relies on the \code{Image} object to store and process images
  but also works on multi-dimensional arrays.
}

\section{Package content}{

  Image methods
  \itemize{
    \item \code{Image}
    \item \code{as.Image}, \code{is.Image}, \code{as.raster}
    \item \code{colorMode}, \code{imageData}
    \item \code{getFrame}, \code{numberOfFrames}
  }

  Image I/O, display
  \itemize{
    \item \code{readImage}, \code{writeImage}
    \item \code{display}
    \item \code{image}
  }

  Spatial transforms
  \itemize{
    \item \code{resize}, \code{flip}, \code{flop}, \code{transpose}
    \item \code{rotate}, \code{translate}, \code{affine}
  }

  Image segmentation, objects manipulation
  \itemize{
    \item \code{thresh}, \code{bwlabel}, \code{otsu}
    \item \code{watershed}, \code{propagate}
    \item \code{ocontour}
    \item \code{paintObjects}, \code{rmObjects}, \code{reenumerate}
  }

  Image enhancement, filtering
  \itemize{
    \item \code{normalize}
    \item \code{filter2}, \code{gblur}, \code{medianFilter}
  }

  Morphological operations
  \itemize{
    \item \code{makeBrush}
    \item \code{erode}, \code{dilate}, \code{opening}, \code{closing}
    \item \code{whiteTopHat}, \code{blackTopHat}, \code{selfComplementaryTopHat}
    \item \code{distmap}
    \item \code{floodFill}, \code{fillHull}
  }

  Color space manipulation
  \itemize{  
    \item \code{rgbImage}, \code{channel}, \code{toRGB}
  }

  Image stacking, combining, tiling
  \itemize{ 
    \item \code{stackObjects}
    \item \code{combine}
    \item \code{tile}, \code{untile}
  }

  Drawing on images
  \itemize{
    \item \code{drawCircle}
  }

  Features extraction
  \itemize{
    \item \code{computeFeatures}
    \item \code{computeFeatures.basic}, \code{computeFeatures.moment}, \code{computeFeatures.shape}, \code{computeFeatures.haralick}
    \item \code{standardExpandRef}
  }

  Defunct
  \itemize{
    \item \code{blur}, \code{equalize}
    \item \code{drawtext}, \code{drawfont}
    \item \code{getFeatures}, \code{hullFeatures}, \code{zernikeMoments}
    \item \code{edgeProfile}, \code{edgeFeatures}, 
    \item \code{haralickFeatures}, \code{haralickMatrix}
    \item \code{moments}, \code{cmoments}, \code{smoments}, \code{rmoments}
  }
}

\section{Authors}{

  Oleg Sklyar, \email{osklyar@ebi.ac.uk}, Copyright 2005-2007

  Gregoire Pau, \email{gpau@ebi.ac.uk}

  Wolfgang Huber, \email{huber@ebi.ac.uk}

  Andrzej Oles, \email{andrzej.oles@embl.de}

  Mike Smith, \email{msmith@ebi.ac.uk}

  \preformatted{
  European Bioinformatics Institute
  European Molecular Biology Laboratory
  Wellcome Trust Genome Campus
  Hinxton 
  Cambridge CB10 1SD
  UK
  }

  The code of \code{\link{propagate}} is based on the \code{CellProfiler} 
  with permission granted to distribute this particular part under LGPL, the 
  corresponding copyright (Jones, Carpenter) applies.

  The source code is released under \code{LGPL} (see the \code{LICENSE} 
  file in the package root for the complete license wording).

  This library is free software; you can redistribute it and/or modify it
  under the terms of the GNU Lesser General Public License as published by the Free Software Foundation; either version 2.1 of the
  License, or (at your option) any later version. This library is distributed
  in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even
  the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.

  See the GNU Lesser General Public License for more details. For LGPL license 
  wording see \url{http://www.gnu.org/licenses/lgpl.html}
}

\examples{
  example(readImage)
  example(display)
  example(rotate)
  example(propagate)
}

\keyword{package}

\name{morphology}

\alias{dilate}
\alias{closing}
\alias{erode}
\alias{opening}
\alias{whiteTopHat}
\alias{blackTopHat}
\alias{selfComplementaryTopHat}
\alias{makeBrush}

\title{Perform morphological operations on images}

\description{
  Functions to perform morphological operations on binary and grayscale images.
}

\usage{
dilate(x, kern)
erode(x, kern)
opening(x, kern)
closing(x, kern)
whiteTopHat(x, kern)
blackTopHat(x, kern)
selfComplementaryTopHat(x, kern)

makeBrush(size, shape=c('box', 'disc', 'diamond', 'Gaussian', 'line'), step=TRUE, sigma=0.3, angle=45)
}

\arguments{
  \item{x}{An \code{Image} object or an array.}
  
  \item{kern}{An \code{Image} object or an array, containing the
    structuring element. \code{kern} is considered as a binary image, with
    pixels of value 0 being the background and pixels with values other than 0 being the foreground.}
  
  \item{size}{A numeric containing the size of the brush in pixels. This should be an odd number; even numbers are rounded to the next odd one, i.e., \code{size = 4} has the same effect as \code{size = 5}. Default is 5}
  
  \item{shape}{A character vector indicating the shape of the brush. Can
    be \code{box}, \code{disc}, \code{diamond}, \code{Gaussian} or \code{line}. Default is
    \code{box}.}
  
  \item{step}{a logical indicating if the brush is binary. Default is
    \code{TRUE}. This argument is relevant only for the \code{disc} and
    \code{diamond} shapes.}

  \item{sigma}{An optional numeric containing the standard deviation of
    the \code{Gaussian} shape. Default is 0.3.}

  \item{angle}{An optional numeric containing the angle at which the line should be drawn. The angle is
    one between the top of the image and the line.}
}

\details{
  \code{dilate} applies the mask \code{kern} by positioning its center over every pixel of the
  image \code{x}, the output value of the pixel is the maximum value of \code{x}
  covered by the mask. In case of binary images this is equivalent of putting the mask over every background pixel, and setting it to foreground if any of the pixels covered by the mask is from the foreground.

  \code{erode} applies the mask \code{kern} by positioning its center over every pixel of the
  image \code{x}, the output value of the pixel is the minimum value of \code{x}
  covered by the mask. In case of binary images this is equivalent of putting the mask over every foreground pixel, and setting it to background if any of the pixels covered by the mask is from the background.

  \code{opening} is an erosion followed by a dilation and \code{closing} is a dilation followed by an erosion.

  \code{whiteTopHat} returns the difference between the original image \code{x} and its opening by the structuring element \code{kern}.
  
  \code{blackTopHat} subtracts the original image \code{x} from its closing by the structuring element \code{kern}.

  \code{selfComplementaryTopHat} is the sum of the \code{whiteTopHat} and the \code{blackTopHat}, simplified
  the difference between the \code{closing} and the \code{opening} of the image.

  \code{makeBrush} generates brushes of various sizes and shapes that can be used
  as structuring elements.

  \subsection{Processing Pixels at Image Borders (Padding Behavior)}{
    Morphological functions position the center of the structuring element over each pixel in the input image. For pixels close to the edge of an image, parts of the neighborhood defined by the structuring element may extend past the border of the image. In such a case, a value is assigned to these undefined pixels, as if the image was padded with additional rows and columns. The value of these padding pixels varies for dilation and erosion operations. For dilation, pixels beyond the image border are assigned the minimum value afforded by the data type, which in case of binary images is equivalent of setting them to background. For erosion, pixels beyond the image border are assigned the maximum value afforded by the data type, which in case of binary images is equivalent of setting them to foreground.}
}

\value{
  \code{dilate}, \code{erode}, \code{opening}, \code{whiteTopHat}, \code{blackTopHat} and
  \code{selfComplementaryTopHat} return the transformed \code{Image} object
  or array \code{x}, after the corresponding morphological operation.
    
  \code{makeBrush} generates a 2D matrix containing the desired brush.
}

\note{
  Morphological operations are implemented using the efficient Urbach-Wilkinson algorithm [1]. Its required computing time is independent of both the image content and the number of gray levels used.
}

\author{
  Ilia Kats <\email{ilia-kats@gmx.net}> (2012), Andrzej Oles <\email{andrzej.oles@embl.de}> (2015)
}

\examples{	
  x = readImage(system.file("images", "shapes.png", package="EBImage"))
  kern = makeBrush(5, shape='diamond')  
  
  display(x)
  display(kern, title='Structuring element')
  display(erode(x, kern), title='Erosion of x')
  display(dilate(x, kern), title='Dilatation of x')

  ## makeBrush
  display(makeBrush(99, shape='diamond'))
  display(makeBrush(99, shape='disc', step=FALSE))
  display(2000*makeBrush(99, shape='Gaussian', sigma=10))
}

\references{
  [1] E. R. Urbach and M.H.F. Wilkinson, "Efficient 2-D grayscale morphological transformations with arbitrary flat structuring elements", IEEE Trans Image Process 17(1), 1-8, 2008
}
